@@ -27,7 +27,7 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 #
-# Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+# Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 import os
 import sys
@@ -39,7 +39,7 @@ import argparse
 import zipfile
 import hashlib
 import ssl
-import urllib.request
+import requests
 import subprocess
 import multiprocessing
 
@@ -138,7 +138,10 @@ def parse_args():
         args.cmake_config_args.extend(["-G", args.cmake_generator])
     if PLATFORM_IS_WINDOWS:
         args.cmake_config_args.append(
-            "-DCMAKE_GENERATOR_PLATFORM=x64")
+            "-DCMAKE_GENERATOR_TOOLSET='host=x64'")
+        if "Win64" not in args.cmake_generator:
+            args.cmake_config_args.append(
+                "-DCMAKE_GENERATOR_PLATFORM=x64")
 
     args.cmake_build_args = ["--"]
     if PLATFORM_IS_WINDOWS:
@@ -182,7 +185,9 @@ def check_md5_hash(path, md5_hash):
 
 def download_zipfile(url, archive_path, unzip_path, md5_hash):
     if not os.path.exists(archive_path):
-        urllib.request.urlretrieve(url, archive_path)
+        r = requests.get(url)
+        with open(archive_path, 'wb') as outfile:
+            outfile.write(r.content)
     check_md5_hash(archive_path, md5_hash)
     with zipfile.ZipFile(archive_path, "r") as fid:
         fid.extractall(unzip_path)
@@ -218,10 +223,10 @@ def build_eigen(args):
     if os.path.exists(path):
         return
 
-    url = "https://bitbucket.org/eigen/eigen/get/3.3.4.zip"
-    archive_path = os.path.join(args.download_path, "eigen-3.3.4.zip")
+    url = "http://gitlab.com/libeigen/eigen/-/archive/3.3.7/eigen-3.3.7.zip"
+    archive_path = os.path.join(args.download_path, "eigen-3.3.7.zip")
     download_zipfile(url, archive_path, args.build_path,
-                     "e337acc279874bc6a56da4d973a723fb")
+                     "888aab45512cc0c734b3e8f60280daba")
     shutil.move(glob.glob(os.path.join(args.build_path, "eigen-*"))[0], path)
 
     build_cmake_project(args, os.path.join(path, "__build__"))
@@ -234,10 +239,10 @@ def build_freeimage(args):
 
     if PLATFORM_IS_WINDOWS:
         url = "https://kent.dl.sourceforge.net/project/freeimage/" \
-              "Binary%20Distribution/3.17.0/FreeImage3170Win32Win64.zip"
-        archive_path = os.path.join(args.download_path, "freeimage-3.17.0.zip")
+              "Binary%20Distribution/3.18.0/FreeImage3180Win32Win64.zip"
+        archive_path = os.path.join(args.download_path, "freeimage-3.18.0.zip")
         download_zipfile(url, archive_path, args.build_path,
-                         "a7e6f2f261e72260ec5b91c2a0f4bde3")
+                         "393d3df75b14cbcb4887da1c395596e2")
         shutil.move(os.path.join(args.build_path, "FreeImage"), path)
         copy_file_if_not_exists(
             os.path.join(path, "Dist/x64/FreeImage.h"),
@@ -250,10 +255,10 @@ def build_freeimage(args):
             os.path.join(args.install_path, "lib/FreeImage.dll"))
     else:
         url = "https://kent.dl.sourceforge.net/project/freeimage/" \
-              "Source%20Distribution/3.17.0/FreeImage3170.zip"
-        archive_path = os.path.join(args.download_path, "freeimage-3.17.0.zip")
+              "Source%20Distribution/3.18.0/FreeImage3180.zip"
+        archive_path = os.path.join(args.download_path, "freeimage-3.18.0.zip")
         download_zipfile(url, archive_path, args.build_path,
-                         "459e15f0ec75d6efa3c7bd63277ead86")
+                         "f8ba138a3be233a3eed9c456e42e2578")
         shutil.move(os.path.join(args.build_path, "FreeImage"), path)
 
         if PLATFORM_IS_MAC:
@@ -268,7 +273,7 @@ def build_freeimage(args):
         elif PLATFORM_IS_LINUX:
             with fileinput.FileInput(
                     os.path.join(path, "Source/LibWebP/src/dsp/"
-                                 "dsp.upsampling_mips_dsp_r2.c"),
+                                 "upsampling_mips_dsp_r2.c"),
                     inplace=True, backup=".bak") as fid:
                 for i, line in enumerate(fid):
                     if i >= 36 and i <= 44:
@@ -277,7 +282,7 @@ def build_freeimage(args):
                     print(line, end="")
             with fileinput.FileInput(
                     os.path.join(path, "Source/LibWebP/src/dsp/"
-                                 "dsp.yuv_mips_dsp_r2.c"),
+                                 "yuv_mips_dsp_r2.c"),
                     inplace=True, backup=".bak") as fid:
                 for i, line in enumerate(fid):
                     if i >= 56 and i <= 58:
@@ -322,11 +327,11 @@ def build_gflags(args):
     if os.path.exists(path):
         return
 
-    url = "https://github.com/gflags/gflags/archive/v2.2.1.zip"
-    archive_path = os.path.join(args.download_path, "gflags-2.2.1.zip")
+    url = "https://github.com/gflags/gflags/archive/v2.2.2.zip"
+    archive_path = os.path.join(args.download_path, "gflags-2.2.2.zip")
     download_zipfile(url, archive_path, args.build_path,
-                     "2d988ef0b50939fb50ada965dafce96b")
-    shutil.move(os.path.join(args.build_path, "gflags-2.2.1"), path)
+                     "ff856ff64757f1381f7da260f79ba79b")
+    shutil.move(os.path.join(args.build_path, "gflags-2.2.2"), path)
     os.remove(os.path.join(path, "BUILD"))
 
     build_cmake_project(args, os.path.join(path, "__build__"))
@@ -367,11 +372,14 @@ def build_suite_sparse(args):
     build_cmake_project(args, os.path.join(path, "__build__"))
 
     if PLATFORM_IS_WINDOWS:
-        lapack_blas_path = os.path.join(args.install_path,
-                                        "lib64/lapack_blas_windows/*.dll")
+        lapack_blas_path = os.path.join(path, "lapack_windows/x64/*")
+        mkdir_if_not_exists(os.path.join(args.install_path, "lib64"))
+        mkdir_if_not_exists(os.path.join(args.install_path,
+                                         "lib64/lapack_blas_windows"))
         for library_path in glob.glob(lapack_blas_path):
             copy_file_if_not_exists(
-                library_path, os.path.join(args.install_path, "lib",
+                library_path, os.path.join(args.install_path,
+                                           "lib64/lapack_blas_windows",
                                            os.path.basename(library_path)))
 
 
@@ -462,6 +470,16 @@ def build_colmap(args):
 
 def build_post_process(args):
     if PLATFORM_IS_WINDOWS:
+        lapack_paths = glob.glob(
+            os.path.join(args.install_path, "lib64/lapack_blas_windows/*.dll"))
+        if lapack_paths:
+            for lapack_path in lapack_paths:
+                copy_file_if_not_exists(
+                    lapack_path,
+                    os.path.join(
+                        args.install_path, "lib",
+                        os.path.basename(lapack_path)))
+
         if args.qt_path:
             copy_file_if_not_exists(
                 os.path.join(args.qt_path, "bin/Qt5Core.dll"),
@@ -477,6 +495,7 @@ def build_post_process(args):
             copy_file_if_not_exists(
                 os.path.join(args.qt_path, "plugins/platforms/qwindows.dll"),
                 os.path.join(args.install_path, "lib/platforms/qwindows.dll"))
+
         if args.with_cuda and args.cuda_path:
             cudart_lib_path = glob.glob(os.path.join(args.cuda_path,
                                                      "bin/cudart64_*.dll"))[0]
@@ -484,17 +503,20 @@ def build_post_process(args):
                 cudart_lib_path,
                 os.path.join(args.install_path, "lib",
                              os.path.basename(cudart_lib_path)))
+
         if args.cgal_path:
             gmp_lib_path = os.path.join(
-                args.cgal_path, "../auxiliary/gmp/lib/libgmp-10.dll")
+                args.cgal_path, "auxiliary/gmp/lib/libgmp-10.dll")
             if os.path.exists(gmp_lib_path):
                 copy_file_if_not_exists(
                     gmp_lib_path,
                     os.path.join(args.install_path, "lib/libgmp-10.dll"))
+            cgal_lib_path = glob.glob(os.path.join(
+                args.cgal_path, "bin/CGAL-vc*-mt-*.dll"))
             copy_file_if_not_exists(
-                os.path.join(args.cgal_path,
-                             "bin/Release/CGAL-vc140-mt-4.11.2.dll"),
-                os.path.join(args.install_path, "lib/CGAL-vc140-mt-4.11.2.dll"))
+                cgal_lib_path[0],
+                os.path.join(args.install_path, "lib",
+                    os.path.basename(cgal_lib_path[0])))
 
 
 def main():
