@@ -27,7 +27,7 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 #
-# Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+# Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 import re
 import argparse
@@ -118,8 +118,8 @@ def main():
                                 float(sensor_width.replace(" ", "")))
                         models_code += "    make_specs.emplace_back(\"%s\", %.4ff);\n" % data
 
-                        print make, model_name
-                        print "   ", sensor_text
+                        print(make, model_name)
+                        print("   ", sensor_text)
 
                         num_models += 1
 
