@@ -27,7 +27,7 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 #
-# Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+# Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 # This script exports a COLMAP database to the file structure to run Bundler.
 
@@ -80,7 +80,7 @@ def main():
             image_id = row[0]
             camera_id = row[1]
             image_name = row[2]
-            print "Copying image", image_name
+            print("Copying image", image_name)
             images[image_id] = (len(images), image_name)
             fid.write("./%s 0 %f\n" % (image_name, cameras[camera_id][0]))
             if not os.path.exists(os.path.join(args.output_path, image_name)):
@@ -88,7 +88,7 @@ def main():
                                 os.path.join(args.output_path, image_name))
 
     for image_id, (image_idx, image_name) in images.iteritems():
-        print "Exporting key file for", image_name
+        print("Exporting key file for", image_name)
         base_name, ext = os.path.splitext(image_name)
         key_file_name = os.path.join(args.output_path, base_name + ".key")
         key_file_name_gz = key_file_name + ".gz"
@@ -98,11 +98,15 @@ def main():
         cursor.execute("SELECT data FROM keypoints WHERE image_id=?;",
                        (image_id,))
         row = next(cursor)
-        keypoints = np.fromstring(row[0], dtype=np.float32).reshape(-1, 4)
-        cursor.execute("SELECT data FROM descriptors WHERE image_id=?;",
-                       (image_id,))
-        row = next(cursor)
-        descriptors = np.fromstring(row[0], dtype=np.uint8).reshape(-1, 128)
+        if row[0] is None:
+            keypoints = np.zeros((0, 6), dtype=np.float32)
+            descriptors = np.zeros((0, 128), dtype=np.uint8)
+        else:
+            keypoints = np.fromstring(row[0], dtype=np.float32).reshape(-1, 6)
+            cursor.execute("SELECT data FROM descriptors WHERE image_id=?;",
+                        (image_id,))
+            row = next(cursor)
+            descriptors = np.fromstring(row[0], dtype=np.uint8).reshape(-1, 128)
 
         with open(key_file_name, "w") as fid:
             fid.write("%d %d\n" % (keypoints.shape[0], descriptors.shape[1]))
