@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_BASE_CAMERA_H_
 #define COLMAP_SRC_BASE_CAMERA_H_
@@ -114,6 +114,9 @@ class Camera {
   // the correct dimensions that match the specified camera model.
   bool VerifyParams() const;
 
+  // Check whether camera is already undistorted
+  bool IsUndistorted() const;
+
   // Check whether camera has bogus parameters.
   bool HasBogusParams(const double min_focal_length_ratio,
                       const double max_focal_length_ratio,
