@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "base/camera_database.h"
 
@@ -45,10 +45,10 @@ bool CameraDatabase::QuerySensorWidth(const std::string& make,
   // Clean the strings from all separators.
   std::string cleaned_make = make;
   std::string cleaned_model = model;
-  StringReplace(cleaned_make, " ", "");
-  StringReplace(cleaned_model, " ", "");
-  StringReplace(cleaned_make, "-", "");
-  StringReplace(cleaned_model, "-", "");
+  cleaned_make = StringReplace(cleaned_make, " ", "");
+  cleaned_model = StringReplace(cleaned_model, " ", "");
+  cleaned_make = StringReplace(cleaned_make, "-", "");
+  cleaned_model = StringReplace(cleaned_model, "-", "");
   StringToLower(&cleaned_make);
   StringToLower(&cleaned_model);
 
