@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_BASE_CAMERA_MODELS_H_
 #define COLMAP_SRC_BASE_CAMERA_MODELS_H_
@@ -1356,8 +1356,7 @@ void RadialFisheyeCameraModel::Distortion(const T* extra_params, const T u,
     const T theta = ceres::atan(r);
     const T theta2 = theta * theta;
     const T theta4 = theta2 * theta2;
-    const T thetad =
-        theta * (T(1) + k1 * theta2 + k2 * theta4);
+    const T thetad = theta * (T(1) + k1 * theta2 + k2 * theta4);
     *du = u * thetad / r - u;
     *dv = v * thetad / r - v;
   } else {
