@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_BASE_CAMERA_RIG_H_
 #define COLMAP_SRC_BASE_CAMERA_RIG_H_
@@ -101,7 +101,7 @@ class CameraRig {
   // the relative poses over all snapshots. The pose of the reference camera
   // will be the identity transformation. This assumes that the camera rig has
   // snapshots that are registered in the reconstruction.
-  void ComputeRelativePoses(const Reconstruction& reconstruction);
+  bool ComputeRelativePoses(const Reconstruction& reconstruction);
 
   // Compute the absolute camera pose of the rig. The absolute camera pose of
   // the rig is computed as the average of all relative camera poses in the rig
