@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #define TEST_NAME "base/camera"
 #include "util/testing.h"
@@ -183,10 +183,13 @@ BOOST_AUTO_TEST_CASE(TestParamsToString) {
 BOOST_AUTO_TEST_CASE(TestParamsFromString) {
   Camera camera;
   camera.SetModelId(SimplePinholeCameraModel::model_id);
-  camera.SetParamsFromString("1.000000, 0.500000, 0.500000");
+  BOOST_CHECK(camera.SetParamsFromString("1.000000, 0.500000, 0.500000"));
   const std::vector<double> params{1.0, 0.5, 0.5};
   BOOST_CHECK_EQUAL_COLLECTIONS(camera.Params().begin(), camera.Params().end(),
                                 params.begin(), params.end());
+  BOOST_CHECK(!camera.SetParamsFromString("1.000000, 0.500000"));
+  BOOST_CHECK_EQUAL_COLLECTIONS(camera.Params().begin(), camera.Params().end(),
+                                params.begin(), params.end());
 }
 
 BOOST_AUTO_TEST_CASE(TestVerifyParams) {
@@ -198,6 +201,28 @@ BOOST_AUTO_TEST_CASE(TestVerifyParams) {
   BOOST_CHECK_EQUAL(camera.VerifyParams(), false);
 }
 
+BOOST_AUTO_TEST_CASE(TestIsUndistorted) { 
+  Camera camera;
+  camera.InitializeWithId(SimplePinholeCameraModel::model_id, 1.0, 1, 1);
+  BOOST_CHECK(camera.IsUndistorted());
+  camera.InitializeWithId(SimpleRadialCameraModel::model_id, 1.0, 1, 1);
+  BOOST_CHECK(camera.IsUndistorted());
+  camera.SetParams({1.0, 0.5, 0.5, 0.005});
+  BOOST_CHECK(!camera.IsUndistorted());
+  camera.InitializeWithId(RadialCameraModel::model_id, 1.0, 1, 1);
+  BOOST_CHECK(camera.IsUndistorted());
+  camera.SetParams({1.0, 0.5, 0.5, 0.0, 0.005});
+  BOOST_CHECK(!camera.IsUndistorted());
+  camera.InitializeWithId(OpenCVCameraModel::model_id, 1.0, 1, 1);
+  BOOST_CHECK(camera.IsUndistorted());
+  camera.SetParams({1.0, 1.0, 0.5, 0.5, 0.0, 0.0, 0.0, 0.001});
+  BOOST_CHECK(!camera.IsUndistorted());
+  camera.InitializeWithId(FullOpenCVCameraModel::model_id, 1.0, 1, 1);
+  BOOST_CHECK(camera.IsUndistorted());
+  camera.SetParams({1.0, 1.0, 0.5, 0.5, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.001});
+  BOOST_CHECK(!camera.IsUndistorted());
+}
+
 BOOST_AUTO_TEST_CASE(TestHasBogusParams) {
   Camera camera;
   BOOST_CHECK_THROW(camera.HasBogusParams(0.0, 0.0, 0.0), std::domain_error);
