@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #define TEST_NAME "base/cost_functions"
 #include "util/testing.h"
@@ -39,9 +39,9 @@
 using namespace colmap;
 
 BOOST_AUTO_TEST_CASE(TestBundleAdjustmentCostFunction) {
-  ceres::CostFunction* cost_function =
+  std::unique_ptr<ceres::CostFunction> cost_function(
       BundleAdjustmentCostFunction<SimplePinholeCameraModel>::Create(
-          Eigen::Vector2d::Zero());
+          Eigen::Vector2d::Zero()));
   double qvec[4] = {1, 0, 0, 0};
   double tvec[3] = {0, 0, 0};
   double point3D[3] = {0, 0, 1};
@@ -69,10 +69,11 @@ BOOST_AUTO_TEST_CASE(TestBundleAdjustmentCostFunction) {
 }
 
 BOOST_AUTO_TEST_CASE(TestBundleAdjustmentConstantPoseCostFunction) {
-  ceres::CostFunction* cost_function = BundleAdjustmentConstantPoseCostFunction<
-      SimplePinholeCameraModel>::Create(ComposeIdentityQuaternion(),
-                                        Eigen::Vector3d::Zero(),
-                                        Eigen::Vector2d::Zero());
+  std::unique_ptr<ceres::CostFunction> cost_function(
+      BundleAdjustmentConstantPoseCostFunction<
+          SimplePinholeCameraModel>::Create(ComposeIdentityQuaternion(),
+                                            Eigen::Vector3d::Zero(),
+                                            Eigen::Vector2d::Zero()));
   double point3D[3] = {0, 0, 1};
   double camera_params[3] = {1, 0, 0};
   double residuals[2];
@@ -98,9 +99,9 @@ BOOST_AUTO_TEST_CASE(TestBundleAdjustmentConstantPoseCostFunction) {
 }
 
 BOOST_AUTO_TEST_CASE(TestRigBundleAdjustmentCostFunction) {
-  ceres::CostFunction* cost_function =
+  std::unique_ptr<ceres::CostFunction> cost_function(
       RigBundleAdjustmentCostFunction<SimplePinholeCameraModel>::Create(
-          Eigen::Vector2d::Zero());
+          Eigen::Vector2d::Zero()));
   double rig_qvec[4] = {1, 0, 0, 0};
   double rig_tvec[3] = {0, 0, -1};
   double rel_qvec[4] = {1, 0, 0, 0};
@@ -131,8 +132,9 @@ BOOST_AUTO_TEST_CASE(TestRigBundleAdjustmentCostFunction) {
 }
 
 BOOST_AUTO_TEST_CASE(TestRelativePoseCostFunction) {
-  ceres::CostFunction* cost_function = RelativePoseCostFunction::Create(
-      Eigen::Vector2d(0, 0), Eigen::Vector2d(0, 0));
+  std::unique_ptr<ceres::CostFunction> cost_function(
+      RelativePoseCostFunction::Create(Eigen::Vector2d(0, 0),
+                                       Eigen::Vector2d(0, 0)));
   double qvec[4] = {1, 0, 0, 0};
   double tvec[3] = {0, 1, 0};
   double residuals[1];
@@ -140,13 +142,13 @@ BOOST_AUTO_TEST_CASE(TestRelativePoseCostFunction) {
   BOOST_CHECK(cost_function->Evaluate(parameters, residuals, nullptr));
   BOOST_CHECK_EQUAL(residuals[0], 0);
 
-  cost_function = RelativePoseCostFunction::Create(Eigen::Vector2d(0, 0),
-                                                   Eigen::Vector2d(1, 0));
+  cost_function.reset(RelativePoseCostFunction::Create(Eigen::Vector2d(0, 0),
+                                                       Eigen::Vector2d(1, 0)));
   BOOST_CHECK(cost_function->Evaluate(parameters, residuals, nullptr));
   BOOST_CHECK_EQUAL(residuals[0], 0.5);
 
-  cost_function = RelativePoseCostFunction::Create(Eigen::Vector2d(0, 0),
-                                                   Eigen::Vector2d(1, 1));
+  cost_function.reset(RelativePoseCostFunction::Create(Eigen::Vector2d(0, 0),
+                                                       Eigen::Vector2d(1, 1)));
   BOOST_CHECK(cost_function->Evaluate(parameters, residuals, nullptr));
   BOOST_CHECK_EQUAL(residuals[0], 0.5);
 }
