@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "base/database.h"
 
@@ -275,6 +275,9 @@ void Database::Open(const std::string& path) {
   // Disabled by default
   SQLITE3_EXEC(database_, "PRAGMA foreign_keys=ON", nullptr);
 
+  // Enable auto vacuum to reduce DB file size
+  SQLITE3_EXEC(database_, "PRAGMA auto_vacuum=1", nullptr);
+
   CreateTables();
   UpdateSchema();
   PrepareSQLStatements();
@@ -283,6 +286,7 @@ void Database::Open(const std::string& path) {
 void Database::Close() {
   if (database_ != nullptr) {
     FinalizeSQLStatements();
+    SQLITE3_EXEC(database_, "VACUUM", nullptr);
     sqlite3_close_v2(database_);
     database_ = nullptr;
   }
@@ -513,6 +517,10 @@ TwoViewGeometry Database::ReadTwoViewGeometry(const image_t image_id1,
       sql_stmt_read_two_view_geometry_, rc, 5);
   two_view_geometry.H = ReadStaticMatrixBlob<Eigen::Matrix3d>(
       sql_stmt_read_two_view_geometry_, rc, 6);
+  two_view_geometry.qvec = ReadStaticMatrixBlob<Eigen::Vector4d>(
+      sql_stmt_read_two_view_geometry_, rc, 7);
+  two_view_geometry.tvec = ReadStaticMatrixBlob<Eigen::Vector3d>(
+      sql_stmt_read_two_view_geometry_, rc, 8);
 
   SQLITE3_CALL(sqlite3_reset(sql_stmt_read_two_view_geometry_));
 
@@ -553,6 +561,10 @@ void Database::ReadTwoViewGeometries(
         sql_stmt_read_two_view_geometries_, rc, 6);
     two_view_geometry.H = ReadStaticMatrixBlob<Eigen::Matrix3d>(
         sql_stmt_read_two_view_geometries_, rc, 7);
+    two_view_geometry.qvec = ReadStaticMatrixBlob<Eigen::Vector4d>(
+        sql_stmt_read_two_view_geometries_, rc, 8);
+    two_view_geometry.tvec = ReadStaticMatrixBlob<Eigen::Vector3d>(
+        sql_stmt_read_two_view_geometries_, rc, 9);
 
     two_view_geometry.F.transposeInPlace();
     two_view_geometry.E.transposeInPlace();
@@ -719,11 +731,15 @@ void Database::WriteTwoViewGeometry(
   const Eigen::Matrix3d Ft = two_view_geometry_ptr->F.transpose();
   const Eigen::Matrix3d Et = two_view_geometry_ptr->E.transpose();
   const Eigen::Matrix3d Ht = two_view_geometry_ptr->H.transpose();
+  const Eigen::Vector4d& qvec = two_view_geometry_ptr->qvec;
+  const Eigen::Vector3d& tvec = two_view_geometry_ptr->tvec;
 
   if (two_view_geometry_ptr->inlier_matches.size() > 0) {
     WriteStaticMatrixBlob(sql_stmt_write_two_view_geometry_, Ft, 6);
     WriteStaticMatrixBlob(sql_stmt_write_two_view_geometry_, Et, 7);
     WriteStaticMatrixBlob(sql_stmt_write_two_view_geometry_, Ht, 8);
+    WriteStaticMatrixBlob(sql_stmt_write_two_view_geometry_, qvec, 9);
+    WriteStaticMatrixBlob(sql_stmt_write_two_view_geometry_, tvec, 10);
   } else {
     WriteStaticMatrixBlob(sql_stmt_write_two_view_geometry_,
                           Eigen::MatrixXd(0, 0), 6);
@@ -731,6 +747,10 @@ void Database::WriteTwoViewGeometry(
                           Eigen::MatrixXd(0, 0), 7);
     WriteStaticMatrixBlob(sql_stmt_write_two_view_geometry_,
                           Eigen::MatrixXd(0, 0), 8);
+    WriteStaticMatrixBlob(sql_stmt_write_two_view_geometry_,
+                          Eigen::MatrixXd(0, 0), 9);
+    WriteStaticMatrixBlob(sql_stmt_write_two_view_geometry_,
+                          Eigen::MatrixXd(0, 0), 10);
   }
 
   SQLITE3_CALL(sqlite3_step(sql_stmt_write_two_view_geometry_));
@@ -804,6 +824,35 @@ void Database::DeleteInlierMatches(const image_t image_id1,
   SQLITE3_CALL(sqlite3_reset(sql_stmt_delete_two_view_geometry_));
 }
 
+void Database::ClearAllTables() const {
+  ClearMatches();
+  ClearTwoViewGeometries();
+  ClearDescriptors();
+  ClearKeypoints();
+  ClearImages();
+  ClearCameras();
+}
+
+void Database::ClearCameras() const {
+  SQLITE3_CALL(sqlite3_step(sql_stmt_clear_cameras_));
+  SQLITE3_CALL(sqlite3_reset(sql_stmt_clear_cameras_));
+}
+
+void Database::ClearImages() const {
+  SQLITE3_CALL(sqlite3_step(sql_stmt_clear_images_));
+  SQLITE3_CALL(sqlite3_reset(sql_stmt_clear_images_));
+}
+
+void Database::ClearDescriptors() const {
+  SQLITE3_CALL(sqlite3_step(sql_stmt_clear_descriptors_));
+  SQLITE3_CALL(sqlite3_reset(sql_stmt_clear_descriptors_));
+}
+
+void Database::ClearKeypoints() const {
+  SQLITE3_CALL(sqlite3_step(sql_stmt_clear_keypoints_));
+  SQLITE3_CALL(sqlite3_reset(sql_stmt_clear_keypoints_));
+}
+
 void Database::ClearMatches() const {
   SQLITE3_CALL(sqlite3_step(sql_stmt_clear_matches_));
   SQLITE3_CALL(sqlite3_reset(sql_stmt_clear_matches_));
@@ -814,6 +863,113 @@ void Database::ClearTwoViewGeometries() const {
   SQLITE3_CALL(sqlite3_reset(sql_stmt_clear_two_view_geometries_));
 }
 
+void Database::Merge(const Database& database1, const Database& database2,
+                     Database* merged_database) {
+  // Merge the cameras.
+
+  std::unordered_map<camera_t, camera_t> new_camera_ids1;
+  for (const auto& camera : database1.ReadAllCameras()) {
+    const camera_t new_camera_id = merged_database->WriteCamera(camera);
+    new_camera_ids1.emplace(camera.CameraId(), new_camera_id);
+  }
+
+  std::unordered_map<camera_t, camera_t> new_camera_ids2;
+  for (const auto& camera : database2.ReadAllCameras()) {
+    const camera_t new_camera_id = merged_database->WriteCamera(camera);
+    new_camera_ids2.emplace(camera.CameraId(), new_camera_id);
+  }
+
+  // Merge the images.
+
+  std::unordered_map<image_t, image_t> new_image_ids1;
+  for (auto& image : database1.ReadAllImages()) {
+    image.SetCameraId(new_camera_ids1.at(image.CameraId()));
+    CHECK(!merged_database->ExistsImageWithName(image.Name()))
+        << "The two databases must not contain images with the same name, but "
+           "the there are images with name "
+        << image.Name() << " in both databases";
+    const image_t new_image_id = merged_database->WriteImage(image);
+    new_image_ids1.emplace(image.ImageId(), new_image_id);
+    const auto keypoints = database1.ReadKeypoints(image.ImageId());
+    const auto descriptors = database1.ReadDescriptors(image.ImageId());
+    merged_database->WriteKeypoints(new_image_id, keypoints);
+    merged_database->WriteDescriptors(new_image_id, descriptors);
+  }
+
+  std::unordered_map<image_t, image_t> new_image_ids2;
+  for (auto& image : database2.ReadAllImages()) {
+    image.SetCameraId(new_camera_ids2.at(image.CameraId()));
+    CHECK(!merged_database->ExistsImageWithName(image.Name()))
+        << "The two databases must not contain images with the same name, but "
+           "the there are images with name "
+        << image.Name() << " in both databases";
+    const image_t new_image_id = merged_database->WriteImage(image);
+    new_image_ids2.emplace(image.ImageId(), new_image_id);
+    const auto keypoints = database2.ReadKeypoints(image.ImageId());
+    const auto descriptors = database2.ReadDescriptors(image.ImageId());
+    merged_database->WriteKeypoints(new_image_id, keypoints);
+    merged_database->WriteDescriptors(new_image_id, descriptors);
+  }
+
+  // Merge the matches.
+
+  for (const auto& matches : database1.ReadAllMatches()) {
+    image_t image_id1, image_id2;
+    Database::PairIdToImagePair(matches.first, &image_id1, &image_id2);
+
+    const image_t new_image_id1 = new_image_ids1.at(image_id1);
+    const image_t new_image_id2 = new_image_ids1.at(image_id2);
+
+    merged_database->WriteMatches(new_image_id1, new_image_id2, matches.second);
+  }
+
+  for (const auto& matches : database2.ReadAllMatches()) {
+    image_t image_id1, image_id2;
+    Database::PairIdToImagePair(matches.first, &image_id1, &image_id2);
+
+    const image_t new_image_id1 = new_image_ids2.at(image_id1);
+    const image_t new_image_id2 = new_image_ids2.at(image_id2);
+
+    merged_database->WriteMatches(new_image_id1, new_image_id2, matches.second);
+  }
+
+  // Merge the two-view geometries.
+
+  {
+    std::vector<image_pair_t> image_pair_ids;
+    std::vector<TwoViewGeometry> two_view_geometries;
+    database1.ReadTwoViewGeometries(&image_pair_ids, &two_view_geometries);
+
+    for (size_t i = 0; i < two_view_geometries.size(); ++i) {
+      image_t image_id1, image_id2;
+      Database::PairIdToImagePair(image_pair_ids[i], &image_id1, &image_id2);
+
+      const image_t new_image_id1 = new_image_ids1.at(image_id1);
+      const image_t new_image_id2 = new_image_ids1.at(image_id2);
+
+      merged_database->WriteTwoViewGeometry(new_image_id1, new_image_id2,
+                                            two_view_geometries[i]);
+    }
+  }
+
+  {
+    std::vector<image_pair_t> image_pair_ids;
+    std::vector<TwoViewGeometry> two_view_geometries;
+    database2.ReadTwoViewGeometries(&image_pair_ids, &two_view_geometries);
+
+    for (size_t i = 0; i < two_view_geometries.size(); ++i) {
+      image_t image_id1, image_id2;
+      Database::PairIdToImagePair(image_pair_ids[i], &image_id1, &image_id2);
+
+      const image_t new_image_id1 = new_image_ids2.at(image_id1);
+      const image_t new_image_id2 = new_image_ids2.at(image_id2);
+
+      merged_database->WriteTwoViewGeometry(new_image_id1, new_image_id2,
+                                            two_view_geometries[i]);
+    }
+  }
+}
+
 void Database::BeginTransaction() const {
   SQLITE3_EXEC(database_, "BEGIN TRANSACTION", nullptr);
 }
@@ -963,8 +1119,8 @@ void Database::PrepareSQLStatements() {
   sql_stmts_.push_back(sql_stmt_read_matches_all_);
 
   sql =
-      "SELECT rows, cols, data, config, F, E, H FROM two_view_geometries WHERE "
-      "pair_id = ?;";
+      "SELECT rows, cols, data, config, F, E, H, qvec, tvec FROM "
+      "two_view_geometries WHERE pair_id = ?;";
   SQLITE3_CALL(sqlite3_prepare_v2(database_, sql.c_str(), -1,
                                   &sql_stmt_read_two_view_geometry_, 0));
   sql_stmts_.push_back(sql_stmt_read_two_view_geometry_);
@@ -1001,7 +1157,7 @@ void Database::PrepareSQLStatements() {
 
   sql =
       "INSERT INTO two_view_geometries(pair_id, rows, cols, data, config, F, "
-      "E, H) VALUES(?, ?, ?, ?, ?, ?, ?, ?);";
+      "E, H, qvec, tvec) VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?);";
   SQLITE3_CALL(sqlite3_prepare_v2(database_, sql.c_str(), -1,
                                   &sql_stmt_write_two_view_geometry_, 0));
   sql_stmts_.push_back(sql_stmt_write_two_view_geometry_);
@@ -1022,6 +1178,26 @@ void Database::PrepareSQLStatements() {
   //////////////////////////////////////////////////////////////////////////////
   // clear_*
   //////////////////////////////////////////////////////////////////////////////
+  sql = "DELETE FROM cameras;";
+  SQLITE3_CALL(sqlite3_prepare_v2(database_, sql.c_str(), -1,
+                                  &sql_stmt_clear_cameras_, 0));
+  sql_stmts_.push_back(sql_stmt_clear_cameras_);
+
+  sql = "DELETE FROM images;";
+  SQLITE3_CALL(sqlite3_prepare_v2(database_, sql.c_str(), -1,
+                                  &sql_stmt_clear_images_, 0));
+  sql_stmts_.push_back(sql_stmt_clear_images_);
+
+  sql = "DELETE FROM descriptors;";
+  SQLITE3_CALL(sqlite3_prepare_v2(database_, sql.c_str(), -1,
+                                  &sql_stmt_clear_descriptors_, 0));
+  sql_stmts_.push_back(sql_stmt_clear_descriptors_);
+
+  sql = "DELETE FROM keypoints;";
+  SQLITE3_CALL(sqlite3_prepare_v2(database_, sql.c_str(), -1,
+                                  &sql_stmt_clear_keypoints_, 0));
+  sql_stmts_.push_back(sql_stmt_clear_keypoints_);
+
   sql = "DELETE FROM matches;";
   SQLITE3_CALL(sqlite3_prepare_v2(database_, sql.c_str(), -1,
                                   &sql_stmt_clear_matches_, 0));
@@ -1132,7 +1308,9 @@ void Database::CreateTwoViewGeometriesTable() const {
         "    config   INTEGER               NOT NULL,"
         "    F        BLOB,"
         "    E        BLOB,"
-        "    H        BLOB);";
+        "    H        BLOB,"
+        "    qvec     BLOB,"
+        "    tvec     BLOB);";
     SQLITE3_EXEC(database_, sql.c_str(), nullptr);
   }
 }
@@ -1153,6 +1331,18 @@ void Database::UpdateSchema() const {
                  "ALTER TABLE two_view_geometries ADD COLUMN H BLOB;", nullptr);
   }
 
+  if (!ExistsColumn("two_view_geometries", "qvec")) {
+    SQLITE3_EXEC(database_,
+                 "ALTER TABLE two_view_geometries ADD COLUMN qvec BLOB;",
+                 nullptr);
+  }
+
+  if (!ExistsColumn("two_view_geometries", "tvec")) {
+    SQLITE3_EXEC(database_,
+                 "ALTER TABLE two_view_geometries ADD COLUMN tvec BLOB;",
+                 nullptr);
+  }
+
   // Update user version number.
   std::unique_lock<std::mutex> lock(update_schema_mutex_);
   const std::string update_user_version_sql =
