@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "base/database_cache.h"
 
@@ -54,7 +54,7 @@ void DatabaseCache::AddImage(const class Image& image) {
 
 void DatabaseCache::Load(const Database& database, const size_t min_num_matches,
                          const bool ignore_watermarks,
-                         const std::set<std::string>& image_names) {
+                         const std::unordered_set<std::string>& image_names) {
   //////////////////////////////////////////////////////////////////////////////
   // Load cameras
   //////////////////////////////////////////////////////////////////////////////
@@ -203,4 +203,14 @@ void DatabaseCache::Load(const Database& database, const size_t min_num_matches,
             << std::endl;
 }
 
+const class Image* DatabaseCache::FindImageWithName(
+    const std::string& name) const {
+  for (const auto& image : images_) {
+    if (image.second.Name() == name) {
+      return &image.second;
+    }
+  }
+  return nullptr;
+}
+
 }  // namespace colmap
