@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #define TEST_NAME "base/database"
 #include "util/testing.h"
@@ -35,6 +35,7 @@
 #include <thread>
 
 #include "base/database.h"
+#include "base/pose.h"
 
 using namespace colmap;
 
@@ -147,6 +148,8 @@ BOOST_AUTO_TEST_CASE(TestCamera) {
   BOOST_CHECK_EQUAL(database.ReadAllCameras()[0].CameraId(), camera.CameraId());
   BOOST_CHECK_EQUAL(database.ReadAllCameras()[1].CameraId(),
                     camera2.CameraId());
+  database.ClearCameras();
+  BOOST_CHECK_EQUAL(database.NumCameras(), 0);
 }
 
 BOOST_AUTO_TEST_CASE(TestImage) {
@@ -209,6 +212,8 @@ BOOST_AUTO_TEST_CASE(TestImage) {
   BOOST_CHECK_EQUAL(database.ExistsImage(image.ImageId()), true);
   BOOST_CHECK_EQUAL(database.ExistsImage(image2.ImageId()), true);
   BOOST_CHECK_EQUAL(database.ReadAllImages().size(), 2);
+  database.ClearImages();
+  BOOST_CHECK_EQUAL(database.NumImages(), 0);
 }
 
 BOOST_AUTO_TEST_CASE(TestKeypoints) {
@@ -244,6 +249,10 @@ BOOST_AUTO_TEST_CASE(TestKeypoints) {
   BOOST_CHECK_EQUAL(database.NumKeypoints(), 30);
   BOOST_CHECK_EQUAL(database.MaxNumKeypoints(), 20);
   BOOST_CHECK_EQUAL(database.NumKeypointsForImage(image.ImageId()), 20);
+  database.ClearKeypoints();
+  BOOST_CHECK_EQUAL(database.NumKeypoints(), 0);
+  BOOST_CHECK_EQUAL(database.MaxNumKeypoints(), 0);
+  BOOST_CHECK_EQUAL(database.NumKeypointsForImage(image.ImageId()), 0);
 }
 
 BOOST_AUTO_TEST_CASE(TestDescriptors) {
@@ -277,6 +286,10 @@ BOOST_AUTO_TEST_CASE(TestDescriptors) {
   BOOST_CHECK_EQUAL(database.NumDescriptors(), 30);
   BOOST_CHECK_EQUAL(database.MaxNumDescriptors(), 20);
   BOOST_CHECK_EQUAL(database.NumDescriptorsForImage(image.ImageId()), 20);
+  database.ClearDescriptors();
+  BOOST_CHECK_EQUAL(database.NumDescriptors(), 0);
+  BOOST_CHECK_EQUAL(database.MaxNumDescriptors(), 0);
+  BOOST_CHECK_EQUAL(database.NumDescriptorsForImage(image.ImageId()), 0);
 }
 
 BOOST_AUTO_TEST_CASE(TestMatches) {
@@ -315,6 +328,8 @@ BOOST_AUTO_TEST_CASE(TestTwoViewGeometry) {
   two_view_geometry.F = Eigen::Matrix3d::Random();
   two_view_geometry.E = Eigen::Matrix3d::Random();
   two_view_geometry.H = Eigen::Matrix3d::Random();
+  two_view_geometry.qvec = Eigen::Vector4d::Random();
+  two_view_geometry.tvec = Eigen::Vector3d::Random();
   database.WriteTwoViewGeometry(image_id1, image_id2, two_view_geometry);
   const TwoViewGeometry two_view_geometry_read =
       database.ReadTwoViewGeometry(image_id1, image_id2);
@@ -331,6 +346,8 @@ BOOST_AUTO_TEST_CASE(TestTwoViewGeometry) {
   BOOST_CHECK_EQUAL(two_view_geometry.F, two_view_geometry_read.F);
   BOOST_CHECK_EQUAL(two_view_geometry.E, two_view_geometry_read.E);
   BOOST_CHECK_EQUAL(two_view_geometry.H, two_view_geometry_read.H);
+  BOOST_CHECK_EQUAL(two_view_geometry.qvec, two_view_geometry_read.qvec);
+  BOOST_CHECK_EQUAL(two_view_geometry.tvec, two_view_geometry_read.tvec);
 
   const TwoViewGeometry two_view_geometry_read_inv =
       database.ReadTwoViewGeometry(image_id2, image_id1);
@@ -352,6 +369,13 @@ BOOST_AUTO_TEST_CASE(TestTwoViewGeometry) {
   BOOST_CHECK(two_view_geometry_read_inv.H.inverse().eval().isApprox(
       two_view_geometry_read.H));
 
+  Eigen::Vector4d read_inv_qvec_inv;
+  Eigen::Vector3d read_inv_tvec_inv;
+  InvertPose(two_view_geometry_read_inv.qvec, two_view_geometry_read_inv.tvec,
+             &read_inv_qvec_inv, &read_inv_tvec_inv);
+  BOOST_CHECK_EQUAL(read_inv_qvec_inv, two_view_geometry_read.qvec);
+  BOOST_CHECK(read_inv_tvec_inv.isApprox(two_view_geometry_read.tvec));
+
   std::vector<image_pair_t> image_pair_ids;
   std::vector<TwoViewGeometry> two_view_geometries;
   database.ReadTwoViewGeometries(&image_pair_ids, &two_view_geometries);
@@ -363,6 +387,8 @@ BOOST_AUTO_TEST_CASE(TestTwoViewGeometry) {
   BOOST_CHECK_EQUAL(two_view_geometry.F, two_view_geometries[0].F);
   BOOST_CHECK_EQUAL(two_view_geometry.E, two_view_geometries[0].E);
   BOOST_CHECK_EQUAL(two_view_geometry.H, two_view_geometries[0].H);
+  BOOST_CHECK_EQUAL(two_view_geometry.qvec, two_view_geometries[0].qvec);
+  BOOST_CHECK_EQUAL(two_view_geometry.tvec, two_view_geometries[0].tvec);
   BOOST_CHECK_EQUAL(two_view_geometry.inlier_matches.size(),
                     two_view_geometries[0].inlier_matches.size());
   std::vector<std::pair<image_t, image_t>> image_pairs;
@@ -381,3 +407,93 @@ BOOST_AUTO_TEST_CASE(TestTwoViewGeometry) {
   database.ClearTwoViewGeometries();
   BOOST_CHECK_EQUAL(database.NumInlierMatches(), 0);
 }
+
+BOOST_AUTO_TEST_CASE(TestMerge) {
+  Database database1(kMemoryDatabasePath);
+  Database database2(kMemoryDatabasePath);
+
+  Camera camera;
+  camera.InitializeWithName("SIMPLE_PINHOLE", 1.0, 1, 1);
+  camera.SetCameraId(database1.WriteCamera(camera));
+  camera.SetCameraId(database2.WriteCamera(camera));
+
+  Image image;
+  image.SetCameraId(camera.CameraId());
+  image.SetQvecPrior(Eigen::Vector4d(0.1, 0.2, 0.3, 0.4));
+  image.SetTvecPrior(Eigen::Vector3d(0.1, 0.2, 0.3));
+
+  image.SetName("test1");
+  const image_t image_id1 = database1.WriteImage(image);
+  image.SetName("test2");
+  const image_t image_id2 = database1.WriteImage(image);
+  image.SetName("test3");
+  const image_t image_id3 = database2.WriteImage(image);
+  image.SetName("test4");
+  const image_t image_id4 = database2.WriteImage(image);
+
+  auto keypoints1 = FeatureKeypoints(10);
+  keypoints1[0].x = 100;
+  auto keypoints2 = FeatureKeypoints(20);
+  keypoints2[0].x = 200;
+  auto keypoints3 = FeatureKeypoints(30);
+  keypoints3[0].x = 300;
+  auto keypoints4 = FeatureKeypoints(40);
+  keypoints4[0].x = 400;
+
+  const auto descriptors1 = FeatureDescriptors::Random(10, 128);
+  const auto descriptors2 = FeatureDescriptors::Random(20, 128);
+  const auto descriptors3 = FeatureDescriptors::Random(30, 128);
+  const auto descriptors4 = FeatureDescriptors::Random(40, 128);
+
+  database1.WriteKeypoints(image_id1, keypoints1);
+  database1.WriteKeypoints(image_id2, keypoints2);
+  database2.WriteKeypoints(image_id3, keypoints3);
+  database2.WriteKeypoints(image_id4, keypoints4);
+  database1.WriteDescriptors(image_id1, descriptors1);
+  database1.WriteDescriptors(image_id2, descriptors2);
+  database2.WriteDescriptors(image_id3, descriptors3);
+  database2.WriteDescriptors(image_id4, descriptors4);
+  database1.WriteMatches(image_id1, image_id2, FeatureMatches(10));
+  database2.WriteMatches(image_id3, image_id4, FeatureMatches(10));
+  database1.WriteTwoViewGeometry(image_id1, image_id2, TwoViewGeometry());
+  database2.WriteTwoViewGeometry(image_id3, image_id4, TwoViewGeometry());
+
+  Database merged_database(kMemoryDatabasePath);
+  Database::Merge(database1, database2, &merged_database);
+  BOOST_CHECK_EQUAL(merged_database.NumCameras(), 2);
+  BOOST_CHECK_EQUAL(merged_database.NumImages(), 4);
+  BOOST_CHECK_EQUAL(merged_database.NumKeypoints(), 100);
+  BOOST_CHECK_EQUAL(merged_database.NumDescriptors(), 100);
+  BOOST_CHECK_EQUAL(merged_database.NumMatches(), 20);
+  BOOST_CHECK_EQUAL(merged_database.NumInlierMatches(), 0);
+  BOOST_CHECK_EQUAL(merged_database.ReadAllImages()[0].CameraId(), 1);
+  BOOST_CHECK_EQUAL(merged_database.ReadAllImages()[1].CameraId(), 1);
+  BOOST_CHECK_EQUAL(merged_database.ReadAllImages()[2].CameraId(), 2);
+  BOOST_CHECK_EQUAL(merged_database.ReadAllImages()[3].CameraId(), 2);
+  BOOST_CHECK_EQUAL(merged_database.ReadKeypoints(1).size(), 10);
+  BOOST_CHECK_EQUAL(merged_database.ReadKeypoints(2).size(), 20);
+  BOOST_CHECK_EQUAL(merged_database.ReadKeypoints(3).size(), 30);
+  BOOST_CHECK_EQUAL(merged_database.ReadKeypoints(4).size(), 40);
+  BOOST_CHECK_EQUAL(merged_database.ReadKeypoints(1)[0].x, 100);
+  BOOST_CHECK_EQUAL(merged_database.ReadKeypoints(2)[0].x, 200);
+  BOOST_CHECK_EQUAL(merged_database.ReadKeypoints(3)[0].x, 300);
+  BOOST_CHECK_EQUAL(merged_database.ReadKeypoints(4)[0].x, 400);
+  BOOST_CHECK_EQUAL(merged_database.ReadDescriptors(1).size(),
+                    descriptors1.size());
+  BOOST_CHECK_EQUAL(merged_database.ReadDescriptors(2).size(),
+                    descriptors2.size());
+  BOOST_CHECK_EQUAL(merged_database.ReadDescriptors(3).size(),
+                    descriptors3.size());
+  BOOST_CHECK_EQUAL(merged_database.ReadDescriptors(4).size(),
+                    descriptors4.size());
+  BOOST_CHECK(merged_database.ExistsMatches(1, 2));
+  BOOST_CHECK(!merged_database.ExistsMatches(2, 3));
+  BOOST_CHECK(!merged_database.ExistsMatches(2, 4));
+  BOOST_CHECK(merged_database.ExistsMatches(3, 4));
+  merged_database.ClearAllTables();
+  BOOST_CHECK_EQUAL(merged_database.NumCameras(), 0);
+  BOOST_CHECK_EQUAL(merged_database.NumImages(), 0);
+  BOOST_CHECK_EQUAL(merged_database.NumKeypoints(), 0);
+  BOOST_CHECK_EQUAL(merged_database.NumDescriptors(), 0);
+  BOOST_CHECK_EQUAL(merged_database.NumMatches(), 0);
+}
