@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "base/gps.h"
 
@@ -38,14 +38,14 @@ namespace colmap {
 GPSTransform::GPSTransform(const int ellipsoid) {
   switch (ellipsoid) {
     case GRS80:
-      a_ = 6378137;
-      b_ = 6.356752314140356e+06;
-      f_ = 0.003352810681182;
+      a_ = 6378137.0;
+      f_ = 1.0 / 298.257222100882711243162837;  // More accurate GRS80 ellipsoid
+      b_ = (1.0 - f_) * a_;
       break;
     case WGS84:
-      a_ = 6378137;
-      b_ = 6.356752314245179e+06;
-      f_ = 0.003352810664747;
+      a_ = 6378137.0;
+      f_ = 1.0 / 298.257223563;  // The WGS84 ellipsoid
+      b_ = (1.0 - f_) * a_;
       break;
     default:
       a_ = std::numeric_limits<double>::quiet_NaN();
@@ -53,8 +53,7 @@ GPSTransform::GPSTransform(const int ellipsoid) {
       f_ = std::numeric_limits<double>::quiet_NaN();
       throw std::invalid_argument("Ellipsoid not defined");
   }
-
-  e2_ = (a_ * a_ - b_ * b_) / (a_ * a_);
+  e2_ = f_ * (2.0 - f_);
 }
 
 std::vector<Eigen::Vector3d> GPSTransform::EllToXYZ(
@@ -91,21 +90,19 @@ std::vector<Eigen::Vector3d> GPSTransform::XYZToEll(
     const double y = xyz[i](1);
     const double z = xyz[i](2);
 
-    const double xx = x * x;
-    const double yy = y * y;
-
+    const double radius_xy = sqrt(x * x + y * y);
     const double kEps = 1e-12;
 
     // Latitude
-    double lat = atan2(z, sqrt(xx + yy));
+    double lat = atan2(z, radius_xy);
     double alt;
 
     for (size_t j = 0; j < 100; ++j) {
       const double sin_lat0 = sin(lat);
       const double N = a_ / sqrt(1 - e2_ * sin_lat0 * sin_lat0);
-      alt = sqrt(xx + yy) / cos(lat) - N;
+      alt = radius_xy / cos(lat) - N;
       const double prev_lat = lat;
-      lat = atan((z / sqrt(xx + yy)) * 1 / (1 - e2_ * N / (N + alt)));
+      lat = atan((z / radius_xy) * 1 / (1 - e2_ * N / (N + alt)));
 
       if (std::abs(prev_lat - lat) < kEps) {
         break;
@@ -123,4 +120,76 @@ std::vector<Eigen::Vector3d> GPSTransform::XYZToEll(
   return ell;
 }
 
+std::vector<Eigen::Vector3d> GPSTransform::EllToENU(
+    const std::vector<Eigen::Vector3d>& ell, const double lat0,
+    const double lon0) const {
+  // Convert GPS (lat / lon / alt) to ECEF
+  std::vector<Eigen::Vector3d> xyz = EllToXYZ(ell);
+
+  return XYZToENU(xyz, lat0, lon0);
+}
+
+std::vector<Eigen::Vector3d> GPSTransform::XYZToENU(
+    const std::vector<Eigen::Vector3d>& xyz, const double lat0,
+    const double lon0) const {
+  std::vector<Eigen::Vector3d> enu(xyz.size());
+
+  // https://en.wikipedia.org/wiki/Geographic_coordinate_conversion#From_ECEF_to_ENU
+
+  // ECEF to ENU Rot :
+  const double cos_lat0 = std::cos(DegToRad(lat0));
+  const double sin_lat0 = std::sin(DegToRad(lat0));
+
+  const double cos_lon0 = std::cos(DegToRad(lon0));
+  const double sin_lon0 = std::sin(DegToRad(lon0));
+
+  Eigen::Matrix3d R;
+  R << -sin_lon0, cos_lon0, 0., -sin_lat0 * cos_lon0, -sin_lat0 * sin_lon0,
+      cos_lat0, cos_lat0 * cos_lon0, cos_lat0 * sin_lon0, sin_lat0;
+
+  // Convert ECEF to ENU coords. (w.r.t. ECEF ref == xyz[0])
+  for (size_t i = 0; i < xyz.size(); ++i) {
+    enu[i] = R * (xyz[i] - xyz[0]);
+  }
+
+  return enu;
+}
+
+std::vector<Eigen::Vector3d> GPSTransform::ENUToEll(
+    const std::vector<Eigen::Vector3d>& enu, const double lat0,
+    const double lon0, const double alt0) const {
+  return XYZToEll(ENUToXYZ(enu, lat0, lon0, alt0));
+}
+
+std::vector<Eigen::Vector3d> GPSTransform::ENUToXYZ(
+    const std::vector<Eigen::Vector3d>& enu, const double lat0,
+    const double lon0, const double alt0) const {
+  std::vector<Eigen::Vector3d> xyz(enu.size());
+
+  // ECEF ref (origin)
+  const Eigen::Vector3d xyz_ref =
+      EllToXYZ({Eigen::Vector3d(lat0, lon0, alt0)})[0];
+
+  // ENU to ECEF Rot :
+  const double cos_lat0 = std::cos(DegToRad(lat0));
+  const double sin_lat0 = std::sin(DegToRad(lat0));
+
+  const double cos_lon0 = std::cos(DegToRad(lon0));
+  const double sin_lon0 = std::sin(DegToRad(lon0));
+
+  Eigen::Matrix3d R;
+  R << -sin_lon0, cos_lon0, 0., -sin_lat0 * cos_lon0, -sin_lat0 * sin_lon0,
+      cos_lat0, cos_lat0 * cos_lon0, cos_lat0 * sin_lon0, sin_lat0;
+
+  // R is ECEF to ENU so Transpose to get inverse
+  R.transposeInPlace();
+
+  // Convert ENU to ECEF coords.
+  for (size_t i = 0; i < enu.size(); ++i) {
+    xyz[i] = (R * enu[i]) + xyz_ref;
+  }
+
+  return xyz;
+}
+
 }  // namespace colmap
