@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_BASE_GPS_H_
 #define COLMAP_SRC_BASE_GPS_H_
@@ -55,6 +55,24 @@ class GPSTransform {
   std::vector<Eigen::Vector3d> XYZToEll(
       const std::vector<Eigen::Vector3d>& xyz) const;
 
+  // Convert GPS (lat / lon / alt) to ENU coords. with lat0 and lon0
+  // defining the origin of the ENU frame
+  std::vector<Eigen::Vector3d> EllToENU(const std::vector<Eigen::Vector3d>& ell,
+                                        const double lat0,
+                                        const double lon0) const;
+
+  std::vector<Eigen::Vector3d> XYZToENU(const std::vector<Eigen::Vector3d>& xyz,
+                                        const double lat0,
+                                        const double lon0) const;
+
+  std::vector<Eigen::Vector3d> ENUToEll(const std::vector<Eigen::Vector3d>& enu,
+                                        const double lat0, const double lon0,
+                                        const double alt0) const;
+
+  std::vector<Eigen::Vector3d> ENUToXYZ(const std::vector<Eigen::Vector3d>& enu,
+                                        const double lat0, const double lon0,
+                                        const double alt0) const;
+
  private:
   // Semimajor axis.
   double a_;
