@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_BASE_IMAGE_H_
 #define COLMAP_SRC_BASE_IMAGE_H_
@@ -125,7 +125,7 @@ class Image {
   inline bool HasQvecPrior() const;
   inline void SetQvecPrior(const Eigen::Vector4d& qvec);
 
-  // Access quaternion vector as (tx, ty, tz) specifying the translation of the
+  // Access translation vector as (tx, ty, tz) specifying the translation of the
   // pose which is defined as the transformation from world to image space.
   inline const Eigen::Vector3d& Tvec() const;
   inline Eigen::Vector3d& Tvec();
@@ -133,7 +133,7 @@ class Image {
   inline double& Tvec(const size_t idx);
   inline void SetTvec(const Eigen::Vector3d& tvec);
 
-  // Quaternion prior, e.g. given by EXIF GPS tag.
+  // Translation prior, e.g. given by EXIF GPS tag.
   inline const Eigen::Vector3d& TvecPrior() const;
   inline Eigen::Vector3d& TvecPrior();
   inline double TvecPrior(const size_t idx) const;
@@ -237,7 +237,7 @@ class Image {
   std::vector<class Point2D> points2D_;
 
   // Per image point, the number of correspondences that have a 3D point.
-  std::vector<image_t> num_correspondences_have_point3D_;
+  std::vector<point2D_t> num_correspondences_have_point3D_;
 
   // Data structure to compute the distribution of triangulated correspondences
   // in the image. Note that this structure is only usable after `SetUp`.
