@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "base/line.h"
 
@@ -38,6 +38,13 @@ extern "C" {
 }
 
 namespace colmap {
+namespace {
+
+struct RawDeleter {
+  void operator()(double* p) { free(p); }
+};
+
+}  // namespace
 
 std::vector<LineSegment> DetectLineSegments(const Bitmap& bitmap,
                                             const double min_length) {
@@ -55,9 +62,9 @@ std::vector<LineSegment> DetectLineSegments(const Bitmap& bitmap,
                                          bitmap_data.end());
 
   int num_segments;
-  std::unique_ptr<double> segments_data(lsd(&num_segments,
-                                            bitmap_data_double.data(),
-                                            bitmap.Width(), bitmap.Height()));
+  std::unique_ptr<double, RawDeleter> segments_data(
+      lsd(&num_segments, bitmap_data_double.data(), bitmap.Width(),
+          bitmap.Height()));
 
   std::vector<LineSegment> segments;
   segments.reserve(num_segments);
