@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "base/pose.h"
 
@@ -47,9 +47,9 @@ Eigen::Matrix3d CrossProductMatrix(const Eigen::Vector3d& vector) {
 
 void RotationMatrixToEulerAngles(const Eigen::Matrix3d& R, double* rx,
                                  double* ry, double* rz) {
-  *rx = std::atan2(-R(1, 2), R(2, 2));
-  *ry = std::asin(R(0, 2));
-  *rz = std::atan2(-R(0, 1), R(0, 0));
+  *rx = std::atan2(R(2, 1), R(2, 2));
+  *ry = std::asin(-R(2, 0));
+  *rz = std::atan2(R(1, 0), R(0, 0));
 
   *rx = IsNaN(*rx) ? 0 : *rx;
   *ry = IsNaN(*ry) ? 0 : *ry;
@@ -86,15 +86,12 @@ Eigen::Vector4d NormalizeQuaternion(const Eigen::Vector4d& qvec) {
     // for automatic differentiation that would lead to a zero derivative.
     return Eigen::Vector4d(1.0, qvec(1), qvec(2), qvec(3));
   } else {
-    const double inv_norm = 1.0 / norm;
-    return inv_norm * qvec;
+    return qvec / norm;
   }
 }
 
 Eigen::Vector4d InvertQuaternion(const Eigen::Vector4d& qvec) {
-  const Eigen::Vector4d normalized_qvec = NormalizeQuaternion(qvec);
-  return Eigen::Vector4d(normalized_qvec(0), -normalized_qvec(1),
-                         -normalized_qvec(2), -normalized_qvec(3));
+  return Eigen::Vector4d(qvec(0), -qvec(1), -qvec(2), -qvec(3));
 }
 
 Eigen::Vector4d ConcatenateQuaternions(const Eigen::Vector4d& qvec1,
@@ -164,8 +161,8 @@ Eigen::Vector3d ProjectionCenterFromMatrix(
   return -proj_matrix.leftCols<3>().transpose() * proj_matrix.rightCols<1>();
 }
 
-Eigen::Vector3d ProjectionCenterFromParameters(const Eigen::Vector4d& qvec,
-                                               const Eigen::Vector3d& tvec) {
+Eigen::Vector3d ProjectionCenterFromPose(const Eigen::Vector4d& qvec,
+                                         const Eigen::Vector3d& tvec) {
   // Inverse rotation as conjugate quaternion.
   const Eigen::Vector4d normalized_qvec = NormalizeQuaternion(qvec);
   const Eigen::Quaterniond quat(normalized_qvec(0), -normalized_qvec(1),
@@ -220,8 +217,8 @@ Eigen::Vector3d CalculateBaseline(const Eigen::Vector4d& qvec1,
                                   const Eigen::Vector3d& tvec1,
                                   const Eigen::Vector4d& qvec2,
                                   const Eigen::Vector3d& tvec2) {
-  const Eigen::Vector3d center1 = ProjectionCenterFromParameters(qvec1, tvec1);
-  const Eigen::Vector3d center2 = ProjectionCenterFromParameters(qvec2, tvec2);
+  const Eigen::Vector3d center1 = ProjectionCenterFromPose(qvec1, tvec1);
+  const Eigen::Vector3d center2 = ProjectionCenterFromPose(qvec2, tvec2);
   return center2 - center1;
 }
 
