@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #define TEST_NAME "base/pose"
 #include "util/testing.h"
@@ -90,6 +90,20 @@ BOOST_AUTO_TEST_CASE(TestEulerAnglesZ) {
   BOOST_CHECK_CLOSE(rz, rzz, 1e-6);
 }
 
+BOOST_AUTO_TEST_CASE(TestEulerAnglesXYZ) {
+  const double rx = 0.1;
+  const double ry = 0.2;
+  const double rz = 0.3;
+  double rxx, ryy, rzz;
+
+  RotationMatrixToEulerAngles(EulerAnglesToRotationMatrix(rx, ry, rz), &rxx,
+                              &ryy, &rzz);
+
+  BOOST_CHECK_CLOSE(rx, rxx, 1e-6);
+  BOOST_CHECK_CLOSE(ry, ryy, 1e-6);
+  BOOST_CHECK_CLOSE(rz, rzz, 1e-6);
+}
+
 BOOST_AUTO_TEST_CASE(TestQuaternionToRotationMatrix) {
   const double rx = 0;
   const double ry = 0;
@@ -125,10 +139,10 @@ BOOST_AUTO_TEST_CASE(TestInvertQuaternion) {
   BOOST_CHECK_EQUAL(InvertQuaternion(ComposeIdentityQuaternion()),
                     Eigen::Vector4d(1, -0, -0, -0));
   BOOST_CHECK_EQUAL(InvertQuaternion(Eigen::Vector4d(2, 0, 0, 0)),
-                    Eigen::Vector4d(1, -0, -0, -0));
-  BOOST_CHECK(
-      InvertQuaternion(InvertQuaternion(Eigen::Vector4d(0.1, 0.2, 0.3, 0.4)))
-          .isApprox(NormalizeQuaternion(Eigen::Vector4d(0.1, 0.2, 0.3, 0.4))));
+                    Eigen::Vector4d(2, -0, -0, -0));
+  BOOST_CHECK_EQUAL(
+      InvertQuaternion(InvertQuaternion(Eigen::Vector4d(0.1, 0.2, 0.3, 0.4))),
+      Eigen::Vector4d(0.1, 0.2, 0.3, 0.4));
 }
 
 BOOST_AUTO_TEST_CASE(TestConcatenateQuaternions) {
@@ -251,7 +265,7 @@ BOOST_AUTO_TEST_CASE(TestPoseFromProjectionParameters) {
   const Eigen::Vector3d tvec(3, 4, 5);
   const Eigen::Matrix3x4d proj_matrix = ComposeProjectionMatrix(qvec, tvec);
   const Eigen::Matrix3x4d inv_proj_matrix = InvertProjectionMatrix(proj_matrix);
-  const Eigen::Vector3d pose = ProjectionCenterFromParameters(qvec, tvec);
+  const Eigen::Vector3d pose = ProjectionCenterFromPose(qvec, tvec);
   BOOST_CHECK((inv_proj_matrix.rightCols<1>() - pose).norm() < 1e-6);
 }
 
@@ -333,19 +347,16 @@ BOOST_AUTO_TEST_CASE(TestConcatenatePoses) {
 BOOST_AUTO_TEST_CASE(TestInvertPose) {
   Eigen::Vector4d inv_qvec;
   Eigen::Vector3d inv_tvec;
-
   InvertPose(ComposeIdentityQuaternion(), Eigen::Vector3d(0, 0, 0), &inv_qvec,
              &inv_tvec);
   BOOST_CHECK_EQUAL(inv_qvec, ComposeIdentityQuaternion());
   BOOST_CHECK_EQUAL(inv_tvec, Eigen::Vector3d(0, 0, 0));
-
   InvertPose(Eigen::Vector4d(0, 1, 2, 3), Eigen::Vector3d(0, 1, 2), &inv_qvec,
              &inv_tvec);
   Eigen::Vector4d inv_inv_qvec;
   Eigen::Vector3d inv_inv_tvec;
   InvertPose(inv_qvec, inv_tvec, &inv_inv_qvec, &inv_inv_tvec);
-  BOOST_CHECK(
-      inv_inv_qvec.isApprox(NormalizeQuaternion(Eigen::Vector4d(0, 1, 2, 3))));
+  BOOST_CHECK_EQUAL(inv_inv_qvec, Eigen::Vector4d(0, 1, 2, 3));
   BOOST_CHECK(inv_inv_tvec.isApprox(Eigen::Vector3d(0, 1, 2)));
 }
 
