@@ -27,19 +27,17 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "base/reconstruction.h"
 
 #include <fstream>
 
 #include "base/database_cache.h"
+#include "base/gps.h"
 #include "base/pose.h"
 #include "base/projection.h"
-#include "base/similarity_transform.h"
 #include "base/triangulation.h"
-#include "estimators/similarity_transform.h"
-#include "optim/loransac.h"
 #include "util/bitmap.h"
 #include "util/misc.h"
 #include "util/ply.h"
@@ -94,7 +92,9 @@ void Reconstruction::Load(const DatabaseCache& database_cache) {
   // Add image pairs.
   for (const auto& image_pair :
        database_cache.CorrespondenceGraph().NumCorrespondencesBetweenImages()) {
-    image_pairs_[image_pair.first] = std::make_pair(0, image_pair.second);
+    ImagePairStat image_pair_stat;
+    image_pair_stat.num_total_corrs = image_pair.second;
+    image_pair_stats_.emplace(image_pair.first, image_pair_stat);
   }
 }
 
@@ -122,6 +122,7 @@ void Reconstruction::SetUp(const CorrespondenceGraph* correspondence_graph) {
 
 void Reconstruction::TearDown() {
   correspondence_graph_ = nullptr;
+  image_pair_stats_.clear();
 
   // Remove all not yet registered images.
   std::unordered_set<camera_t> keep_camera_ids;
@@ -277,6 +278,23 @@ void Reconstruction::DeleteObservation(const image_t image_id,
   image.ResetPoint3DForPoint2D(point2D_idx);
 }
 
+void Reconstruction::DeleteAllPoints2DAndPoints3D() {
+  points3D_.clear();
+  for (auto& image : images_) {
+    class Image new_image;
+    new_image.SetImageId(image.second.ImageId());
+    new_image.SetName(image.second.Name());
+    new_image.SetCameraId(image.second.CameraId());
+    new_image.SetRegistered(image.second.IsRegistered());
+    new_image.SetNumCorrespondences(image.second.NumCorrespondences());
+    new_image.SetQvec(image.second.Qvec());
+    new_image.SetQvecPrior(image.second.QvecPrior());
+    new_image.SetTvec(image.second.Tvec());
+    new_image.SetTvecPrior(image.second.TvecPrior());
+    image.second = new_image;
+  }
+}
+
 void Reconstruction::RegisterImage(const image_t image_id) {
   class Image& image = Image(image_id);
   if (!image.IsRegistered()) {
@@ -305,23 +323,54 @@ void Reconstruction::DeRegisterImage(const image_t image_id) {
 void Reconstruction::Normalize(const double extent, const double p0,
                                const double p1, const bool use_images) {
   CHECK_GT(extent, 0);
-  CHECK_GE(p0, 0);
-  CHECK_LE(p0, 1);
-  CHECK_GE(p1, 0);
-  CHECK_LE(p1, 1);
-  CHECK_LE(p0, p1);
 
   if ((use_images && reg_image_ids_.size() < 2) ||
       (!use_images && points3D_.size() < 2)) {
     return;
   }
 
-  EIGEN_STL_UMAP(class Image*, Eigen::Vector3d) proj_centers;
+  auto bound = ComputeBoundsAndCentroid(p0, p1, use_images);
 
-  for (size_t i = 0; i < reg_image_ids_.size(); ++i) {
-    class Image& image = Image(reg_image_ids_[i]);
-    const Eigen::Vector3d proj_center = image.ProjectionCenter();
-    proj_centers[&image] = proj_center;
+  // Calculate scale and translation, such that
+  // translation is applied before scaling.
+  const double old_extent = (std::get<1>(bound) - std::get<0>(bound)).norm();
+  double scale;
+  if (old_extent < std::numeric_limits<double>::epsilon()) {
+    scale = 1;
+  } else {
+    scale = extent / old_extent;
+  }
+
+  SimilarityTransform3 tform(scale, ComposeIdentityQuaternion(),
+                             -scale * std::get<2>(bound));
+  Transform(tform);
+}
+
+Eigen::Vector3d Reconstruction::ComputeCentroid(const double p0,
+                                                const double p1) const {
+  return std::get<2>(ComputeBoundsAndCentroid(p0, p1, false));
+}
+
+std::pair<Eigen::Vector3d, Eigen::Vector3d> Reconstruction::ComputeBoundingBox(
+    const double p0, const double p1) const {
+  auto bound = ComputeBoundsAndCentroid(p0, p1, false);
+  return std::make_pair(std::get<0>(bound), std::get<1>(bound));
+}
+
+std::tuple<Eigen::Vector3d, Eigen::Vector3d, Eigen::Vector3d>
+Reconstruction::ComputeBoundsAndCentroid(const double p0, const double p1,
+                                         const bool use_images) const {
+  CHECK_GE(p0, 0);
+  CHECK_LE(p0, 1);
+  CHECK_GE(p1, 0);
+  CHECK_LE(p1, 1);
+  CHECK_LE(p0, p1);
+
+  const size_t num_elements =
+      use_images ? reg_image_ids_.size() : points3D_.size();
+  if (num_elements == 0) {
+    return std::make_tuple(Eigen::Vector3d(0, 0, 0), Eigen::Vector3d(0, 0, 0),
+                           Eigen::Vector3d(0, 0, 0));
   }
 
   // Coordinates of image centers or point locations.
@@ -329,13 +378,14 @@ void Reconstruction::Normalize(const double extent, const double p0,
   std::vector<float> coords_y;
   std::vector<float> coords_z;
   if (use_images) {
-    coords_x.reserve(proj_centers.size());
-    coords_y.reserve(proj_centers.size());
-    coords_z.reserve(proj_centers.size());
-    for (const auto& proj_center : proj_centers) {
-      coords_x.push_back(static_cast<float>(proj_center.second(0)));
-      coords_y.push_back(static_cast<float>(proj_center.second(1)));
-      coords_z.push_back(static_cast<float>(proj_center.second(2)));
+    coords_x.reserve(reg_image_ids_.size());
+    coords_y.reserve(reg_image_ids_.size());
+    coords_z.reserve(reg_image_ids_.size());
+    for (const image_t im_id : reg_image_ids_) {
+      const Eigen::Vector3d proj_center = Image(im_id).ProjectionCenter();
+      coords_x.push_back(static_cast<float>(proj_center(0)));
+      coords_y.push_back(static_cast<float>(proj_center(1)));
+      coords_z.push_back(static_cast<float>(proj_center(2)));
     }
   } else {
     coords_x.reserve(points3D_.size());
@@ -370,32 +420,7 @@ void Reconstruction::Normalize(const double extent, const double p0,
   }
   mean_coord /= P1 - P0 + 1;
 
-  // Calculate scale and translation, such that
-  // translation is applied before scaling.
-  const double old_extent = (bbox_max - bbox_min).norm();
-  double scale;
-  if (old_extent < std::numeric_limits<double>::epsilon()) {
-    scale = 1;
-  } else {
-    scale = extent / old_extent;
-  }
-
-  const Eigen::Vector3d translation = mean_coord;
-
-  // Transform images.
-  for (auto& elem : proj_centers) {
-    elem.second -= translation;
-    elem.second *= scale;
-    const Eigen::Quaterniond quat(elem.first->Qvec(0), elem.first->Qvec(1),
-                                  elem.first->Qvec(2), elem.first->Qvec(3));
-    elem.first->SetTvec(quat * -elem.second);
-  }
-
-  // Transform points.
-  for (auto& point3D : points3D_) {
-    point3D.second.XYZ() -= translation;
-    point3D.second.XYZ() *= scale;
-  }
+  return std::make_tuple(bbox_min, bbox_max, mean_coord);
 }
 
 void Reconstruction::Transform(const SimilarityTransform3& tform) {
@@ -407,6 +432,34 @@ void Reconstruction::Transform(const SimilarityTransform3& tform) {
   }
 }
 
+Reconstruction Reconstruction::Crop(
+    const std::pair<Eigen::Vector3d, Eigen::Vector3d>& bbox) const {
+  // add all cameras and images. Only the registered images will be used.
+  Reconstruction reconstruction;
+  for (const auto& camera_el : cameras_) {
+    reconstruction.AddCamera(camera_el.second);
+  }
+  for (const auto& image_el : images_) {
+    reconstruction.AddImage(image_el.second);
+    auto& image = reconstruction.Image(image_el.first);
+    image.SetRegistered(false);
+    for (point2D_t pid = 0; pid < image.NumPoints2D(); ++pid) {
+      image.ResetPoint3DForPoint2D(pid);
+    }
+  }
+  for (const auto& point_el : points3D_) {
+    const auto& point = point_el.second;
+    if ((point.XYZ().array() >= bbox.first.array()).all() &&
+        (point.XYZ().array() <= bbox.second.array()).all()) {
+      for (const auto& track_el : point.Track().Elements()) {
+        reconstruction.RegisterImage(track_el.image_id);
+      }
+      reconstruction.AddPoint3D(point.XYZ(), point.Track(), point.Color());
+    }
+  }
+  return reconstruction;
+}
+
 bool Reconstruction::Merge(const Reconstruction& reconstruction,
                            const double max_reproj_error) {
   const double kMinInlierObservations = 0.3;
@@ -460,7 +513,7 @@ bool Reconstruction::Merge(const Reconstruction& reconstruction,
   for (const auto& point3D : reconstruction.Points3D()) {
     Track new_track;
     Track old_track;
-    std::set<point3D_t> old_point3D_ids;
+    std::unordered_set<point3D_t> old_point3D_ids;
     for (const auto& track_el : point3D.second.Track().Elements()) {
       if (common_image_ids.count(track_el.image_id) > 0) {
         const auto& point2D =
@@ -492,108 +545,16 @@ bool Reconstruction::Merge(const Reconstruction& reconstruction,
     }
   }
 
-  return true;
-}
-
-bool Reconstruction::Align(const std::vector<std::string>& image_names,
-                           const std::vector<Eigen::Vector3d>& locations,
-                           const int min_common_images) {
-  CHECK_GE(min_common_images, 3);
-  CHECK_EQ(image_names.size(), locations.size());
-
-  // Find out which images are contained in the reconstruction and get the
-  // positions of their camera centers.
-  std::set<image_t> common_image_ids;
-  std::vector<Eigen::Vector3d> src;
-  std::vector<Eigen::Vector3d> dst;
-  for (size_t i = 0; i < image_names.size(); ++i) {
-    const class Image* image = FindImageWithName(image_names[i]);
-    if (image == nullptr) {
-      continue;
-    }
-
-    if (!IsImageRegistered(image->ImageId())) {
-      continue;
-    }
-
-    // Ignore duplicate images.
-    if (common_image_ids.count(image->ImageId()) > 0) {
-      continue;
-    }
-
-    common_image_ids.insert(image->ImageId());
-    src.push_back(image->ProjectionCenter());
-    dst.push_back(locations[i]);
-  }
-
-  // Only compute the alignment if there are enough correspondences.
-  if (common_image_ids.size() < static_cast<size_t>(min_common_images)) {
-    return false;
-  }
-
-  SimilarityTransform3 tform;
-  tform.Estimate(src, dst);
-  Transform(tform);
-
-  return true;
-}
-
-bool Reconstruction::AlignRobust(const std::vector<std::string>& image_names,
-                                 const std::vector<Eigen::Vector3d>& locations,
-                                 const int min_common_images,
-                                 const RANSACOptions& ransac_options) {
-  CHECK_GE(min_common_images, 3);
-  CHECK_EQ(image_names.size(), locations.size());
-
-  // Find out which images are contained in the reconstruction and get the
-  // positions of their camera centers.
-  std::set<image_t> common_image_ids;
-  std::vector<Eigen::Vector3d> src;
-  std::vector<Eigen::Vector3d> dst;
-  for (size_t i = 0; i < image_names.size(); ++i) {
-    const class Image* image = FindImageWithName(image_names[i]);
-    if (image == nullptr) {
-      continue;
-    }
-
-    if (!IsImageRegistered(image->ImageId())) {
-      continue;
-    }
-
-    // Ignore duplicate images.
-    if (common_image_ids.count(image->ImageId()) > 0) {
-      continue;
-    }
-
-    common_image_ids.insert(image->ImageId());
-    src.push_back(image->ProjectionCenter());
-    dst.push_back(locations[i]);
-  }
-
-  // Only compute the alignment if there are enough correspondences.
-  if (common_image_ids.size() < static_cast<size_t>(min_common_images)) {
-    return false;
-  }
-
-  LORANSAC<SimilarityTransformEstimator<3>, SimilarityTransformEstimator<3>>
-      ransac(ransac_options);
-
-  const auto report = ransac.Estimate(src, dst);
-
-  if (report.support.num_inliers < static_cast<size_t>(min_common_images)) {
-    return false;
-  }
-
-  Transform(SimilarityTransform3(report.model));
+  FilterPoints3DWithLargeReprojectionError(max_reproj_error, Point3DIds());
 
   return true;
 }
 
 const class Image* Reconstruction::FindImageWithName(
     const std::string& name) const {
-  for (const auto& elem : images_) {
-    if (elem.second.Name() == name) {
-      return &elem.second;
+  for (const auto& image : images_) {
+    if (image.second.Name() == name) {
+      return &image.second;
     }
   }
   return nullptr;
@@ -612,6 +573,39 @@ std::vector<image_t> Reconstruction::FindCommonRegImageIds(
   return common_reg_image_ids;
 }
 
+void Reconstruction::TranscribeImageIdsToDatabase(const Database& database) {
+  std::unordered_map<image_t, image_t> old_to_new_image_ids;
+  old_to_new_image_ids.reserve(NumImages());
+
+  EIGEN_STL_UMAP(image_t, class Image) new_images;
+  new_images.reserve(NumImages());
+
+  for (auto& image : images_) {
+    if (!database.ExistsImageWithName(image.second.Name())) {
+      LOG(FATAL) << "Image with name " << image.second.Name()
+                 << " does not exist in database";
+    }
+
+    const auto database_image = database.ReadImageWithName(image.second.Name());
+    old_to_new_image_ids.emplace(image.second.ImageId(),
+                                 database_image.ImageId());
+    image.second.SetImageId(database_image.ImageId());
+    new_images.emplace(database_image.ImageId(), image.second);
+  }
+
+  images_ = std::move(new_images);
+
+  for (auto& image_id : reg_image_ids_) {
+    image_id = old_to_new_image_ids.at(image_id);
+  }
+
+  for (auto& point3D : points3D_) {
+    for (auto& track_el : point3D.second.Track().Elements()) {
+      track_el.image_id = old_to_new_image_ids.at(track_el.image_id);
+    }
+  }
+}
+
 size_t Reconstruction::FilterPoints3D(
     const double max_reproj_error, const double min_tri_angle,
     const std::unordered_set<point3D_t>& point3D_ids) {
@@ -809,10 +803,23 @@ void Reconstruction::ImportPLY(const std::string& path) {
   }
 }
 
-bool Reconstruction::ExportNVM(const std::string& path) const {
+void Reconstruction::ImportPLY(const std::vector<PlyPoint>& ply_points) {
+  points3D_.clear();
+  points3D_.reserve(ply_points.size());
+  for (const auto& ply_point : ply_points) {
+    AddPoint3D(Eigen::Vector3d(ply_point.x, ply_point.y, ply_point.z), Track(),
+               Eigen::Vector3ub(ply_point.r, ply_point.g, ply_point.b));
+  }
+}
+
+bool Reconstruction::ExportNVM(const std::string& path,
+                               bool skip_distortion) const {
   std::ofstream file(path, std::ios::trunc);
   CHECK(file.is_open()) << path;
 
+  // Ensure that we don't lose any precision by storing in text.
+  file.precision(17);
+
   // White space added for compatibility with Meshlab.
   file << "NVM_V3 " << std::endl << " " << std::endl;
   file << reg_image_ids_.size() << "  " << std::endl;
@@ -824,20 +831,24 @@ bool Reconstruction::ExportNVM(const std::string& path) const {
     const class Image& image = Image(image_id);
     const class Camera& camera = Camera(image.CameraId());
 
-    if (camera.ModelId() != SimpleRadialCameraModel::model_id) {
-      std::cout << "WARNING: NVM only supports `SIMPLE_RADIAL` camera model."
+    double k;
+    if (skip_distortion ||
+        camera.ModelId() == SimplePinholeCameraModel::model_id ||
+        camera.ModelId() == PinholeCameraModel::model_id) {
+      k = 0.0;
+    } else if (camera.ModelId() == SimpleRadialCameraModel::model_id) {
+      k = -1 * camera.Params(SimpleRadialCameraModel::extra_params_idxs[0]);
+    } else {
+      std::cout << "WARNING: NVM only supports `SIMPLE_RADIAL` "
+                   "and pinhole camera models."
                 << std::endl;
       return false;
     }
 
-    const double f =
-        camera.Params(SimpleRadialCameraModel::focal_length_idxs[0]);
-    const double k =
-        -1 * camera.Params(SimpleRadialCameraModel::extra_params_idxs[0]);
     const Eigen::Vector3d proj_center = image.ProjectionCenter();
 
     file << image.Name() << " ";
-    file << f << " ";
+    file << camera.MeanFocalLength() << " ";
     file << image.Qvec(0) << " ";
     file << image.Qvec(1) << " ";
     file << image.Qvec(2) << " ";
@@ -889,14 +900,204 @@ bool Reconstruction::ExportNVM(const std::string& path) const {
   return true;
 }
 
+bool Reconstruction::ExportCam(const std::string& path,
+                               bool skip_distortion) const {
+  CreateImageDirs(path);
+  for (const auto image_id : reg_image_ids_) {
+    std::string name, ext;
+    const class Image& image = Image(image_id);
+    const class Camera& camera = Camera(image.CameraId());
+
+    SplitFileExtension(image.Name(), &name, &ext);
+    name = JoinPaths(path, name + ".cam");
+    std::ofstream file(name, std::ios::trunc);
+
+    CHECK(file.is_open()) << name;
+
+    // Ensure that we don't lose any precision by storing in text.
+    file.precision(17);
+
+    double k1, k2;
+    if (skip_distortion ||
+        camera.ModelId() == SimplePinholeCameraModel::model_id ||
+        camera.ModelId() == PinholeCameraModel::model_id) {
+      k1 = 0.0;
+      k2 = 0.0;
+    } else if (camera.ModelId() == SimpleRadialCameraModel::model_id) {
+      k1 = camera.Params(SimpleRadialCameraModel::extra_params_idxs[0]);
+      k2 = 0.0;
+    } else if (camera.ModelId() == RadialCameraModel::model_id) {
+      k1 = camera.Params(RadialCameraModel::extra_params_idxs[0]);
+      k2 = camera.Params(RadialCameraModel::extra_params_idxs[1]);
+    } else {
+      std::cout << "WARNING: CAM only supports `SIMPLE_RADIAL`, `RADIAL`, "
+                   "and pinhole camera models."
+                << std::endl;
+      return false;
+    }
+
+    // If both k1 and k2 values are non-zero, then the CAM format assumes
+    // a Bundler-like radial distortion model, which converts well from
+    // COLMAP. However, if k2 is zero, then a different model is used
+    // that does not translate as well, so we avoid setting k2 to zero.
+    if (k1 != 0.0 && k2 == 0.0) {
+      k2 = 1e-10;
+    }
+
+    double fx, fy;
+    if (camera.FocalLengthIdxs().size() == 2) {
+      fx = camera.FocalLengthX();
+      fy = camera.FocalLengthY();
+    } else {
+      fx = fy = camera.MeanFocalLength();
+    }
+
+    double focal_length;
+    if (camera.Width() * fy < camera.Height() * fx) {
+      focal_length = fy / camera.Height();
+    } else {
+      focal_length = fx / camera.Width();
+    }
+
+    const Eigen::Matrix3d rot_mat = image.RotationMatrix();
+    file << image.Tvec(0) << " " << image.Tvec(1) << " " << image.Tvec(2) << " "
+         << rot_mat(0, 0) << " " << rot_mat(0, 1) << " " << rot_mat(0, 2) << " "
+         << rot_mat(1, 0) << " " << rot_mat(1, 1) << " " << rot_mat(1, 2) << " "
+         << rot_mat(2, 0) << " " << rot_mat(2, 1) << " " << rot_mat(2, 2)
+         << std::endl;
+    file << focal_length << " " << k1 << " " << k2 << " " << fy / fx << " "
+         << camera.PrincipalPointX() / camera.Width() << " "
+         << camera.PrincipalPointY() / camera.Height() << std::endl;
+  }
+
+  return true;
+}
+
+bool Reconstruction::ExportRecon3D(const std::string& path,
+                                   bool skip_distortion) const {
+  std::string base_path = EnsureTrailingSlash(StringReplace(path, "\\", "/"));
+  CreateDirIfNotExists(base_path);
+  base_path = base_path.append("Recon/");
+  CreateDirIfNotExists(base_path);
+  std::string synth_path = base_path + "synth_0.out";
+  std::string image_list_path = base_path + "urd-images.txt";
+  std::string image_map_path = base_path + "imagemap_0.txt";
+
+  std::ofstream synth_file(synth_path, std::ios::trunc);
+  CHECK(synth_file.is_open()) << synth_path;
+  std::ofstream image_list_file(image_list_path, std::ios::trunc);
+  CHECK(image_list_file.is_open()) << image_list_path;
+  std::ofstream image_map_file(image_map_path, std::ios::trunc);
+  CHECK(image_map_file.is_open()) << image_map_path;
+
+  // Ensure that we don't lose any precision by storing in text.
+  synth_file.precision(17);
+
+  // Write header info
+  synth_file << "colmap 1.0" << std::endl;
+  synth_file << reg_image_ids_.size() << " " << points3D_.size() << std::endl;
+
+  std::unordered_map<image_t, size_t> image_id_to_idx_;
+  size_t image_idx = 0;
+
+  // Write image/camera info
+  for (const auto image_id : reg_image_ids_) {
+    const class Image& image = Image(image_id);
+    const class Camera& camera = Camera(image.CameraId());
+
+    double k1, k2;
+    if (skip_distortion ||
+        camera.ModelId() == SimplePinholeCameraModel::model_id ||
+        camera.ModelId() == PinholeCameraModel::model_id) {
+      k1 = 0.0;
+      k2 = 0.0;
+    } else if (camera.ModelId() == SimpleRadialCameraModel::model_id) {
+      k1 = -1 * camera.Params(SimpleRadialCameraModel::extra_params_idxs[0]);
+      k2 = 0.0;
+    } else if (camera.ModelId() == RadialCameraModel::model_id) {
+      k1 = -1 * camera.Params(RadialCameraModel::extra_params_idxs[0]);
+      k2 = -1 * camera.Params(RadialCameraModel::extra_params_idxs[1]);
+    } else {
+      std::cout << "WARNING: Recon3D only supports `SIMPLE_RADIAL`, "
+                   "`RADIAL`, and pinhole camera models."
+                << std::endl;
+      return false;
+    }
+
+    const double scale =
+        1.0 / (double)std::max(camera.Width(), camera.Height());
+    synth_file << scale * camera.MeanFocalLength() << " " << k1 << " " << k2
+               << std::endl;
+    synth_file << QuaternionToRotationMatrix(NormalizeQuaternion(image.Qvec()))
+               << std::endl;
+    synth_file << image.Tvec(0) << " " << image.Tvec(1) << " " << image.Tvec(2)
+               << std::endl;
+
+    image_id_to_idx_[image_id] = image_idx;
+    image_list_file << image.Name() << std::endl
+                    << camera.Width() << " " << camera.Height() << std::endl;
+    image_map_file << image_idx << std::endl;
+
+    image_idx += 1;
+  }
+  image_list_file.close();
+  image_map_file.close();
+
+  // Write point info
+  for (const auto& point3D : points3D_) {
+    auto& p = point3D.second;
+    synth_file << p.XYZ()(0) << " " << p.XYZ()(1) << " " << p.XYZ()(2)
+               << std::endl;
+    synth_file << (int)p.Color(0) << " " << (int)p.Color(1) << " "
+               << (int)p.Color(2) << std::endl;
+
+    std::ostringstream line;
+
+    std::unordered_set<image_t> image_ids;
+    for (const auto& track_el : p.Track().Elements()) {
+      // Make sure that each point only has a single observation per image,
+      // since VisualSfM does not support with multiple observations.
+      if (image_ids.count(track_el.image_id) == 0) {
+        const class Image& image = Image(track_el.image_id);
+        const class Camera& camera = Camera(image.CameraId());
+        const Point2D& point2D = image.Point2D(track_el.point2D_idx);
+
+        const double scale =
+            1.0 / (double)std::max(camera.Width(), camera.Height());
+
+        line << image_id_to_idx_[track_el.image_id] << " ";
+        line << track_el.point2D_idx << " ";
+        // Use a scale of -1.0 to mark as invalid as it is not needed currently
+        line << "-1.0 ";
+        line << (point2D.X() - camera.PrincipalPointX()) * scale << " ";
+        line << (point2D.Y() - camera.PrincipalPointY()) * scale << " ";
+        image_ids.insert(track_el.image_id);
+      }
+    }
+
+    std::string line_string = line.str();
+    line_string = line_string.substr(0, line_string.size() - 1);
+
+    synth_file << image_ids.size() << " ";
+    synth_file << line_string << std::endl;
+  }
+  synth_file.close();
+
+  return true;
+}
+
 bool Reconstruction::ExportBundler(const std::string& path,
-                                   const std::string& list_path) const {
+                                   const std::string& list_path,
+                                   bool skip_distortion) const {
   std::ofstream file(path, std::ios::trunc);
   CHECK(file.is_open()) << path;
 
   std::ofstream list_file(list_path, std::ios::trunc);
   CHECK(list_file.is_open()) << list_path;
 
+  // Ensure that we don't lose any precision by storing in text.
+  file.precision(17);
+
   file << "# Bundle file v0.3" << std::endl;
 
   file << reg_image_ids_.size() << " " << points3D_.size() << std::endl;
@@ -908,30 +1109,26 @@ bool Reconstruction::ExportBundler(const std::string& path,
     const class Image& image = Image(image_id);
     const class Camera& camera = Camera(image.CameraId());
 
-    double f;
-    double k1;
-    double k2;
-    if (camera.ModelId() == SimplePinholeCameraModel::model_id ||
+    double k1, k2;
+    if (skip_distortion ||
+        camera.ModelId() == SimplePinholeCameraModel::model_id ||
         camera.ModelId() == PinholeCameraModel::model_id) {
-      f = camera.MeanFocalLength();
       k1 = 0.0;
       k2 = 0.0;
     } else if (camera.ModelId() == SimpleRadialCameraModel::model_id) {
-      f = camera.Params(SimpleRadialCameraModel::focal_length_idxs[0]);
       k1 = camera.Params(SimpleRadialCameraModel::extra_params_idxs[0]);
       k2 = 0.0;
     } else if (camera.ModelId() == RadialCameraModel::model_id) {
-      f = camera.Params(RadialCameraModel::focal_length_idxs[0]);
       k1 = camera.Params(RadialCameraModel::extra_params_idxs[0]);
       k2 = camera.Params(RadialCameraModel::extra_params_idxs[1]);
     } else {
-      std::cout << "WARNING: Bundler only supports `SIMPLE_RADIAL` and "
-                   "`RADIAL` camera models."
+      std::cout << "WARNING: Bundler only supports `SIMPLE_RADIAL`, "
+                   "`RADIAL`, and pinhole camera models."
                 << std::endl;
       return false;
     }
 
-    file << f << " " << k1 << " " << k2 << std::endl;
+    file << camera.MeanFocalLength() << " " << k1 << " " << k2 << std::endl;
 
     const Eigen::Matrix3d R = image.RotationMatrix();
     file << R(0, 0) << " " << R(0, 1) << " " << R(0, 2) << std::endl;
@@ -1122,7 +1319,7 @@ bool Reconstruction::ExtractColorsForImage(const image_t image_id,
   }
 
   const Eigen::Vector3ub kBlackColor(0, 0, 0);
-  for (const Point2D point2D : image.Points2D()) {
+  for (const Point2D& point2D : image.Points2D()) {
     if (point2D.HasPoint3D()) {
       class Point3D& point3D = Point3D(point2D.Point3DId());
       if (point3D.Color() == kBlackColor) {
@@ -1157,7 +1354,7 @@ void Reconstruction::ExtractColorsForAllImages(const std::string& path) {
       continue;
     }
 
-    for (const Point2D point2D : image.Points2D()) {
+    for (const Point2D& point2D : image.Points2D()) {
       if (point2D.HasPoint3D()) {
         BitmapColor<float> color;
         // COLMAP assumes that the upper left pixel center is (0.5, 0.5).
@@ -1184,7 +1381,9 @@ void Reconstruction::ExtractColorsForAllImages(const std::string& path) {
     if (color_sums.count(point3D.first)) {
       Eigen::Vector3d color =
           color_sums[point3D.first] / color_counts[point3D.first];
-      color.unaryExpr(std::ptr_fun<double, double>(std::round));
+      for (Eigen::Index i = 0; i < color.size(); ++i) {
+        color[i] = std::round(color[i]);
+      }
       point3D.second.SetColor(color.cast<uint8_t>());
     } else {
       point3D.second.SetColor(kBlackColor);
@@ -1193,7 +1392,7 @@ void Reconstruction::ExtractColorsForAllImages(const std::string& path) {
 }
 
 void Reconstruction::CreateImageDirs(const std::string& path) const {
-  std::set<std::string> image_dirs;
+  std::unordered_set<std::string> image_dirs;
   for (const auto& image : images_) {
     const std::vector<std::string> name_split =
         StringSplit(image.second.Name(), "/");
@@ -1275,12 +1474,11 @@ size_t Reconstruction::FilterPoints3DWithSmallTriangulationAngle(
 size_t Reconstruction::FilterPoints3DWithLargeReprojectionError(
     const double max_reproj_error,
     const std::unordered_set<point3D_t>& point3D_ids) {
+  const double max_squared_reproj_error = max_reproj_error * max_reproj_error;
+
   // Number of filtered points.
   size_t num_filtered = 0;
 
-  // Cache for projection matrices.
-  EIGEN_STL_UMAP(image_t, Eigen::Matrix3x4d) proj_matrices;
-
   for (const auto point3D_id : point3D_ids) {
     if (!ExistsPoint3D(point3D_id)) {
       continue;
@@ -1289,6 +1487,7 @@ size_t Reconstruction::FilterPoints3DWithLargeReprojectionError(
     class Point3D& point3D = Point3D(point3D_id);
 
     if (point3D.Track().Length() < 2) {
+      num_filtered += point3D.Track().Length();
       DeletePoint3D(point3D_id);
       continue;
     }
@@ -1299,32 +1498,18 @@ size_t Reconstruction::FilterPoints3DWithLargeReprojectionError(
 
     for (const auto& track_el : point3D.Track().Elements()) {
       const class Image& image = Image(track_el.image_id);
-
-      Eigen::Matrix3x4d proj_matrix;
-      if (proj_matrices.count(track_el.image_id) == 0) {
-        proj_matrix = image.ProjectionMatrix();
-        proj_matrices[track_el.image_id] = proj_matrix;
-      } else {
-        proj_matrix = proj_matrices[track_el.image_id];
-      }
-
-      if (HasPointPositiveDepth(proj_matrix, point3D.XYZ())) {
-        const class Camera& camera = Camera(image.CameraId());
-        const Point2D& point2D = image.Point2D(track_el.point2D_idx);
-        const double reproj_error = CalculateReprojectionError(
-            point2D.XY(), point3D.XYZ(), proj_matrix, camera);
-        if (reproj_error > max_reproj_error) {
-          track_els_to_delete.push_back(track_el);
-        } else {
-          reproj_error_sum += reproj_error;
-        }
-      } else {
+      const class Camera& camera = Camera(image.CameraId());
+      const Point2D& point2D = image.Point2D(track_el.point2D_idx);
+      const double squared_reproj_error = CalculateSquaredReprojectionError(
+          point2D.XY(), point3D.XYZ(), image.Qvec(), image.Tvec(), camera);
+      if (squared_reproj_error > max_squared_reproj_error) {
         track_els_to_delete.push_back(track_el);
+      } else {
+        reproj_error_sum += std::sqrt(squared_reproj_error);
       }
     }
 
-    if (track_els_to_delete.size() == point3D.Track().Length() ||
-        track_els_to_delete.size() == point3D.Track().Length() - 1) {
+    if (track_els_to_delete.size() >= point3D.Track().Length() - 1) {
       num_filtered += point3D.Track().Length();
       DeletePoint3D(point3D_id);
     } else {
@@ -1682,12 +1867,16 @@ void Reconstruction::WriteCamerasText(const std::string& path) const {
   std::ofstream file(path, std::ios::trunc);
   CHECK(file.is_open()) << path;
 
+  // Ensure that we don't loose any precision by storing in text.
+  file.precision(17);
+
   file << "# Camera list with one line of data per camera:" << std::endl;
   file << "#   CAMERA_ID, MODEL, WIDTH, HEIGHT, PARAMS[]" << std::endl;
   file << "# Number of cameras: " << cameras_.size() << std::endl;
 
   for (const auto& camera : cameras_) {
     std::ostringstream line;
+    line.precision(17);
 
     line << camera.first << " ";
     line << camera.second.ModelName() << " ";
@@ -1709,6 +1898,9 @@ void Reconstruction::WriteImagesText(const std::string& path) const {
   std::ofstream file(path, std::ios::trunc);
   CHECK(file.is_open()) << path;
 
+  // Ensure that we don't loose any precision by storing in text.
+  file.precision(17);
+
   file << "# Image list with two lines of data per image:" << std::endl;
   file << "#   IMAGE_ID, QW, QX, QY, QZ, TX, TY, TZ, CAMERA_ID, "
           "NAME"
@@ -1724,6 +1916,8 @@ void Reconstruction::WriteImagesText(const std::string& path) const {
     }
 
     std::ostringstream line;
+    line.precision(17);
+
     std::string line_string;
 
     line << image.first << " ";
@@ -1769,6 +1963,9 @@ void Reconstruction::WritePoints3DText(const std::string& path) const {
   std::ofstream file(path, std::ios::trunc);
   CHECK(file.is_open()) << path;
 
+  // Ensure that we don't loose any precision by storing in text.
+  file.precision(17);
+
   file << "# 3D point list with one line of data per point:" << std::endl;
   file << "#   POINT3D_ID, X, Y, Z, R, G, B, ERROR, "
           "TRACK[] as (IMAGE_ID, POINT2D_IDX)"
@@ -1787,6 +1984,7 @@ void Reconstruction::WritePoints3DText(const std::string& path) const {
     file << point3D.second.Error() << " ";
 
     std::ostringstream line;
+    line.precision(17);
 
     for (const auto& track_el : point3D.second.Track().Elements()) {
       line << track_el.image_id << " ";
@@ -1904,9 +2102,11 @@ void Reconstruction::SetObservationAsTriangulated(
         (is_continued_point3D || image_id < corr.image_id)) {
       const image_pair_t pair_id =
           Database::ImagePairToPairId(image_id, corr.image_id);
-      image_pairs_[pair_id].first += 1;
-      CHECK_LE(image_pairs_[pair_id].first, image_pairs_[pair_id].second)
-          << "The correspondence graph graph must not contain duplicate matches";
+      image_pair_stats_[pair_id].num_tri_corrs += 1;
+      CHECK_LE(image_pair_stats_[pair_id].num_tri_corrs,
+               image_pair_stats_[pair_id].num_total_corrs)
+          << "The correspondence graph graph must not contain duplicate "
+             "matches";
     }
   }
 }
@@ -1936,8 +2136,8 @@ void Reconstruction::ResetTriObservations(const image_t image_id,
         (!is_deleted_point3D || image_id < corr.image_id)) {
       const image_pair_t pair_id =
           Database::ImagePairToPairId(image_id, corr.image_id);
-      image_pairs_[pair_id].first -= 1;
-      CHECK_GE(image_pairs_[pair_id].first, 0)
+      image_pair_stats_[pair_id].num_tri_corrs -= 1;
+      CHECK_GE(image_pair_stats_[pair_id].num_tri_corrs, 0)
           << "The scene graph graph must not contain duplicate matches";
     }
   }
