@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "base/reconstruction_manager.h"
 
@@ -82,10 +82,20 @@ size_t ReconstructionManager::Read(const std::string& path) {
 
 void ReconstructionManager::Write(const std::string& path,
                                   const OptionManager* options) const {
+  std::vector<std::pair<size_t, size_t>> recon_sizes(reconstructions_.size());
+  for (size_t i = 0; i < reconstructions_.size(); ++i) {
+    recon_sizes[i] = std::make_pair(i, reconstructions_[i]->NumPoints3D());
+  }
+  std::sort(recon_sizes.begin(), recon_sizes.end(),
+            [](const std::pair<size_t, size_t>& first,
+               const std::pair<size_t, size_t>& second) {
+              return first.second > second.second;
+            });
+
   for (size_t i = 0; i < reconstructions_.size(); ++i) {
     const std::string reconstruction_path = JoinPaths(path, std::to_string(i));
     CreateDirIfNotExists(reconstruction_path);
-    reconstructions_[i]->Write(reconstruction_path);
+    reconstructions_[recon_sizes[i].first]->Write(reconstruction_path);
     if (options != nullptr) {
       options->Write(JoinPaths(reconstruction_path, "project.ini"));
     }
