@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_BASE_SCENE_CLUSTERING_H_
 #define COLMAP_SRC_BASE_SCENE_CLUSTERING_H_
@@ -35,6 +35,7 @@
 #include <list>
 #include <vector>
 
+#include "base/database.h"
 #include "util/types.h"
 
 namespace colmap {
@@ -45,12 +46,18 @@ namespace colmap {
 class SceneClustering {
  public:
   struct Options {
+    // Flag for hierarchical vs flat clustering
+    bool is_hierarchical = true;
+
     // The branching factor of the hierarchical clustering.
     int branching = 2;
 
     // The number of overlapping images between child clusters.
     int image_overlap = 50;
 
+    // The max related images matches to look for in a flat cluster
+    int num_image_matches = 20;
+
     // The maximum number of images in a leaf node cluster, otherwise the
     // cluster is further partitioned using the given branching factor. Note
     // that a cluster leaf node will have at most `leaf_max_num_images +
@@ -73,9 +80,16 @@ class SceneClustering {
   const Cluster* GetRootCluster() const;
   std::vector<const Cluster*> GetLeafClusters() const;
 
+  static SceneClustering Create(const Options& options,
+                                const Database& database);
+
  private:
-  void PartitionCluster(const std::vector<std::pair<int, int>>& edges,
-                        const std::vector<int>& weights, Cluster* cluster);
+  void PartitionHierarchicalCluster(
+      const std::vector<std::pair<int, int>>& edges,
+      const std::vector<int>& weights, Cluster* cluster);
+
+  void PartitionFlatCluster(const std::vector<std::pair<int, int>>& edges,
+                            const std::vector<int>& weights);
 
   const Options options_;
   std::unique_ptr<Cluster> root_cluster_;
