@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #define TEST_NAME "base/scene_clustering"
 #include "util/testing.h"
@@ -214,3 +214,89 @@ BOOST_AUTO_TEST_CASE(TestTwoOverlap) {
   BOOST_CHECK(image_ids1.count(2));
   BOOST_CHECK(image_ids1.count(3));
 }
+
+BOOST_AUTO_TEST_CASE(TestThreeFlatClusters) {
+  const std::vector<std::pair<image_t, image_t>> image_pairs = {
+      {0, 1}, {2, 3}, {4, 5}, {1, 2}, {3, 4}, {5, 0}, {0, 3}, {2, 5}, {4, 1}};
+  const std::vector<int> num_inliers = {100, 100, 100, 10, 10, 10, 1, 1, 1};
+  SceneClustering::Options options;
+  options.branching = 3;
+  options.image_overlap = 0;
+  options.branching = 3;
+  options.is_hierarchical = false;
+  SceneClustering scene_clustering(options);
+  BOOST_CHECK(scene_clustering.GetRootCluster() == nullptr);
+  scene_clustering.Partition(image_pairs, num_inliers);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids.size(), 6);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids[0], 0);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids[1], 1);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids[2], 2);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids[3], 3);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids[4], 4);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids[5], 5);
+  BOOST_CHECK_EQUAL(scene_clustering.GetLeafClusters().size(), 3);
+  BOOST_CHECK_EQUAL(scene_clustering.GetLeafClusters()[0]->image_ids.size(), 2);
+  const std::set<image_t> image_ids0(
+      scene_clustering.GetLeafClusters()[0]->image_ids.begin(),
+      scene_clustering.GetLeafClusters()[0]->image_ids.end());
+  BOOST_CHECK(image_ids0.count(0));
+  BOOST_CHECK(image_ids0.count(1));
+  BOOST_CHECK_EQUAL(scene_clustering.GetLeafClusters()[1]->image_ids.size(), 2);
+  const std::set<image_t> image_ids1(
+      scene_clustering.GetLeafClusters()[1]->image_ids.begin(),
+      scene_clustering.GetLeafClusters()[1]->image_ids.end());
+  BOOST_CHECK(image_ids1.count(2));
+  BOOST_CHECK(image_ids1.count(3));
+  BOOST_CHECK_EQUAL(scene_clustering.GetLeafClusters()[2]->image_ids.size(), 2);
+  const std::set<image_t> image_ids2(
+      scene_clustering.GetLeafClusters()[2]->image_ids.begin(),
+      scene_clustering.GetLeafClusters()[2]->image_ids.end());
+  BOOST_CHECK(image_ids2.count(4));
+  BOOST_CHECK(image_ids2.count(5));
+}
+
+BOOST_AUTO_TEST_CASE(TestThreeFlatClustersTwoOverlap) {
+  const std::vector<std::pair<image_t, image_t>> image_pairs = {
+      {0, 1}, {2, 3}, {4, 5}, {1, 2}, {3, 4}, {5, 0}, {0, 3}, {2, 5}, {4, 1}};
+  const std::vector<int> num_inliers = {100, 100, 100, 10, 10, 10, 1, 1, 1};
+  SceneClustering::Options options;
+  options.branching = 3;
+  options.image_overlap = 2;
+  options.branching = 3;
+  options.is_hierarchical = false;
+  SceneClustering scene_clustering(options);
+  BOOST_CHECK(scene_clustering.GetRootCluster() == nullptr);
+  scene_clustering.Partition(image_pairs, num_inliers);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids.size(), 6);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids[0], 0);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids[1], 1);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids[2], 2);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids[3], 3);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids[4], 4);
+  BOOST_CHECK_EQUAL(scene_clustering.GetRootCluster()->image_ids[5], 5);
+  BOOST_CHECK_EQUAL(scene_clustering.GetLeafClusters().size(), 3);
+  BOOST_CHECK_EQUAL(scene_clustering.GetLeafClusters()[0]->image_ids.size(), 4);
+  const std::set<image_t> image_ids0(
+      scene_clustering.GetLeafClusters()[0]->image_ids.begin(),
+      scene_clustering.GetLeafClusters()[0]->image_ids.end());
+  BOOST_CHECK(image_ids0.count(0));
+  BOOST_CHECK(image_ids0.count(1));
+  BOOST_CHECK(image_ids0.count(2));
+  BOOST_CHECK(image_ids0.count(5));
+  BOOST_CHECK_EQUAL(scene_clustering.GetLeafClusters()[1]->image_ids.size(), 4);
+  const std::set<image_t> image_ids1(
+      scene_clustering.GetLeafClusters()[1]->image_ids.begin(),
+      scene_clustering.GetLeafClusters()[1]->image_ids.end());
+  BOOST_CHECK(image_ids1.count(1));
+  BOOST_CHECK(image_ids1.count(2));
+  BOOST_CHECK(image_ids1.count(3));
+  BOOST_CHECK(image_ids1.count(4));
+  BOOST_CHECK_EQUAL(scene_clustering.GetLeafClusters()[2]->image_ids.size(), 4);
+  const std::set<image_t> image_ids2(
+      scene_clustering.GetLeafClusters()[2]->image_ids.begin(),
+      scene_clustering.GetLeafClusters()[2]->image_ids.end());
+  BOOST_CHECK(image_ids2.count(0));
+  BOOST_CHECK(image_ids2.count(3));
+  BOOST_CHECK(image_ids2.count(4));
+  BOOST_CHECK(image_ids2.count(5));
+}
