@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "base/similarity_transform.h"
 
@@ -37,6 +37,8 @@
 #include "estimators/similarity_transform.h"
 #include "optim/loransac.h"
 
+#include <fstream>
+
 namespace colmap {
 namespace {
 
@@ -48,7 +50,7 @@ struct ReconstructionAlignmentEstimator {
   typedef Eigen::Matrix3x4d M_t;
 
   void SetMaxReprojError(const double max_reproj_error) {
-    max_reproj_error_ = max_reproj_error;
+    max_squared_reproj_error_ = max_reproj_error * max_reproj_error;
   }
 
   void SetReconstructions(const Reconstruction* reconstruction1,
@@ -64,10 +66,11 @@ struct ReconstructionAlignmentEstimator {
                             const std::vector<Y_t>& images2) const {
     CHECK_GE(images1.size(), 3);
     CHECK_GE(images2.size(), 3);
+    CHECK_EQ(images1.size(), images2.size());
 
     std::vector<Eigen::Vector3d> proj_centers1(images1.size());
     std::vector<Eigen::Vector3d> proj_centers2(images2.size());
-    for (size_t i = 0; i < 3; ++i) {
+    for (size_t i = 0; i < images1.size(); ++i) {
       CHECK_EQ(images1[i]->ImageId(), images2[i]->ImageId());
       proj_centers1[i] = images1[i]->ProjectionCenter();
       proj_centers2[i] = images2[i]->ProjectionCenter();
@@ -131,26 +134,22 @@ struct ReconstructionAlignmentEstimator {
         num_common_points += 1;
 
         // Reproject 3D point in image 1 to image 2.
-
         const Eigen::Vector3d xyz12 =
             alignment12 *
             reconstruction1_->Point3D(point2D1.Point3DId()).XYZ().homogeneous();
-
-        if (!HasPointPositiveDepth(proj_matrix2, xyz12) ||
-            CalculateReprojectionError(point2D2.XY(), xyz12, proj_matrix2,
-                                       camera2) > max_reproj_error_) {
+        if (CalculateSquaredReprojectionError(point2D2.XY(), xyz12,
+                                              proj_matrix2, camera2) >
+            max_squared_reproj_error_) {
           continue;
         }
 
         // Reproject 3D point in image 2 to image 1.
-
         const Eigen::Vector3d xyz21 =
             alignment21 *
             reconstruction2_->Point3D(point2D2.Point3DId()).XYZ().homogeneous();
-
-        if (!HasPointPositiveDepth(proj_matrix1, xyz21) ||
-            CalculateReprojectionError(point2D1.XY(), xyz21, proj_matrix1,
-                                       camera1) > max_reproj_error_) {
+        if (CalculateSquaredReprojectionError(point2D1.XY(), xyz21,
+                                              proj_matrix1, camera1) >
+            max_squared_reproj_error_) {
           continue;
         }
 
@@ -169,17 +168,16 @@ struct ReconstructionAlignmentEstimator {
   }
 
  private:
-  double max_reproj_error_ = 0.0;
+  double max_squared_reproj_error_ = 0.0;
   const Reconstruction* reconstruction1_ = nullptr;
   const Reconstruction* reconstruction2_ = nullptr;
 };
 
 }  // namespace
 
-SimilarityTransform3::SimilarityTransform3() {
-  SimilarityTransform3(1, ComposeIdentityQuaternion(),
-                       Eigen::Vector3d(0, 0, 0));
-}
+SimilarityTransform3::SimilarityTransform3()
+    : SimilarityTransform3(1, ComposeIdentityQuaternion(),
+                           Eigen::Vector3d(0, 0, 0)) {}
 
 SimilarityTransform3::SimilarityTransform3(const Eigen::Matrix3x4d& matrix) {
   transform_.matrix().topLeftCorner<3, 4>() = matrix;
@@ -198,10 +196,12 @@ SimilarityTransform3::SimilarityTransform3(const double scale,
   transform_.matrix() = matrix;
 }
 
-void SimilarityTransform3::Estimate(const std::vector<Eigen::Vector3d>& src,
-                                    const std::vector<Eigen::Vector3d>& dst) {
-  transform_.matrix().topLeftCorner<3, 4>() =
-      SimilarityTransformEstimator<3>().Estimate(src, dst)[0];
+void SimilarityTransform3::Write(const std::string& path) {
+  std::ofstream file(path, std::ios::trunc);
+  CHECK(file.is_open()) << path;
+  // Ensure that we don't loose any precision by storing in text.
+  file.precision(17);
+  file << transform_.matrix() << std::endl;
 }
 
 SimilarityTransform3 SimilarityTransform3::Inverse() const {
@@ -254,6 +254,21 @@ Eigen::Vector3d SimilarityTransform3::Translation() const {
   return Matrix().block<3, 1>(0, 3);
 }
 
+SimilarityTransform3 SimilarityTransform3::FromFile(const std::string& path) {
+  std::ifstream file(path);
+  CHECK(file.is_open()) << path;
+
+  Eigen::Matrix4d matrix = Eigen::MatrixXd::Identity(4, 4);
+  for (int i = 0; i < matrix.rows(); ++i) {
+    for (int j = 0; j < matrix.cols(); ++j) {
+      file >> matrix(i, j);
+    }
+  }
+  SimilarityTransform3 tform;
+  tform.transform_.matrix() = matrix;
+  return tform;
+}
+
 bool ComputeAlignmentBetweenReconstructions(
     const Reconstruction& src_reconstruction,
     const Reconstruction& ref_reconstruction,
