@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_BASE_SIMILARITY_TRANSFORM_H_
 #define COLMAP_SRC_BASE_SIMILARITY_TRANSFORM_H_
@@ -37,6 +37,7 @@
 #include <Eigen/Core>
 #include <Eigen/Geometry>
 
+#include "estimators/similarity_transform.h"
 #include "util/alignment.h"
 #include "util/types.h"
 
@@ -60,7 +61,10 @@ class SimilarityTransform3 {
   SimilarityTransform3(const double scale, const Eigen::Vector4d& qvec,
                        const Eigen::Vector3d& tvec);
 
-  void Estimate(const std::vector<Eigen::Vector3d>& src,
+  void Write(const std::string& path);
+
+  template <bool kEstimateScale = true>
+  bool Estimate(const std::vector<Eigen::Vector3d>& src,
                 const std::vector<Eigen::Vector3d>& dst);
 
   SimilarityTransform3 Inverse() const;
@@ -73,6 +77,8 @@ class SimilarityTransform3 {
   Eigen::Vector4d Rotation() const;
   Eigen::Vector3d Translation() const;
 
+  static SimilarityTransform3 FromFile(const std::string& path);
+
  private:
   Eigen::Transform<double, 3, Eigen::Affine> transform_;
 };
@@ -89,6 +95,25 @@ bool ComputeAlignmentBetweenReconstructions(
     const double min_inlier_observations, const double max_reproj_error,
     Eigen::Matrix3x4d* alignment);
 
+////////////////////////////////////////////////////////////////////////////////
+// Implementation
+////////////////////////////////////////////////////////////////////////////////
+
+template <bool kEstimateScale>
+bool SimilarityTransform3::Estimate(const std::vector<Eigen::Vector3d>& src,
+                                    const std::vector<Eigen::Vector3d>& dst) {
+  const auto results =
+      SimilarityTransformEstimator<3, kEstimateScale>().Estimate(src, dst);
+  if (results.empty()) {
+    return false;
+  }
+
+  CHECK_EQ(results.size(), 1);
+  transform_.matrix().topLeftCorner<3, 4>() = results[0];
+
+  return true;
+}
+
 }  // namespace colmap
 
 EIGEN_DEFINE_STL_VECTOR_SPECIALIZATION_CUSTOM(colmap::SimilarityTransform3)
