@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #define TEST_NAME "base/similarity_transform"
 #include "util/testing.h"
@@ -37,8 +37,25 @@
 #include "base/pose.h"
 #include "base/similarity_transform.h"
 
+#include <fstream>
+
 using namespace colmap;
 
+BOOST_AUTO_TEST_CASE(TestDefaultInitialization) {
+  const SimilarityTransform3 tform;
+
+  BOOST_CHECK_EQUAL(tform.Scale(), 1);
+
+  BOOST_CHECK_EQUAL(tform.Rotation()[0], 1);
+  BOOST_CHECK_EQUAL(tform.Rotation()[1], 0);
+  BOOST_CHECK_EQUAL(tform.Rotation()[2], 0);
+  BOOST_CHECK_EQUAL(tform.Rotation()[3], 0);
+
+  BOOST_CHECK_EQUAL(tform.Translation()[0], 0);
+  BOOST_CHECK_EQUAL(tform.Translation()[1], 0);
+  BOOST_CHECK_EQUAL(tform.Translation()[2], 0);
+}
+
 BOOST_AUTO_TEST_CASE(TestInitialization) {
   const Eigen::Vector4d qvec =
       NormalizeQuaternion(Eigen::Vector4d(0.1, 0.3, 0.2, 0.4));
@@ -47,10 +64,10 @@ BOOST_AUTO_TEST_CASE(TestInitialization) {
 
   BOOST_CHECK_CLOSE(tform.Scale(), 2, 1e-10);
 
-  BOOST_CHECK_CLOSE(tform.Rotation()(0), qvec(0), 1e-10);
-  BOOST_CHECK_CLOSE(tform.Rotation()(1), qvec(1), 1e-10);
-  BOOST_CHECK_CLOSE(tform.Rotation()(2), qvec(2), 1e-10);
-  BOOST_CHECK_CLOSE(tform.Rotation()(3), qvec(3), 1e-10);
+  BOOST_CHECK_CLOSE(tform.Rotation()[0], qvec(0), 1e-10);
+  BOOST_CHECK_CLOSE(tform.Rotation()[1], qvec(1), 1e-10);
+  BOOST_CHECK_CLOSE(tform.Rotation()[2], qvec(2), 1e-10);
+  BOOST_CHECK_CLOSE(tform.Rotation()[3], qvec(3), 1e-10);
 
   BOOST_CHECK_CLOSE(tform.Translation()[0], 100, 1e-10);
   BOOST_CHECK_CLOSE(tform.Translation()[1], 10, 1e-10);
@@ -71,12 +88,31 @@ void TestEstimationWithNumCoords(const size_t num_coords) {
   }
 
   SimilarityTransform3 est_tform;
-  est_tform.Estimate(src, dst);
+  BOOST_CHECK(est_tform.Estimate(src, dst));
 
   BOOST_CHECK((orig_tform.Matrix() - est_tform.Matrix()).norm() < 1e-6);
+
+  std::vector<Eigen::Vector3d> invalid_src_dst(3, Eigen::Vector3d::Zero());
+  BOOST_CHECK(!est_tform.Estimate(invalid_src_dst, invalid_src_dst));
 }
 
 BOOST_AUTO_TEST_CASE(TestEstimation) {
   TestEstimationWithNumCoords(3);
   TestEstimationWithNumCoords(100);
 }
+
+BOOST_AUTO_TEST_CASE(TestFromFile) {
+  // Create transform file
+  const std::string path = "test_from_file_transform.txt";
+  {
+    std::ofstream out(path);
+    out << "0.0 2.0 0.0 3.0 0.0 0.0 2.0 4.0 2.0 0.0 0.0 5.0 0.0 0.0 0.0 1.0"
+        << std::endl;
+  }
+  SimilarityTransform3 tform = SimilarityTransform3::FromFile(path);
+  BOOST_CHECK_CLOSE(tform.Scale(), 2.0, 1e-10);
+  BOOST_CHECK_LE((tform.Translation() - Eigen::Vector3d(3.0, 4.0, 5.0)).norm(),
+                 1e-6);
+  BOOST_CHECK_LE(
+      (tform.Rotation() - Eigen::Vector4d(-0.5, 0.5, 0.5, 0.5)).norm(), 1e-6);
+}
