@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "base/undistortion.h"
 
@@ -83,7 +83,8 @@ void WriteCOLMAPCommands(const bool geometric,
                          const std::string& output_prefix,
                          const std::string& indent, std::ofstream* file) {
   if (geometric) {
-    *file << indent << "$COLMAP_EXE_PATH/patch_match_stereo \\" << std::endl;
+    *file << indent << "$COLMAP_EXE_PATH/colmap patch_match_stereo \\"
+          << std::endl;
     *file << indent << "  --workspace_path " << workspace_path << " \\"
           << std::endl;
     *file << indent << "  --workspace_format " << workspace_format << " \\"
@@ -97,7 +98,8 @@ void WriteCOLMAPCommands(const bool geometric,
     *file << indent << "  --PatchMatchStereo.geom_consistency true"
           << std::endl;
   } else {
-    *file << indent << "$COLMAP_EXE_PATH/patch_match_stereo \\" << std::endl;
+    *file << indent << "$COLMAP_EXE_PATH/colmap patch_match_stereo \\"
+          << std::endl;
     *file << indent << "  --workspace_path " << workspace_path << " \\"
           << std::endl;
     *file << indent << "  --workspace_format " << workspace_format << " \\"
@@ -112,7 +114,7 @@ void WriteCOLMAPCommands(const bool geometric,
           << std::endl;
   }
 
-  *file << indent << "$COLMAP_EXE_PATH/stereo_fusion \\" << std::endl;
+  *file << indent << "$COLMAP_EXE_PATH/colmap stereo_fusion \\" << std::endl;
   *file << indent << "  --workspace_path " << workspace_path << " \\"
         << std::endl;
   *file << indent << "  --workspace_format " << workspace_format << " \\"
@@ -129,7 +131,7 @@ void WriteCOLMAPCommands(const bool geometric,
   *file << indent << "  --output_path "
         << JoinPaths(workspace_path, output_prefix + "fused.ply") << std::endl;
 
-  *file << indent << "$COLMAP_EXE_PATH/poisson_mesher \\" << std::endl;
+  *file << indent << "$COLMAP_EXE_PATH/colmap poisson_mesher \\" << std::endl;
   *file << indent << "  --input_path "
         << JoinPaths(workspace_path, output_prefix + "fused.ply") << " \\"
         << std::endl;
@@ -137,10 +139,10 @@ void WriteCOLMAPCommands(const bool geometric,
         << JoinPaths(workspace_path, output_prefix + "meshed-poisson.ply")
         << std::endl;
 
-  *file << indent << "$COLMAP_EXE_PATH/delaunay_mesher \\" << std::endl;
+  *file << indent << "$COLMAP_EXE_PATH/colmap delaunay_mesher \\" << std::endl;
   *file << indent << "  --input_path "
         << JoinPaths(workspace_path, output_prefix) << " \\" << std::endl;
-  *file << indent << "  --input_type dense " << std::endl;
+  *file << indent << "  --input_type dense \\" << std::endl;
   *file << indent << "  --output_path "
         << JoinPaths(workspace_path, output_prefix + "meshed-delaunay.ply")
         << std::endl;
@@ -151,11 +153,17 @@ void WriteCOLMAPCommands(const bool geometric,
 COLMAPUndistorter::COLMAPUndistorter(const UndistortCameraOptions& options,
                                      const Reconstruction& reconstruction,
                                      const std::string& image_path,
-                                     const std::string& output_path)
+                                     const std::string& output_path,
+                                     const int num_patch_match_src_images,
+                                     const CopyType copy_type,
+                                     const std::vector<image_t>& image_ids)
     : options_(options),
       image_path_(image_path),
       output_path_(output_path),
-      reconstruction_(reconstruction) {}
+      copy_type_(copy_type),
+      num_patch_match_src_images_(num_patch_match_src_images),
+      reconstruction_(reconstruction),
+      image_ids_(image_ids) {}
 
 void COLMAPUndistorter::Run() {
   PrintHeading1("Image undistortion");
@@ -174,13 +182,24 @@ void COLMAPUndistorter::Run() {
       JoinPaths(output_path_, "stereo/consistency_graphs"));
 
   ThreadPool thread_pool;
-  std::vector<std::future<void>> futures;
+  std::vector<std::future<bool>> futures;
   futures.reserve(reconstruction_.NumRegImages());
-  for (size_t i = 0; i < reconstruction_.NumRegImages(); ++i) {
-    futures.push_back(
-        thread_pool.AddTask(&COLMAPUndistorter::Undistort, this, i));
+  if (image_ids_.empty()) {
+    for (size_t i = 0; i < reconstruction_.NumRegImages(); ++i) {
+      const image_t image_id = reconstruction_.RegImageIds().at(i);
+      futures.push_back(
+          thread_pool.AddTask(&COLMAPUndistorter::Undistort, this, image_id));
+    }
+  } else {
+    for (const image_t image_id : image_ids_) {
+      futures.push_back(
+          thread_pool.AddTask(&COLMAPUndistorter::Undistort, this, image_id));
+    }
   }
 
+  // Only use the image names for the successfully undistorted images
+  // when writing the MVS config files
+  image_names_.clear();
   for (size_t i = 0; i < futures.size(); ++i) {
     if (IsStopped()) {
       break;
@@ -190,7 +209,14 @@ void COLMAPUndistorter::Run() {
                               futures.size())
               << std::endl;
 
-    futures[i].get();
+    if (futures[i].get()) {
+      if (image_ids_.empty()) {
+        const image_t image_id = reconstruction_.RegImageIds().at(i);
+        image_names_.push_back(reconstruction_.Image(image_id).Name());
+      } else {
+        image_names_.push_back(reconstruction_.Image(image_ids_[i]).Name());
+      }
+    }
   }
 
   std::cout << "Writing reconstruction..." << std::endl;
@@ -209,38 +235,46 @@ void COLMAPUndistorter::Run() {
   GetTimer().PrintMinutes();
 }
 
-void COLMAPUndistorter::Undistort(const size_t reg_image_idx) const {
-  const image_t image_id = reconstruction_.RegImageIds().at(reg_image_idx);
+bool COLMAPUndistorter::Undistort(const image_t image_id) const {
   const Image& image = reconstruction_.Image(image_id);
-  const Camera& camera = reconstruction_.Camera(image.CameraId());
 
+  Bitmap distorted_bitmap;
+  Bitmap undistorted_bitmap;
+  const Camera& camera = reconstruction_.Camera(image.CameraId());
+  Camera undistorted_camera;
+
+  const std::string input_image_path = JoinPaths(image_path_, image.Name());
   const std::string output_image_path =
       JoinPaths(output_path_, "images", image.Name());
 
-  Bitmap distorted_bitmap;
-  const std::string input_image_path = JoinPaths(image_path_, image.Name());
+  // Check if the image is already undistorted and copy from source if no
+  // scaling is needed
+  if (camera.IsUndistorted() && options_.max_image_size < 0 &&
+      ExistsFile(input_image_path)) {
+    std::cout << "Undistorted image found; copying to location: "
+              << output_image_path << std::endl;
+    FileCopy(input_image_path, output_image_path, copy_type_);
+    return true;
+  }
+
   if (!distorted_bitmap.Read(input_image_path)) {
     std::cerr << "ERROR: Cannot read image at path " << input_image_path
               << std::endl;
-    return;
+    return false;
   }
 
-  Bitmap undistorted_bitmap;
-  Camera undistorted_camera;
   UndistortImage(options_, distorted_bitmap, camera, &undistorted_bitmap,
                  &undistorted_camera);
-
-  undistorted_bitmap.Write(output_image_path);
+  return undistorted_bitmap.Write(output_image_path);
 }
 
 void COLMAPUndistorter::WritePatchMatchConfig() const {
   const auto path = JoinPaths(output_path_, "stereo/patch-match.cfg");
   std::ofstream file(path, std::ios::trunc);
   CHECK(file.is_open()) << path;
-  for (const auto image_id : reconstruction_.RegImageIds()) {
-    const auto& image = reconstruction_.Image(image_id);
-    file << image.Name() << std::endl;
-    file << "__auto__, 20" << std::endl;
+  for (const auto& image_name : image_names_) {
+    file << image_name << std::endl;
+    file << "__auto__, " << num_patch_match_src_images_ << std::endl;
   }
 }
 
@@ -248,9 +282,8 @@ void COLMAPUndistorter::WriteFusionConfig() const {
   const auto path = JoinPaths(output_path_, "stereo/fusion.cfg");
   std::ofstream file(path, std::ios::trunc);
   CHECK(file.is_open()) << path;
-  for (const auto image_id : reconstruction_.RegImageIds()) {
-    const auto& image = reconstruction_.Image(image_id);
-    file << image.Name() << std::endl;
+  for (const auto& image_name : image_names_) {
+    file << image_name << std::endl;
   }
 }
 
@@ -284,7 +317,7 @@ void PMVSUndistorter::Run() {
   CreateDirIfNotExists(JoinPaths(output_path_, "pmvs/models"));
 
   ThreadPool thread_pool;
-  std::vector<std::future<void>> futures;
+  std::vector<std::future<bool>> futures;
   futures.reserve(reconstruction_.NumRegImages());
   for (size_t i = 0; i < reconstruction_.NumRegImages(); ++i) {
     futures.push_back(
@@ -332,7 +365,7 @@ void PMVSUndistorter::Run() {
   GetTimer().PrintMinutes();
 }
 
-void PMVSUndistorter::Undistort(const size_t reg_image_idx) const {
+bool PMVSUndistorter::Undistort(const size_t reg_image_idx) const {
   const std::string output_image_path = JoinPaths(
       output_path_, StringPrintf("pmvs/visualize/%08d.jpg", reg_image_idx));
   const std::string proj_matrix_path =
@@ -348,7 +381,7 @@ void PMVSUndistorter::Undistort(const size_t reg_image_idx) const {
     std::cerr << StringPrintf("ERROR: Cannot read image at path %s",
                               input_image_path.c_str())
               << std::endl;
-    return;
+    return false;
   }
 
   Bitmap undistorted_bitmap;
@@ -356,8 +389,8 @@ void PMVSUndistorter::Undistort(const size_t reg_image_idx) const {
   UndistortImage(options_, distorted_bitmap, camera, &undistorted_bitmap,
                  &undistorted_camera);
 
-  undistorted_bitmap.Write(output_image_path);
   WriteProjectionMatrix(proj_matrix_path, undistorted_camera, image, "CONTOUR");
+  return undistorted_bitmap.Write(output_image_path);
 }
 
 void PMVSUndistorter::WriteVisibilityData() const {
@@ -379,7 +412,7 @@ void PMVSUndistorter::WriteVisibilityData() const {
       const Point2D& point2D = image.Point2D(point2D_idx);
       if (point2D.HasPoint3D()) {
         const Point3D& point3D = reconstruction_.Point3D(point2D.Point3DId());
-        for (const TrackElement track_el : point3D.Track().Elements()) {
+        for (const TrackElement& track_el : point3D.Track().Elements()) {
           if (track_el.image_id != image_id) {
             visible_image_ids.insert(track_el.image_id);
           }
@@ -511,7 +544,7 @@ void CMPMVSUndistorter::Run() {
   PrintHeading1("Image undistortion (CMP-MVS)");
 
   ThreadPool thread_pool;
-  std::vector<std::future<void>> futures;
+  std::vector<std::future<bool>> futures;
   futures.reserve(reconstruction_.NumRegImages());
   for (size_t i = 0; i < reconstruction_.NumRegImages(); ++i) {
     futures.push_back(
@@ -533,7 +566,7 @@ void CMPMVSUndistorter::Run() {
   GetTimer().PrintMinutes();
 }
 
-void CMPMVSUndistorter::Undistort(const size_t reg_image_idx) const {
+bool CMPMVSUndistorter::Undistort(const size_t reg_image_idx) const {
   const std::string output_image_path =
       JoinPaths(output_path_, StringPrintf("%05d.jpg", reg_image_idx + 1));
   const std::string proj_matrix_path =
@@ -548,7 +581,7 @@ void CMPMVSUndistorter::Undistort(const size_t reg_image_idx) const {
   if (!distorted_bitmap.Read(input_image_path)) {
     std::cerr << "ERROR: Cannot read image at path " << input_image_path
               << std::endl;
-    return;
+    return false;
   }
 
   Bitmap undistorted_bitmap;
@@ -556,8 +589,68 @@ void CMPMVSUndistorter::Undistort(const size_t reg_image_idx) const {
   UndistortImage(options_, distorted_bitmap, camera, &undistorted_bitmap,
                  &undistorted_camera);
 
-  undistorted_bitmap.Write(output_image_path);
   WriteProjectionMatrix(proj_matrix_path, undistorted_camera, image, "CONTOUR");
+  return undistorted_bitmap.Write(output_image_path);
+}
+
+PureImageUndistorter::PureImageUndistorter(
+    const UndistortCameraOptions& options, const std::string& image_path,
+    const std::string& output_path,
+    const std::vector<std::pair<std::string, Camera>>& image_names_and_cameras)
+    : options_(options),
+      image_path_(image_path),
+      output_path_(output_path),
+      image_names_and_cameras_(image_names_and_cameras) {}
+
+void PureImageUndistorter::Run() {
+  PrintHeading1("Image undistortion");
+
+  CreateDirIfNotExists(output_path_);
+
+  ThreadPool thread_pool;
+  std::vector<std::future<bool>> futures;
+  size_t num_images = image_names_and_cameras_.size();
+  futures.reserve(num_images);
+  for (size_t i = 0; i < num_images; ++i) {
+    futures.push_back(
+        thread_pool.AddTask(&PureImageUndistorter::Undistort, this, i));
+  }
+
+  for (size_t i = 0; i < futures.size(); ++i) {
+    if (IsStopped()) {
+      break;
+    }
+
+    std::cout << StringPrintf("Undistorting image [%d/%d]", i + 1,
+                              futures.size())
+              << std::endl;
+
+    futures[i].get();
+  }
+
+  GetTimer().PrintMinutes();
+}
+
+bool PureImageUndistorter::Undistort(const size_t image_idx) const {
+  const std::string& image_name = image_names_and_cameras_[image_idx].first;
+  const Camera& camera = image_names_and_cameras_[image_idx].second;
+
+  const std::string output_image_path = JoinPaths(output_path_, image_name);
+
+  Bitmap distorted_bitmap;
+  const std::string input_image_path = JoinPaths(image_path_, image_name);
+  if (!distorted_bitmap.Read(input_image_path)) {
+    std::cerr << "ERROR: Cannot read image at path " << input_image_path
+              << std::endl;
+    return false;
+  }
+
+  Bitmap undistorted_bitmap;
+  Camera undistorted_camera;
+  UndistortImage(options_, distorted_bitmap, camera, &undistorted_bitmap,
+                 &undistorted_camera);
+    
+  return undistorted_bitmap.Write(output_image_path);
 }
 
 StereoImageRectifier::StereoImageRectifier(
