@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "controllers/incremental_mapper.h"
 
@@ -48,28 +48,29 @@ size_t TriangulateImage(const IncrementalMapperOptions& options,
 
 void AdjustGlobalBundle(const IncrementalMapperOptions& options,
                         IncrementalMapper* mapper) {
-  BundleAdjustmentOptions custom_options = options.GlobalBundleAdjustment();
+  BundleAdjustmentOptions custom_ba_options = options.GlobalBundleAdjustment();
 
   const size_t num_reg_images = mapper->GetReconstruction().NumRegImages();
 
   // Use stricter convergence criteria for first registered images.
-  const size_t kMinNumRegImages = 10;
-  if (num_reg_images < kMinNumRegImages) {
-    custom_options.solver_options.function_tolerance /= 10;
-    custom_options.solver_options.gradient_tolerance /= 10;
-    custom_options.solver_options.parameter_tolerance /= 10;
-    custom_options.solver_options.max_num_iterations *= 2;
-    custom_options.solver_options.max_linear_solver_iterations = 200;
+  const size_t kMinNumRegImagesForFastBA = 10;
+  if (num_reg_images < kMinNumRegImagesForFastBA) {
+    custom_ba_options.solver_options.function_tolerance /= 10;
+    custom_ba_options.solver_options.gradient_tolerance /= 10;
+    custom_ba_options.solver_options.parameter_tolerance /= 10;
+    custom_ba_options.solver_options.max_num_iterations *= 2;
+    custom_ba_options.solver_options.max_linear_solver_iterations = 200;
   }
 
   PrintHeading1("Global bundle adjustment");
-  if (options.ba_global_use_pba && num_reg_images >= kMinNumRegImages &&
-      ParallelBundleAdjuster::IsSupported(custom_options,
+  if (options.ba_global_use_pba && !options.fix_existing_images &&
+      num_reg_images >= kMinNumRegImagesForFastBA &&
+      ParallelBundleAdjuster::IsSupported(custom_ba_options,
                                           mapper->GetReconstruction())) {
     mapper->AdjustParallelGlobalBundle(
-        custom_options, options.ParallelGlobalBundleAdjustment());
+        custom_ba_options, options.ParallelGlobalBundleAdjustment());
   } else {
-    mapper->AdjustGlobalBundle(custom_options);
+    mapper->AdjustGlobalBundle(options.Mapper(), custom_ba_options);
   }
 }
 
@@ -177,11 +178,11 @@ size_t CompleteAndMergeTracks(const IncrementalMapperOptions& options,
                               IncrementalMapper* mapper) {
   const size_t num_completed_observations =
       mapper->CompleteTracks(options.Triangulation());
-  std::cout << "  => Merged observations: " << num_completed_observations
+  std::cout << "  => Completed observations: " << num_completed_observations
             << std::endl;
   const size_t num_merged_observations =
       mapper->MergeTracks(options.Triangulation());
-  std::cout << "  => Completed observations: " << num_merged_observations
+  std::cout << "  => Merged observations: " << num_merged_observations
             << std::endl;
   return num_completed_observations + num_merged_observations;
 }
@@ -194,6 +195,8 @@ IncrementalMapper::Options IncrementalMapperOptions::Mapper() const {
   options.max_focal_length_ratio = max_focal_length_ratio;
   options.max_extra_param = max_extra_param;
   options.num_threads = num_threads;
+  options.local_ba_num_images = ba_local_num_images;
+  options.fix_existing_images = fix_existing_images;
   return options;
 }
 
@@ -209,7 +212,7 @@ IncrementalTriangulator::Options IncrementalMapperOptions::Triangulation()
 BundleAdjustmentOptions IncrementalMapperOptions::LocalBundleAdjustment()
     const {
   BundleAdjustmentOptions options;
-  options.solver_options.function_tolerance = 0.0;
+  options.solver_options.function_tolerance = ba_local_function_tolerance;
   options.solver_options.gradient_tolerance = 10.0;
   options.solver_options.parameter_tolerance = 0.0;
   options.solver_options.max_num_iterations = ba_local_max_num_iterations;
@@ -223,16 +226,18 @@ BundleAdjustmentOptions IncrementalMapperOptions::LocalBundleAdjustment()
   options.refine_focal_length = ba_refine_focal_length;
   options.refine_principal_point = ba_refine_principal_point;
   options.refine_extra_params = ba_refine_extra_params;
+  options.min_num_residuals_for_multi_threading =
+      ba_min_num_residuals_for_multi_threading;
   options.loss_function_scale = 1.0;
   options.loss_function_type =
-      BundleAdjustmentOptions::LossFunctionType::CAUCHY;
+      BundleAdjustmentOptions::LossFunctionType::SOFT_L1;
   return options;
 }
 
 BundleAdjustmentOptions IncrementalMapperOptions::GlobalBundleAdjustment()
     const {
   BundleAdjustmentOptions options;
-  options.solver_options.function_tolerance = 0.0;
+  options.solver_options.function_tolerance = ba_global_function_tolerance;
   options.solver_options.gradient_tolerance = 1.0;
   options.solver_options.parameter_tolerance = 0.0;
   options.solver_options.max_num_iterations = ba_global_max_num_iterations;
@@ -246,6 +251,8 @@ BundleAdjustmentOptions IncrementalMapperOptions::GlobalBundleAdjustment()
   options.refine_focal_length = ba_refine_focal_length;
   options.refine_principal_point = ba_refine_principal_point;
   options.refine_extra_params = ba_refine_extra_params;
+  options.min_num_residuals_for_multi_threading =
+      ba_min_num_residuals_for_multi_threading;
   options.loss_function_type =
       BundleAdjustmentOptions::LossFunctionType::TRIVIAL;
   return options;
@@ -258,6 +265,8 @@ IncrementalMapperOptions::ParallelGlobalBundleAdjustment() const {
   options.print_summary = true;
   options.gpu_index = ba_global_pba_gpu_index;
   options.num_threads = num_threads;
+  options.min_num_residuals_for_multi_threading =
+      ba_min_num_residuals_for_multi_threading;
   return options;
 }
 
@@ -335,12 +344,23 @@ void IncrementalMapperController::Run() {
 bool IncrementalMapperController::LoadDatabase() {
   PrintHeading1("Loading database");
 
+  // Make sure images of the given reconstruction are also included when
+  // manually specifying images for the reconstrunstruction procedure.
+  std::unordered_set<std::string> image_names = options_->image_names;
+  if (reconstruction_manager_->Size() == 1 && !options_->image_names.empty()) {
+    const Reconstruction& reconstruction = reconstruction_manager_->Get(0);
+    for (const image_t image_id : reconstruction.RegImageIds()) {
+      const auto& image = reconstruction.Image(image_id);
+      image_names.insert(image.Name());
+    }
+  }
+
   Database database(database_path_);
   Timer timer;
   timer.Start();
   const size_t min_num_matches = static_cast<size_t>(options_->min_num_matches);
   database_cache_.Load(database, min_num_matches, options_->ignore_watermarks,
-                       options_->image_names);
+                       image_names);
   std::cout << std::endl;
   timer.PrintMinutes();
 
@@ -402,6 +422,7 @@ void IncrementalMapperController::Reconstruct(
 
       // Try to find good initial pair.
       if (options_->init_image_id1 == -1 || options_->init_image_id2 == -1) {
+        PrintHeading1("Finding good initial image pair");
         const bool find_init_success = mapper.FindInitialImagePair(
             init_mapper_options, &image_id1, &image_id2);
         if (!find_init_success) {
