@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "estimators/generalized_absolute_pose.h"
 
@@ -195,9 +195,9 @@ std::vector<Eigen::Vector3d> ComputeDepthsSylvester(
           const double kMaxLambdaRatio = 1e-2;
           if (std::abs(lambda_1_1 - lambda_1_2) <
               kMaxLambdaRatio * std::max(lambda_1_1, lambda_1_2)) {
-              const double lambda_1 = (lambda_1_1 + lambda_1_2) / 2;
-              depths.emplace_back(lambda_1, lambda_2, lambda_3);
-            }
+            const double lambda_1 = (lambda_1_1 + lambda_1_2) / 2;
+            depths.emplace_back(lambda_1, lambda_2, lambda_3);
+          }
         }
       }
     }
@@ -314,10 +314,21 @@ void GP3PEstimator::Residuals(const std::vector<X_t>& points2D,
 
       const double x_0 = points2D[i].xy(0);
       const double x_1 = points2D[i].xy(1);
-      const double inv_x_norm = 1 / std::sqrt(x_0 * x_0 + x_1 * x_1 + 1);
 
-      (*residuals)[i] =
-          1 - inv_pcx_norm * inv_x_norm * (pcx_0 * x_0 + pcx_1 * x_1 + pcx_2);
+      if (residual_type == ResidualType::CosineDistance) {
+        const double inv_x_norm = 1 / std::sqrt(x_0 * x_0 + x_1 * x_1 + 1);
+        const double cosine_dist =
+            1 - inv_pcx_norm * inv_x_norm * (pcx_0 * x_0 + pcx_1 * x_1 + pcx_2);
+        (*residuals)[i] = cosine_dist * cosine_dist;
+      } else if (residual_type == ResidualType::ReprojectionError) {
+        const double inv_pcx_2 = 1.0 / pcx_2;
+        const double dx_0 = x_0 - pcx_0 * inv_pcx_2;
+        const double dx_1 = x_1 - pcx_1 * inv_pcx_2;
+        const double reproj_error = dx_0 * dx_0 + dx_1 * dx_1;
+        (*residuals)[i] = reproj_error;
+      } else {
+         LOG(FATAL) << "Invalid residual type";
+      }
     } else {
       (*residuals)[i] = std::numeric_limits<double>::max();
     }
