@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_ESTIMATORS_GENERALIZED_ABSOLUTE_POSE_H_
 #define COLMAP_SRC_ESTIMATORS_GENERALIZED_ABSOLUTE_POSE_H_
@@ -69,16 +69,28 @@ class GP3PEstimator {
   // The minimum number of samples needed to estimate a model.
   static const int kMinNumSamples = 3;
 
+  enum class ResidualType {
+    CosineDistance,
+    ReprojectionError,
+  };
+
+  // Whether to compute the cosine similarity or the reprojection error.
+  // [WARNING] The reprojection error being in normalized coordinates,
+  // the unique error threshold of RANSAC corresponds to different pixel values
+  // in the different cameras of the rig if they have different intrinsics.
+  ResidualType residual_type = ResidualType::CosineDistance;
+
   // Estimate the most probable solution of the GP3P problem from a set of
   // three 2D-3D point correspondences.
   static std::vector<M_t> Estimate(const std::vector<X_t>& points2D,
                                    const std::vector<Y_t>& points3D);
 
-  // Calculate the cosine distance error between the rays given a set of 2D-3D
-  // point correspondences and a projection matrix of the generalized camera.
-  static void Residuals(const std::vector<X_t>& points2D,
-                        const std::vector<Y_t>& points3D,
-                        const M_t& proj_matrix, std::vector<double>* residuals);
+  // Calculate the squared cosine distance error between the rays given a set of
+  // 2D-3D point correspondences and a projection matrix of the generalized
+  // camera.
+  void Residuals(const std::vector<X_t>& points2D,
+                 const std::vector<Y_t>& points3D,
+                 const M_t& proj_matrix, std::vector<double>* residuals);
 };
 
 }  // namespace colmap
