@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_ESTIMATORS_GENERALIZED_RELATIVE_POSE_H_
 #define COLMAP_SRC_ESTIMATORS_GENERALIZED_RELATIVE_POSE_H_
@@ -83,8 +83,8 @@ class GR6PEstimator {
 
   // Calculate the squared Sampson error between corresponding points.
   static void Residuals(const std::vector<X_t>& points1,
-                        const std::vector<Y_t>& points2,
-                        const M_t& proj_matrix, std::vector<double>* residuals);
+                        const std::vector<Y_t>& points2, const M_t& proj_matrix,
+                        std::vector<double>* residuals);
 };
 
 }  // namespace colmap
