@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_FEATURE_SIFT_H_
 #define COLMAP_SRC_FEATURE_SIFT_H_
@@ -144,7 +144,7 @@ struct SiftMatchingOptions {
 
   // Minimum/maximum number of RANSAC iterations. Note that this option
   // overrules the min_inlier_ratio option.
-  int min_num_trials = 30;
+  int min_num_trials = 100;
   int max_num_trials = 10000;
 
   // A priori assumed minimum inlier ratio, which determines the maximum
@@ -211,6 +211,14 @@ void LoadSiftFeaturesFromTextFile(const std::string& path,
                                   FeatureDescriptors* descriptors);
 
 // Match the given SIFT features on the CPU.
+void MatchSiftFeaturesCPUBruteForce(const SiftMatchingOptions& match_options,
+                                    const FeatureDescriptors& descriptors1,
+                                    const FeatureDescriptors& descriptors2,
+                                    FeatureMatches* matches);
+void MatchSiftFeaturesCPUFLANN(const SiftMatchingOptions& match_options,
+                               const FeatureDescriptors& descriptors1,
+                               const FeatureDescriptors& descriptors2,
+                               FeatureMatches* matches);
 void MatchSiftFeaturesCPU(const SiftMatchingOptions& match_options,
                           const FeatureDescriptors& descriptors1,
                           const FeatureDescriptors& descriptors2,
