@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "feature/utils.h"
 
@@ -95,8 +95,7 @@ void ExtractTopScaleFeatures(FeatureKeypoints* keypoints,
     scales.emplace_back(i, (*keypoints)[i].ComputeScale());
   }
 
-  std::partial_sort(scales.begin(), scales.begin() + num_features,
-                    scales.end(),
+  std::partial_sort(scales.begin(), scales.begin() + num_features, scales.end(),
                     [](const std::pair<size_t, float> scale1,
                        const std::pair<size_t, float> scale2) {
                       return scale1.second > scale2.second;
