@@ -27,11 +27,12 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_MVS_FUSION_H_
 #define COLMAP_SRC_MVS_FUSION_H_
 
+#include <cfloat>
 #include <unordered_set>
 #include <vector>
 
@@ -53,6 +54,12 @@ namespace colmap {
 namespace mvs {
 
 struct StereoFusionOptions {
+  // Path for PNG masks. Same format expected as ImageReaderOptions.
+  std::string mask_path = "";
+
+  // The number of threads to use during fusion.
+  int num_threads = -1;
+
   // Maximum image size in either dimension.
   int max_image_size = -1;
 
@@ -77,6 +84,9 @@ struct StereoFusionOptions {
   // Number of overlapping images to transitively check for fusing points.
   int check_num_images = 50;
 
+  // Flag indicating whether to use LRU cache or pre-load all data
+  bool use_cache = false;
+
   // Cache size in gigabytes for fusion. The fusion keeps the bitmaps, depth
   // maps, normal maps, and consistency graphs of this number of images in
   // memory. A higher value leads to less disk access and faster fusion, while
@@ -84,6 +94,10 @@ struct StereoFusionOptions {
   // consume a lot of memory, if the consistency graph is dense.
   double cache_size = 32.0;
 
+  std::pair<Eigen::Vector3f, Eigen::Vector3f> bounding_box =
+      std::make_pair(Eigen::Vector3f(-FLT_MAX, -FLT_MAX, -FLT_MAX),
+                     Eigen::Vector3f(FLT_MAX, FLT_MAX, FLT_MAX));
+
   // Check the options for validity.
   bool Check() const;
 
@@ -106,7 +120,9 @@ class StereoFusion : public Thread {
 
  private:
   void Run();
-  void Fuse();
+  void InitFusedPixelMask(int image_idx, size_t width, size_t height);
+  void Fuse(const int thread_id, const int image_idx, const int row,
+            const int col);
 
   const StereoFusionOptions options_;
   const std::string workspace_path_;
@@ -120,7 +136,9 @@ class StereoFusion : public Thread {
   std::vector<char> used_images_;
   std::vector<char> fused_images_;
   std::vector<std::vector<int>> overlapping_images_;
-  std::vector<Mat<bool>> fused_pixel_masks_;
+  // Contains image masks of pre-masked and already fused pixels.
+  // Initialized from image masks if provided in StereoFusionOptions.
+  std::vector<Mat<char>> fused_pixel_masks_;
   std::vector<std::pair<int, int>> depth_map_sizes_;
   std::vector<std::pair<float, float>> bitmap_scales_;
   std::vector<Eigen::Matrix<float, 3, 4, Eigen::RowMajor>> P_;
@@ -132,29 +150,22 @@ class StereoFusion : public Thread {
     int row = 0;
     int col = 0;
     int traversal_depth = -1;
+    FusionData(int image_idx, int row, int col, int traversal_depth)
+        : image_idx(image_idx),
+          row(row),
+          col(col),
+          traversal_depth(traversal_depth) {}
     bool operator()(const FusionData& data1, const FusionData& data2) {
       return data1.image_idx > data2.image_idx;
     }
   };
 
-  // Next points to fuse.
-  std::vector<FusionData> fusion_queue_;
-
   // Already fused points.
   std::vector<PlyPoint> fused_points_;
   std::vector<std::vector<int>> fused_points_visibility_;
 
-  // Points of different pixels of the currently point to be fused.
-  std::vector<float> fused_point_x_;
-  std::vector<float> fused_point_y_;
-  std::vector<float> fused_point_z_;
-  std::vector<float> fused_point_nx_;
-  std::vector<float> fused_point_ny_;
-  std::vector<float> fused_point_nz_;
-  std::vector<uint8_t> fused_point_r_;
-  std::vector<uint8_t> fused_point_g_;
-  std::vector<uint8_t> fused_point_b_;
-  std::unordered_set<int> fused_point_visibility_;
+  std::vector<std::vector<PlyPoint>> task_fused_points_;
+  std::vector<std::vector<std::vector<int>>> task_fused_points_visibility_;
 };
 
 // Write the visiblity information into a binary file of the following format:
