@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_MVS_MODEL_H_
 #define COLMAP_SRC_MVS_MODEL_H_
@@ -58,7 +58,9 @@ struct Model {
 
   // Read the model from different data formats.
   void Read(const std::string& path, const std::string& format);
-  void ReadFromCOLMAP(const std::string& path);
+  void ReadFromCOLMAP(const std::string& path,
+                      const std::string& sparse_path = "sparse",
+                      const std::string& images_path = "images");
   void ReadFromPMVS(const std::string& path);
 
   // Get the image index for the given image name.
