@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "mvs/patch_match.h"
 
@@ -74,6 +74,7 @@ void PatchMatchOptions::Print() const {
   PrintOption(filter_min_num_consistent);
   PrintOption(filter_geom_consistency_max_cost);
   PrintOption(write_consistency_graph);
+  PrintOption(allow_missing_files);
 }
 
 void PatchMatch::Problem::Print() const {
@@ -183,11 +184,13 @@ ConsistencyGraph PatchMatch::GetConsistencyGraph() const {
 PatchMatchController::PatchMatchController(const PatchMatchOptions& options,
                                            const std::string& workspace_path,
                                            const std::string& workspace_format,
-                                           const std::string& pmvs_option_name)
+                                           const std::string& pmvs_option_name,
+                                           const std::string& config_path)
     : options_(options),
       workspace_path_(workspace_path),
       workspace_format_(workspace_format),
-      pmvs_option_name_(pmvs_option_name) {
+      pmvs_option_name_(pmvs_option_name),
+      config_path_(config_path) {
   std::vector<int> gpu_indices = CSVToVector<int>(options_.gpu_index);
 }
 
@@ -243,7 +246,7 @@ void PatchMatchController::ReadWorkspace() {
   workspace_options.workspace_format = workspace_format_;
   workspace_options.input_type = options_.geom_consistency ? "photometric" : "";
 
-  workspace_.reset(new Workspace(workspace_options));
+  workspace_.reset(new CachedWorkspace(workspace_options));
 
   if (workspace_format_lower_case == "pmvs") {
     std::cout << StringPrintf("Importing PMVS workspace (option %s)...",
@@ -262,9 +265,12 @@ void PatchMatchController::ReadProblems() {
 
   const auto& model = workspace_->GetModel();
 
-  std::vector<std::string> config = ReadTextFileLines(
-      JoinPaths(workspace_path_, workspace_->GetOptions().stereo_folder,
-                "patch-match.cfg"));
+  const std::string config_path =
+      config_path_.empty()
+          ? JoinPaths(workspace_path_, workspace_->GetOptions().stereo_folder,
+                      "patch-match.cfg")
+          : config_path_;
+  std::vector<std::string> config = ReadTextFileLines(config_path);
 
   std::vector<std::map<int, int>> shared_num_points;
   std::vector<std::map<int, float>> triangulation_angles;
@@ -314,8 +320,8 @@ void PatchMatchController::ReadProblems() {
       // Use all images as source images.
       problem.src_image_idxs.clear();
       problem.src_image_idxs.reserve(model.images.size() - 1);
-      for (const int image_idx : ref_image_idxs) {
-        if (image_idx != problem.ref_image_idx) {
+      for (size_t image_idx = 0; image_idx < model.images.size(); ++image_idx) {
+        if (static_cast<int>(image_idx) != problem.ref_image_idx) {
           problem.src_image_idxs.push_back(image_idx);
         }
       }
@@ -346,9 +352,8 @@ void PatchMatchController::ReadProblems() {
       std::vector<std::pair<int, int>> src_images;
       src_images.reserve(overlapping_images.size());
       for (const auto& image : overlapping_images) {
-        if (ref_image_idxs.count(image.first) &&
-            overlapping_triangulation_angles.at(image.first) >=
-                min_triangulation_angle_rad) {
+        if (overlapping_triangulation_angles.at(image.first) >=
+            min_triangulation_angle_rad) {
           src_images.emplace_back(image.first, image.second);
         }
       }
@@ -433,8 +438,8 @@ void PatchMatchController::ProcessProblem(const PatchMatchOptions& options,
     return;
   }
 
-  PrintHeading1(StringPrintf("Processing view %d / %d", problem_idx + 1,
-                             problems_.size()));
+  PrintHeading1(StringPrintf("Processing view %d / %d for %s", problem_idx + 1,
+                             problems_.size(), image_name.c_str()));
 
   auto patch_match_options = options;
 
@@ -482,13 +487,41 @@ void PatchMatchController::ProcessProblem(const PatchMatchOptions& options,
     std::unique_lock<std::mutex> lock(workspace_mutex_);
 
     std::cout << "Reading inputs..." << std::endl;
+    std::vector<int> src_image_idxs;
     for (const auto image_idx : used_image_idxs) {
+      std::string image_path = workspace_->GetBitmapPath(image_idx);
+      std::string depth_path = workspace_->GetDepthMapPath(image_idx);
+      std::string normal_path = workspace_->GetNormalMapPath(image_idx);
+
+      if (!ExistsFile(image_path) ||
+          (options.geom_consistency && !ExistsFile(depth_path)) ||
+          (options.geom_consistency && !ExistsFile(normal_path))) {
+        if (options.allow_missing_files) {
+          std::cout << StringPrintf(
+                           "WARN: Skipping source image %d: %s for missing "
+                           "image or depth/normal map",
+                           image_idx, model.GetImageName(image_idx).c_str())
+                    << std::endl;
+          continue;
+        } else {
+          std::cout
+              << StringPrintf(
+                     "ERROR: Missing image or map dependency for image %d: %s",
+                     image_idx, model.GetImageName(image_idx).c_str())
+              << std::endl;
+        }
+      }
+
+      if (image_idx != problem.ref_image_idx) {
+        src_image_idxs.push_back(image_idx);
+      }
       images.at(image_idx).SetBitmap(workspace_->GetBitmap(image_idx));
       if (options.geom_consistency) {
         depth_maps.at(image_idx) = workspace_->GetDepthMap(image_idx);
         normal_maps.at(image_idx) = workspace_->GetNormalMap(image_idx);
       }
     }
+    problem.src_image_idxs = src_image_idxs;
   }
 
   problem.Print();
