@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_MVS_PATCH_MATCH_H_
 #define COLMAP_SRC_MVS_PATCH_MATCH_H_
@@ -134,6 +134,9 @@ struct PatchMatchOptions {
   // of memory, if the consistency graph is dense.
   double cache_size = 32.0;
 
+  // Whether to tolerate missing images/maps in the problem setup
+  bool allow_missing_files = false;
+
   // Whether to write the consistency graph.
   bool write_consistency_graph = false;
 
@@ -250,7 +253,8 @@ class PatchMatchController : public Thread {
   PatchMatchController(const PatchMatchOptions& options,
                        const std::string& workspace_path,
                        const std::string& workspace_format,
-                       const std::string& pmvs_option_name);
+                       const std::string& pmvs_option_name,
+                       const std::string& config_path = "");
 
  private:
   void Run();
@@ -264,6 +268,7 @@ class PatchMatchController : public Thread {
   const std::string workspace_path_;
   const std::string workspace_format_;
   const std::string pmvs_option_name_;
+  const std::string config_path_;
 
   std::unique_ptr<ThreadPool> thread_pool_;
   std::mutex workspace_mutex_;
