@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "optim/sprt.h"
 
@@ -45,13 +45,13 @@ void SPRT::Update(const Options& options) {
 bool SPRT::Evaluate(const std::vector<double>& residuals,
                     const double max_residual, size_t* num_inliers,
                     size_t* num_eval_samples) {
-  num_inliers = 0;
+  *num_inliers = 0;
 
   double likelihood_ratio = 1;
 
   for (size_t i = 0; i < residuals.size(); ++i) {
     if (std::abs(residuals[i]) <= max_residual) {
-      num_inliers += 1;
+      *num_inliers += 1;
       likelihood_ratio *= delta_epsilon_;
     } else {
       likelihood_ratio *= delta_1_epsilon_1_;
