@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "ui/line_painter.h"
 
@@ -82,14 +82,14 @@ void LinePainter::Upload(const std::vector<LinePainter::Data>& data) {
                 static_cast<int>(data.size() * sizeof(LinePainter::Data)));
 
   // in_position
-  shader_program_.enableAttributeArray(0);
-  shader_program_.setAttributeBuffer(0, GL_FLOAT, 0, 3,
+  shader_program_.enableAttributeArray("a_pos");
+  shader_program_.setAttributeBuffer("a_pos", GL_FLOAT, 0, 3,
                                      sizeof(PointPainter::Data));
 
   // in_color
-  shader_program_.enableAttributeArray(1);
-  shader_program_.setAttributeBuffer(1, GL_FLOAT, 3 * sizeof(GLfloat), 4,
-                                     sizeof(PointPainter::Data));
+  shader_program_.enableAttributeArray("a_color");
+  shader_program_.setAttributeBuffer("a_color", GL_FLOAT, 3 * sizeof(GLfloat),
+                                     4, sizeof(PointPainter::Data));
 
   // Make sure they are not changed from the outside
   vbo_.release();
@@ -114,7 +114,8 @@ void LinePainter::Render(const QMatrix4x4& pmv_matrix, const int width,
                                   QVector2D(1.0f / width, 1.0f / height));
   shader_program_.setUniformValue("u_line_width", line_width);
 
-  glDrawArrays(GL_LINES, 0, (GLsizei)(2 * num_geoms_));
+  QOpenGLFunctions* gl_funcs = QOpenGLContext::currentContext()->functions();
+  gl_funcs->glDrawArrays(GL_LINES, 0, (GLsizei)(2 * num_geoms_));
 
   // Make sure the VAO is not changed from the outside
   vao_.release();
