@@ -27,7 +27,9 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
+
+#include <clocale>
 
 #include "ui/main_window.h"
 
@@ -39,6 +41,7 @@ MainWindow::MainWindow(const OptionManager& options)
     : options_(options),
       thread_control_widget_(new ThreadControlWidget(this)),
       window_closed_(false) {
+  std::setlocale(LC_NUMERIC, "C");
   resize(1024, 600);
   UpdateWindowTitle();
 
@@ -54,8 +57,11 @@ MainWindow::MainWindow(const OptionManager& options)
   options_.AddAllOptions();
 }
 
-const ReconstructionManager& MainWindow::GetReconstructionManager() const {
-  return reconstruction_manager_;
+void MainWindow::ImportReconstruction(const std::string& path) {
+  const size_t idx = reconstruction_manager_.Read(path);
+  reconstruction_manager_widget_->Update();
+  reconstruction_manager_widget_->SelectReconstruction(idx);
+  RenderNow();
 }
 
 void MainWindow::closeEvent(QCloseEvent* event) {
@@ -1241,11 +1247,11 @@ void MainWindow::ResetOptions() {
 void MainWindow::About() {
   QMessageBox::about(
       this, tr("About"),
-      QString().sprintf("<span style='font-weight:normal'><b>%s</b><br />"
-                        "<small>(%s)</small><br /><br />"
-                        "<b>Author:</b> Johannes L. Schönberger<br /><br />"
-                        "<b>Email:</b> jsch at inf.ethz.ch</span>",
-                        GetVersionInfo().c_str(), GetBuildInfo().c_str()));
+      QString().asprintf("<span style='font-weight:normal'><b>%s</b><br />"
+                         "<small>(%s)</small><br /><br />"
+                         "<b>Author:</b> Johannes L. Schönberger<br /><br />"
+                         "<b>Email:</b> jsch-at-demuc-dot-de</span>",
+                         GetVersionInfo().c_str(), GetBuildInfo().c_str()));
 }
 
 void MainWindow::Documentation() {
@@ -1278,7 +1284,7 @@ void MainWindow::UpdateTimer() {
   const int minutes = (elapsed_time / 60) % 60;
   const int hours = (elapsed_time / 3600) % 24;
   const int days = elapsed_time / 86400;
-  statusbar_timer_label_->setText(QString().sprintf(
+  statusbar_timer_label_->setText(QString().asprintf(
       "Time %02d:%02d:%02d:%02d", days, hours, minutes, seconds));
 }
 
