@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "ui/movie_grabber_widget.h"
 
@@ -136,7 +136,9 @@ void MovieGrabberWidget::Add() {
 
 void MovieGrabberWidget::Delete() {
   QModelIndexList selection = table_->selectionModel()->selectedIndexes();
-  foreach (QModelIndex index, selection) { table_->removeRow(index.row()); }
+  for (const auto& index : selection) {
+    table_->removeRow(index.row());
+  }
   UpdateViews();
   model_viewer_widget_->UpdateMovieGrabber();
 }
@@ -254,7 +256,7 @@ void MovieGrabberWidget::Assemble() {
 
       QImage image = model_viewer_widget_->GrabImage();
       image.save(dir.filePath(
-          "frame" + QString().sprintf("%06zu", frame_number) + ".png"));
+          "frame" + QString().asprintf("%06zu", frame_number) + ".png"));
       frame_number += 1;
     }
 
@@ -278,7 +280,7 @@ void MovieGrabberWidget::TimeChanged(QTableWidgetItem* item) {
 
 void MovieGrabberWidget::SelectionChanged(const QItemSelection& selected,
                                           const QItemSelection& deselected) {
-  foreach (QModelIndex index, table_->selectionModel()->selectedIndexes()) {
+  for (const auto& index : table_->selectionModel()->selectedIndexes()) {
     model_viewer_widget_->SelectMoviewGrabberView(index.row());
   }
 }
