@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "ui/point_painter.h"
 
@@ -80,14 +80,14 @@ void PointPainter::Upload(const std::vector<PointPainter::Data>& data) {
                 static_cast<int>(data.size() * sizeof(PointPainter::Data)));
 
   // in_position
-  shader_program_.enableAttributeArray(0);
-  shader_program_.setAttributeBuffer(0, GL_FLOAT, 0, 3,
+  shader_program_.enableAttributeArray("a_position");
+  shader_program_.setAttributeBuffer("a_position", GL_FLOAT, 0, 3,
                                      sizeof(PointPainter::Data));
 
   // in_color
-  shader_program_.enableAttributeArray(1);
-  shader_program_.setAttributeBuffer(1, GL_FLOAT, 3 * sizeof(GLfloat), 4,
-                                     sizeof(PointPainter::Data));
+  shader_program_.enableAttributeArray("a_color");
+  shader_program_.setAttributeBuffer("a_color", GL_FLOAT, 3 * sizeof(GLfloat),
+                                     4, sizeof(PointPainter::Data));
 
   // Make sure they are not changed from the outside
   vbo_.release();
@@ -110,7 +110,8 @@ void PointPainter::Render(const QMatrix4x4& pmv_matrix,
   shader_program_.setUniformValue("u_pmv_matrix", pmv_matrix);
   shader_program_.setUniformValue("u_point_size", point_size);
 
-  glDrawArrays(GL_POINTS, 0, (GLsizei)num_geoms_);
+  QOpenGLFunctions* gl_funcs = QOpenGLContext::currentContext()->functions();
+  gl_funcs->glDrawArrays(GL_POINTS, 0, (GLsizei)num_geoms_);
 
   // Make sure the VAO is not changed from the outside
   vao_.release();
