@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "ui/reconstruction_manager_widget.h"
 
@@ -59,7 +59,7 @@ void ReconstructionManagerWidget::Update() {
 
   int max_width = 0;
   for (size_t i = 0; i < reconstruction_manager_->Size(); ++i) {
-    const QString item = QString().sprintf(
+    const QString item = QString().asprintf(
         "Model %d (%d images, %d points)", static_cast<int>(i + 1),
         static_cast<int>(reconstruction_manager_->Get(i).NumRegImages()),
         static_cast<int>(reconstruction_manager_->Get(i).NumPoints3D()));
