@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "util/cuda.h"
 
@@ -56,7 +56,7 @@ bool CompareCudaDevice(const cudaDeviceProp& d1, const cudaDeviceProp& d2) {
 
 int GetNumCudaDevices() {
   int num_cuda_devices;
-  cudaGetDeviceCount(&num_cuda_devices);
+  CUDA_SAFE_CALL(cudaGetDeviceCount(&num_cuda_devices));
   return num_cuda_devices;
 }
 
