@@ -27,12 +27,13 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_UTIL_ENDIAN_H_
 #define COLMAP_SRC_UTIL_ENDIAN_H_
 
 #include <algorithm>
+#include <iostream>
 
 namespace colmap {
 
