@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "util/misc.h"
 
@@ -82,6 +82,21 @@ void SplitFileExtension(const std::string& path, std::string* root,
   }
 }
 
+void FileCopy(const std::string& src_path, const std::string& dst_path,
+              CopyType type) {
+  switch (type) {
+    case CopyType::COPY:
+      boost::filesystem::copy_file(src_path, dst_path);
+      break;
+    case CopyType::HARD_LINK:
+      boost::filesystem::create_hard_link(src_path, dst_path);
+      break;
+    case CopyType::SOFT_LINK:
+      boost::filesystem::create_symlink(src_path, dst_path);
+      break;
+  }
+}
+
 bool ExistsFile(const std::string& path) {
   return boost::filesystem::is_regular_file(path);
 }
@@ -114,6 +129,45 @@ std::string GetParentDir(const std::string& path) {
   return boost::filesystem::path(path).parent_path().string();
 }
 
+std::string GetRelativePath(const std::string& from, const std::string& to) {
+  // This implementation is adapted from:
+  // https://stackoverflow.com/questions/10167382
+  // A native implementation in boost::filesystem is only available starting
+  // from boost version 1.60.
+  using namespace boost::filesystem;
+
+  path from_path = canonical(path(from));
+  path to_path = canonical(path(to));
+
+  // Start at the root path and while they are the same then do nothing then
+  // when they first diverge take the entire from path, swap it with '..'
+  // segments, and then append the remainder of the to path.
+  path::const_iterator from_iter = from_path.begin();
+  path::const_iterator to_iter = to_path.begin();
+
+  // Loop through both while they are the same to find nearest common directory
+  while (from_iter != from_path.end() && to_iter != to_path.end() &&
+         (*to_iter) == (*from_iter)) {
+    ++to_iter;
+    ++from_iter;
+  }
+
+  // Replace from path segments with '..' (from => nearest common directory)
+  path rel_path;
+  while (from_iter != from_path.end()) {
+    rel_path /= "..";
+    ++from_iter;
+  }
+
+  // Append the remainder of the to path (nearest common directory => to)
+  while (to_iter != to_path.end()) {
+    rel_path /= *to_iter;
+    ++to_iter;
+  }
+
+  return rel_path.string();
+}
+
 std::vector<std::string> GetFileList(const std::string& path) {
   std::vector<std::string> file_list;
   for (auto it = boost::filesystem::directory_iterator(path);
@@ -206,7 +260,7 @@ std::vector<int> CSVToVector(const std::string& csv) {
     }
     try {
       values.push_back(std::stoi(elem));
-    } catch (std::exception) {
+    } catch (const std::invalid_argument&) {
       return std::vector<int>(0);
     }
   }
@@ -225,7 +279,7 @@ std::vector<float> CSVToVector(const std::string& csv) {
     }
     try {
       values.push_back(std::stod(elem));
-    } catch (std::exception) {
+    } catch (const std::invalid_argument&) {
       return std::vector<float>(0);
     }
   }
@@ -244,7 +298,7 @@ std::vector<double> CSVToVector(const std::string& csv) {
     }
     try {
       values.push_back(std::stold(elem));
-    } catch (std::exception) {
+    } catch (const std::invalid_argument&) {
       return std::vector<double>(0);
     }
   }
