@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_UTIL_MISC_H_
 #define COLMAP_SRC_UTIL_MISC_H_
@@ -50,6 +50,8 @@ namespace colmap {
 #define STRINGIFY_(s) #s
 #endif  // STRINGIFY
 
+enum class CopyType { COPY, HARD_LINK, SOFT_LINK };
+
 // Append trailing slash to string if it does not yet end with a slash.
 std::string EnsureTrailingSlash(const std::string& str);
 
@@ -61,6 +63,10 @@ bool HasFileExtension(const std::string& file_name, const std::string& ext);
 void SplitFileExtension(const std::string& path, std::string* root,
                         std::string* ext);
 
+// Copy or link file from source to destination path
+void FileCopy(const std::string& src_path, const std::string& dst_path,
+              CopyType type = CopyType::COPY);
+
 // Check if the path points to an existing directory.
 bool ExistsFile(const std::string& path);
 
@@ -79,6 +85,10 @@ std::string GetPathBaseName(const std::string& path);
 // Get the path of the parent directory for the given path.
 std::string GetParentDir(const std::string& path);
 
+// Get the relative path between from and to. Both the from and to paths must
+// exist.
+std::string GetRelativePath(const std::string& from, const std::string& to);
+
 // Join multiple paths into one path.
 template <typename... T>
 std::string JoinPaths(T const&... paths);
