@@ -27,17 +27,17 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "util/opengl_utils.h"
 
-#include <QApplication>
-
 #include "util/logging.h"
 
 namespace colmap {
 
-OpenGLContextManager::OpenGLContextManager()
+#ifdef GUI_ENABLED
+OpenGLContextManager::OpenGLContextManager(int opengl_major_version,
+                                           int opengl_minor_version)
     : parent_thread_(QThread::currentThread()),
       current_thread_(nullptr),
       make_current_action_(new QAction(this)) {
@@ -46,8 +46,8 @@ OpenGLContextManager::OpenGLContextManager()
 
   QSurfaceFormat format;
   format.setDepthBufferSize(24);
-  format.setMajorVersion(3);
-  format.setMinorVersion(2);
+  format.setMajorVersion(opengl_major_version);
+  format.setMinorVersion(opengl_minor_version);
   format.setSamples(4);
   format.setProfile(QSurfaceFormat::CompatibilityProfile);
   context_.setFormat(format);
@@ -57,13 +57,14 @@ OpenGLContextManager::OpenGLContextManager()
   context_.makeCurrent(&surface_);
   CHECK(context_.isValid()) << "Could not create valid OpenGL context";
 
-  connect(make_current_action_, &QAction::triggered, this,
-          [this]() {
-            CHECK_NOTNULL(current_thread_);
-            context_.doneCurrent();
-            context_.moveToThread(current_thread_);
-          },
-          Qt::BlockingQueuedConnection);
+  connect(
+      make_current_action_, &QAction::triggered, this,
+      [this]() {
+        CHECK_NOTNULL(current_thread_);
+        context_.doneCurrent();
+        context_.moveToThread(current_thread_);
+      },
+      Qt::BlockingQueuedConnection);
 }
 
 void OpenGLContextManager::MakeCurrent() {
@@ -128,4 +129,6 @@ void GLError(const char* file, const int line) {
   }
 }
 
+#endif
+
 }  // namespace colmap
