@@ -27,16 +27,19 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_OPENGL_UTILS_H_
 #define COLMAP_SRC_OPENGL_UTILS_H_
 
+#ifdef GUI_ENABLED
 #include <QAction>
+#include <QApplication>
 #include <QOffscreenSurface>
 #include <QOpenGLContext>
 #include <QThread>
 #include <QWaitCondition>
+#endif
 
 #include "util/threading.h"
 
@@ -48,12 +51,15 @@ namespace colmap {
 #define glDebugLog()
 #endif
 
+#ifdef GUI_ENABLED
+
 // This class manages a thread-safe OpenGL context. Note that this class must be
 // instantiated in the main Qt thread, since an OpenGL context must be created
 // in it. The context can then be made current in any other thread.
 class OpenGLContextManager : public QObject {
  public:
-  OpenGLContextManager();
+  OpenGLContextManager(int opengl_major_version = 2,
+                       int opengl_minor_version = 1);
 
   // Make the OpenGL context available by moving it from the thread where it was
   // created to the current thread and making it current.
@@ -86,6 +92,23 @@ void RunThreadWithOpenGLContext(Thread* thread);
 // Get the OpenGL errors and print them to stderr.
 void GLError(const char* file, const int line);
 
+#else
+
+// Dummy implementation when GUI support is disabled
+class OpenGLContextManager {
+ public:
+  OpenGLContextManager(int opengl_major_version = 2,
+                       int opengl_minor_version = 1) {}
+  inline void MakeCurrent() {}
+  inline static bool HasOpenGL() { return false; }
+};
+
+inline void RunThreadWithOpenGLContext(Thread* thread) {}
+
+inline void GLError(const char* file, const int line) {}
+
+#endif
+
 }  // namespace colmap
 
 #endif  // COLMAP_SRC_OPENGL_UTILS_H_
