@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "util/option_manager.h"
 
@@ -45,13 +45,14 @@
 #include "optim/bundle_adjustment.h"
 #include "ui/render_options.h"
 #include "util/misc.h"
+#include "util/random.h"
 #include "util/version.h"
 
 namespace config = boost::program_options;
 
 namespace colmap {
 
-OptionManager::OptionManager() {
+OptionManager::OptionManager(bool add_project_options) {
   project_path.reset(new std::string());
   database_path.reset(new std::string());
   image_path.reset(new std::string());
@@ -64,6 +65,7 @@ OptionManager::OptionManager() {
   vocab_tree_matching.reset(new VocabTreeMatchingOptions());
   spatial_matching.reset(new SpatialMatchingOptions());
   transitive_matching.reset(new TransitiveMatchingOptions());
+  image_pairs_matching.reset(new ImagePairsMatchingOptions());
   bundle_adjustment.reset(new BundleAdjustmentOptions());
   mapper.reset(new IncrementalMapperOptions());
   patch_match_stereo.reset(new mvs::PatchMatchOptions());
@@ -75,12 +77,18 @@ OptionManager::OptionManager() {
   Reset();
 
   desc_->add_options()("help,h", "");
-  desc_->add_options()("project_path", config::value<std::string>());
+
+  AddRandomOptions();
+  AddLogOptions();
+
+  if (add_project_options) {
+    desc_->add_options()("project_path", config::value<std::string>());
+  }
 }
 
 void OptionManager::ModifyForIndividualData() {
-  mapper->min_focal_length_ratio = 0.01;
-  mapper->max_focal_length_ratio = std::numeric_limits<double>::max();
+  mapper->min_focal_length_ratio = 0.1;
+  mapper->max_focal_length_ratio = 10;
   mapper->max_extra_param = std::numeric_limits<double>::max();
 }
 
@@ -90,8 +98,8 @@ void OptionManager::ModifyForVideoData() {
   mapper->mapper.init_min_tri_angle /= 2;
   mapper->ba_global_images_ratio = 1.4;
   mapper->ba_global_points_ratio = 1.4;
-  mapper->min_focal_length_ratio = 0.01;
-  mapper->max_focal_length_ratio = std::numeric_limits<double>::max();
+  mapper->min_focal_length_ratio = 0.1;
+  mapper->max_focal_length_ratio = 10;
   mapper->max_extra_param = std::numeric_limits<double>::max();
   stereo_fusion->min_num_pixels = 15;
 }
@@ -142,6 +150,9 @@ void OptionManager::ModifyForHighQuality() {
   sift_extraction->estimate_affine_shape = true;
   sift_extraction->max_image_size = 2400;
   sift_matching->guided_matching = true;
+  mapper->ba_local_max_num_iterations = 30;
+  mapper->ba_local_max_refinements = 3;
+  mapper->ba_global_max_num_iterations = 75;
   patch_match_stereo->max_image_size = 2400;
   stereo_fusion->max_image_size = 2400;
 }
@@ -151,10 +162,14 @@ void OptionManager::ModifyForExtremeQuality() {
   sift_extraction->estimate_affine_shape = true;
   sift_extraction->domain_size_pooling = true;
   sift_matching->guided_matching = true;
+  mapper->ba_local_max_num_iterations = 40;
+  mapper->ba_local_max_refinements = 3;
+  mapper->ba_global_max_num_iterations = 100;
 }
 
 void OptionManager::AddAllOptions() {
   AddLogOptions();
+  AddRandomOptions();
   AddDatabaseOptions();
   AddImageOptions();
   AddExtractionOptions();
@@ -164,6 +179,7 @@ void OptionManager::AddAllOptions() {
   AddVocabTreeMatchingOptions();
   AddSpatialMatchingOptions();
   AddTransitiveMatchingOptions();
+  AddImagePairsMatchingOptions();
   AddBundleAdjustmentOptions();
   AddMapperOptions();
   AddPatchMatchStereoOptions();
@@ -183,6 +199,15 @@ void OptionManager::AddLogOptions() {
   AddAndRegisterDefaultOption("log_level", &FLAGS_v);
 }
 
+void OptionManager::AddRandomOptions() {
+  if (added_random_options_) {
+    return;
+  }
+  added_random_options_ = true;
+
+  AddAndRegisterDefaultOption("random_seed", &kDefaultPRNGSeed);
+}
+
 void OptionManager::AddDatabaseOptions() {
   if (added_database_options_) {
     return;
@@ -207,18 +232,24 @@ void OptionManager::AddExtractionOptions() {
   }
   added_extraction_options_ = true;
 
+  AddAndRegisterDefaultOption("ImageReader.mask_path",
+                              &image_reader->mask_path);
   AddAndRegisterDefaultOption("ImageReader.camera_model",
                               &image_reader->camera_model);
   AddAndRegisterDefaultOption("ImageReader.single_camera",
                               &image_reader->single_camera);
   AddAndRegisterDefaultOption("ImageReader.single_camera_per_folder",
                               &image_reader->single_camera_per_folder);
+  AddAndRegisterDefaultOption("ImageReader.single_camera_per_image",
+                              &image_reader->single_camera_per_image);
   AddAndRegisterDefaultOption("ImageReader.existing_camera_id",
                               &image_reader->existing_camera_id);
   AddAndRegisterDefaultOption("ImageReader.camera_params",
                               &image_reader->camera_params);
   AddAndRegisterDefaultOption("ImageReader.default_focal_length_factor",
                               &image_reader->default_focal_length_factor);
+  AddAndRegisterDefaultOption("ImageReader.camera_mask_path",
+                              &image_reader->camera_mask_path);
 
   AddAndRegisterDefaultOption("SiftExtraction.num_threads",
                               &sift_extraction->num_threads);
@@ -393,6 +424,18 @@ void OptionManager::AddTransitiveMatchingOptions() {
                               &transitive_matching->num_iterations);
 }
 
+void OptionManager::AddImagePairsMatchingOptions() {
+  if (added_image_pairs_match_options_) {
+    return;
+  }
+  added_image_pairs_match_options_ = true;
+
+  AddMatchingOptions();
+
+  AddAndRegisterDefaultOption("ImagePairsMatching.block_size",
+                              &image_pairs_matching->block_size);
+}
+
 void OptionManager::AddBundleAdjustmentOptions() {
   if (added_ba_options_) {
     return;
@@ -420,6 +463,8 @@ void OptionManager::AddBundleAdjustmentOptions() {
                               &bundle_adjustment->refine_principal_point);
   AddAndRegisterDefaultOption("BundleAdjustment.refine_extra_params",
                               &bundle_adjustment->refine_extra_params);
+  AddAndRegisterDefaultOption("BundleAdjustment.refine_extrinsics",
+                              &bundle_adjustment->refine_extrinsics);
 }
 
 void OptionManager::AddMapperOptions() {
@@ -456,8 +501,13 @@ void OptionManager::AddMapperOptions() {
                               &mapper->ba_refine_principal_point);
   AddAndRegisterDefaultOption("Mapper.ba_refine_extra_params",
                               &mapper->ba_refine_extra_params);
+  AddAndRegisterDefaultOption(
+      "Mapper.ba_min_num_residuals_for_multi_threading",
+      &mapper->ba_min_num_residuals_for_multi_threading);
   AddAndRegisterDefaultOption("Mapper.ba_local_num_images",
                               &mapper->ba_local_num_images);
+  AddAndRegisterDefaultOption("Mapper.ba_local_function_tolerance",
+                              &mapper->ba_local_function_tolerance);
   AddAndRegisterDefaultOption("Mapper.ba_local_max_num_iterations",
                               &mapper->ba_local_max_num_iterations);
   AddAndRegisterDefaultOption("Mapper.ba_global_use_pba",
@@ -472,6 +522,8 @@ void OptionManager::AddMapperOptions() {
                               &mapper->ba_global_images_freq);
   AddAndRegisterDefaultOption("Mapper.ba_global_points_freq",
                               &mapper->ba_global_points_freq);
+  AddAndRegisterDefaultOption("Mapper.ba_global_function_tolerance",
+                              &mapper->ba_global_function_tolerance);
   AddAndRegisterDefaultOption("Mapper.ba_global_max_num_iterations",
                               &mapper->ba_global_max_num_iterations);
   AddAndRegisterDefaultOption("Mapper.ba_global_max_refinements",
@@ -485,6 +537,8 @@ void OptionManager::AddMapperOptions() {
   AddAndRegisterDefaultOption("Mapper.snapshot_path", &mapper->snapshot_path);
   AddAndRegisterDefaultOption("Mapper.snapshot_images_freq",
                               &mapper->snapshot_images_freq);
+  AddAndRegisterDefaultOption("Mapper.fix_existing_images",
+                              &mapper->fix_existing_images);
 
   // IncrementalMapper.
   AddAndRegisterDefaultOption("Mapper.init_min_num_inliers",
@@ -509,6 +563,8 @@ void OptionManager::AddMapperOptions() {
                               &mapper->mapper.filter_min_tri_angle);
   AddAndRegisterDefaultOption("Mapper.max_reg_trials",
                               &mapper->mapper.max_reg_trials);
+  AddAndRegisterDefaultOption("Mapper.local_ba_min_tri_angle",
+                              &mapper->mapper.local_ba_min_tri_angle);
 
   // IncrementalTriangulator.
   AddAndRegisterDefaultOption("Mapper.tri_max_transitivity",
@@ -588,6 +644,8 @@ void OptionManager::AddPatchMatchStereoOptions() {
       &patch_match_stereo->filter_geom_consistency_max_cost);
   AddAndRegisterDefaultOption("PatchMatchStereo.cache_size",
                               &patch_match_stereo->cache_size);
+  AddAndRegisterDefaultOption("PatchMatchStereo.allow_missing_files",
+                              &patch_match_stereo->allow_missing_files);
   AddAndRegisterDefaultOption("PatchMatchStereo.write_consistency_graph",
                               &patch_match_stereo->write_consistency_graph);
 }
@@ -598,6 +656,10 @@ void OptionManager::AddStereoFusionOptions() {
   }
   added_stereo_fusion_options_ = true;
 
+  AddAndRegisterDefaultOption("StereoFusion.mask_path",
+                              &stereo_fusion->mask_path);
+  AddAndRegisterDefaultOption("StereoFusion.num_threads",
+                              &stereo_fusion->num_threads);
   AddAndRegisterDefaultOption("StereoFusion.max_image_size",
                               &stereo_fusion->max_image_size);
   AddAndRegisterDefaultOption("StereoFusion.min_num_pixels",
@@ -616,6 +678,8 @@ void OptionManager::AddStereoFusionOptions() {
                               &stereo_fusion->check_num_images);
   AddAndRegisterDefaultOption("StereoFusion.cache_size",
                               &stereo_fusion->cache_size);
+  AddAndRegisterDefaultOption("StereoFusion.use_cache",
+                              &stereo_fusion->use_cache);
 }
 
 void OptionManager::AddPoissonMeshingOptions() {
@@ -643,6 +707,8 @@ void OptionManager::AddDelaunayMeshingOptions() {
                               &delaunay_meshing->max_proj_dist);
   AddAndRegisterDefaultOption("DelaunayMeshing.max_depth_dist",
                               &delaunay_meshing->max_depth_dist);
+  AddAndRegisterDefaultOption("DelaunayMeshing.visibility_sigma",
+                              &delaunay_meshing->visibility_sigma);    
   AddAndRegisterDefaultOption("DelaunayMeshing.distance_sigma_factor",
                               &delaunay_meshing->distance_sigma_factor);
   AddAndRegisterDefaultOption("DelaunayMeshing.quality_regularization",
@@ -687,6 +753,7 @@ void OptionManager::Reset() {
   options_string_.clear();
 
   added_log_options_ = false;
+  added_random_options_ = false;
   added_database_options_ = false;
   added_image_options_ = false;
   added_extraction_options_ = false;
@@ -696,6 +763,7 @@ void OptionManager::Reset() {
   added_vocab_tree_match_options_ = false;
   added_spatial_match_options_ = false;
   added_transitive_match_options_ = false;
+  added_image_pairs_match_options_ = false;
   added_ba_options_ = false;
   added_mapper_options_ = false;
   added_patch_match_stereo_options_ = false;
@@ -719,6 +787,7 @@ void OptionManager::ResetOptions(const bool reset_paths) {
   *vocab_tree_matching = VocabTreeMatchingOptions();
   *spatial_matching = SpatialMatchingOptions();
   *transitive_matching = TransitiveMatchingOptions();
+  *image_pairs_matching = ImagePairsMatchingOptions();
   *bundle_adjustment = BundleAdjustmentOptions();
   *mapper = IncrementalMapperOptions();
   *patch_match_stereo = mvs::PatchMatchOptions();
@@ -749,6 +818,8 @@ bool OptionManager::Check() {
   if (sequential_matching) success = success && sequential_matching->Check();
   if (vocab_tree_matching) success = success && vocab_tree_matching->Check();
   if (spatial_matching) success = success && spatial_matching->Check();
+  if (transitive_matching) success = success && transitive_matching->Check();
+  if (image_pairs_matching) success = success && image_pairs_matching->Check();
 
   if (bundle_adjustment) success = success && bundle_adjustment->Check();
   if (mapper) success = success && mapper->Check();
@@ -758,7 +829,9 @@ bool OptionManager::Check() {
   if (poisson_meshing) success = success && poisson_meshing->Check();
   if (delaunay_meshing) success = success && delaunay_meshing->Check();
 
+#ifdef GUI_ENABLED
   if (render) success = success && render->Check();
+#endif
 
   return success;
 }
