@@ -27,29 +27,29 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_UTIL_RANDOM_H_
 #define COLMAP_SRC_UTIL_RANDOM_H_
 
 #include <chrono>
+#include <memory>
 #include <random>
 #include <thread>
 
 #include "util/logging.h"
-#include "util/threading.h"
 
 namespace colmap {
 
-extern thread_local std::mt19937* PRNG;
+extern thread_local std::unique_ptr<std::mt19937> PRNG;
 
-static const unsigned kRandomPRNGSeed = std::numeric_limits<unsigned>::max();
+static int kDefaultPRNGSeed = 0;
 
 // Initialize the PRNG with the given seed.
 //
 // @param seed   The seed for the PRNG. If the seed is -1, the current time
 //               is used as the seed.
-void SetPRNGSeed(unsigned seed = kRandomPRNGSeed);
+void SetPRNGSeed(unsigned seed = kDefaultPRNGSeed);
 
 // Generate uniformly distributed random integer number.
 //
