@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #include "util/string.h"
 
@@ -108,7 +108,7 @@ void StringAppendV(std::string* dst, const char* format, va_list ap) {
   // Increase the buffer size to the size requested by vsnprintf,
   // plus one for the closing \0.
   const int variable_buffer_size = result + 1;
-  std::unique_ptr<char> variable_buffer(new char[variable_buffer_size]);
+  std::unique_ptr<char[]> variable_buffer(new char[variable_buffer_size]);
 
   // Restore the va_list before we use it again.
   va_copy(backup_ap, ap);
@@ -157,7 +157,8 @@ std::string StringGetAfter(const std::string& str, const std::string& key) {
   }
   std::size_t found = str.rfind(key);
   if (found != std::string::npos) {
-    return str.substr(found + key.length(), str.length() - (found + key.length()));
+    return str.substr(found + key.length(),
+                      str.length() - (found + key.length()));
   }
   return "";
 }
