@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #ifndef COLMAP_SRC_UTIL_THREADING_
 #define COLMAP_SRC_UTIL_THREADING_
@@ -49,20 +49,6 @@ namespace colmap {
 #pragma clang diagnostic ignored "-Wkeyword-macro"
 #endif
 
-// Define `thread_local` cross-platform.
-#ifndef thread_local
-#if __STDC_VERSION__ >= 201112 && !defined __STDC_NO_THREADS__
-#define thread_local _Thread_local
-#elif defined _WIN32 && (defined _MSC_VER || defined __ICL || \
-                         defined __DMC__ || defined __BORLANDC__)
-#define thread_local __declspec(thread)
-#elif defined __GNUC__ || defined __SUNPRO_C || defined __xlC__
-#define thread_local __thread
-#else
-#error "Cannot define thread_local"
-#endif
-#endif
-
 #ifdef __clang__
 #pragma clang diagnostic pop  // -Wkeyword-macro
 #endif
