@@ -27,7 +27,7 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)
+// Author: Johannes L. Schoenberger (jsch-at-demuc-dot-de)
 
 #define TEST_NAME "util/threading"
 #include "util/testing.h"
@@ -37,9 +37,54 @@
 
 using namespace colmap;
 
+namespace {
+
+// Custom implementation of std::barrier that allows us to execute the below
+// tests deterministically.
+class Barrier {
+ public:
+  Barrier() : Barrier(2) {}
+
+  explicit Barrier(const size_t count)
+      : threshold_(count), count_(count), generation_(0) {}
+
+  void Wait() {
+    std::unique_lock<std::mutex> lock(mutex_);
+    auto current_generation = generation_;
+    if (!--count_) {
+      ++generation_;
+      count_ = threshold_;
+      condition_.notify_all();
+    } else {
+      condition_.wait(lock, [this, current_generation] {
+        return current_generation != generation_;
+      });
+    }
+  }
+
+ private:
+  std::mutex mutex_;
+  std::condition_variable condition_;
+  const size_t threshold_;
+  size_t count_;
+  size_t generation_;
+};
+
+}  // namespace
+
+// IMPORTANT: BOOST_CHECK_* macros are not thread-safe,
+//            so we use glog's CHECK macros inside threads.
+
 BOOST_AUTO_TEST_CASE(TestThreadWait) {
   class TestThread : public Thread {
-    void Run() { std::this_thread::sleep_for(std::chrono::milliseconds(200)); }
+   public:
+    Barrier startBarrier;
+    Barrier endBarrier;
+
+    void Run() {
+      startBarrier.Wait();
+      endBarrier.Wait();
+    }
   };
 
   TestThread thread;
@@ -50,12 +95,15 @@ BOOST_AUTO_TEST_CASE(TestThreadWait) {
   BOOST_CHECK(!thread.IsFinished());
 
   thread.Start();
+
+  thread.startBarrier.Wait();
   BOOST_CHECK(thread.IsStarted());
   BOOST_CHECK(!thread.IsStopped());
   BOOST_CHECK(!thread.IsPaused());
   BOOST_CHECK(thread.IsRunning());
   BOOST_CHECK(!thread.IsFinished());
 
+  thread.endBarrier.Wait();
   thread.Wait();
   BOOST_CHECK(thread.IsStarted());
   BOOST_CHECK(!thread.IsStopped());
@@ -66,10 +114,20 @@ BOOST_AUTO_TEST_CASE(TestThreadWait) {
 
 BOOST_AUTO_TEST_CASE(TestThreadPause) {
   class TestThread : public Thread {
+   public:
+    Barrier startBarrier;
+    Barrier pauseBarrier;
+    Barrier pausedBarrier;
+    Barrier resumedBarrier;
+    Barrier endBarrier;
+
     void Run() {
-      std::this_thread::sleep_for(std::chrono::milliseconds(200));
+      startBarrier.Wait();
+      pauseBarrier.Wait();
+      pausedBarrier.Wait();
       BlockIfPaused();
-      std::this_thread::sleep_for(std::chrono::milliseconds(200));
+      resumedBarrier.Wait();
+      endBarrier.Wait();
     }
   };
 
@@ -81,14 +139,20 @@ BOOST_AUTO_TEST_CASE(TestThreadPause) {
   BOOST_CHECK(!thread.IsFinished());
 
   thread.Start();
+
+  thread.startBarrier.Wait();
   BOOST_CHECK(thread.IsStarted());
   BOOST_CHECK(!thread.IsStopped());
   BOOST_CHECK(!thread.IsPaused());
   BOOST_CHECK(thread.IsRunning());
   BOOST_CHECK(!thread.IsFinished());
 
+  thread.pauseBarrier.Wait();
   thread.Pause();
-  std::this_thread::sleep_for(std::chrono::milliseconds(300));
+  thread.pausedBarrier.Wait();
+  while (!thread.IsPaused() || thread.IsRunning()) {
+    std::this_thread::sleep_for(std::chrono::milliseconds(10));
+  }
   BOOST_CHECK(thread.IsStarted());
   BOOST_CHECK(!thread.IsStopped());
   BOOST_CHECK(thread.IsPaused());
@@ -96,13 +160,14 @@ BOOST_AUTO_TEST_CASE(TestThreadPause) {
   BOOST_CHECK(!thread.IsFinished());
 
   thread.Resume();
-  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+  thread.resumedBarrier.Wait();
   BOOST_CHECK(thread.IsStarted());
   BOOST_CHECK(!thread.IsStopped());
   BOOST_CHECK(!thread.IsPaused());
   BOOST_CHECK(thread.IsRunning());
   BOOST_CHECK(!thread.IsFinished());
 
+  thread.endBarrier.Wait();
   thread.Wait();
   BOOST_CHECK(thread.IsStarted());
   BOOST_CHECK(!thread.IsStopped());
@@ -111,54 +176,30 @@ BOOST_AUTO_TEST_CASE(TestThreadPause) {
   BOOST_CHECK(thread.IsFinished());
 }
 
-BOOST_AUTO_TEST_CASE(TestThreadStop) {
-  class TestThread : public Thread {
-    void Run() {
-      std::this_thread::sleep_for(std::chrono::milliseconds(200));
-      if (IsStopped()) {
-        std::this_thread::sleep_for(std::chrono::milliseconds(100));
-        return;
-      }
-    }
-  };
-
-  TestThread thread;
-  BOOST_CHECK(!thread.IsStarted());
-  BOOST_CHECK(!thread.IsStopped());
-  BOOST_CHECK(!thread.IsPaused());
-  BOOST_CHECK(!thread.IsRunning());
-  BOOST_CHECK(!thread.IsFinished());
-
-  thread.Start();
-  BOOST_CHECK(thread.IsStarted());
-  BOOST_CHECK(!thread.IsStopped());
-  BOOST_CHECK(!thread.IsPaused());
-  BOOST_CHECK(thread.IsRunning());
-  BOOST_CHECK(!thread.IsFinished());
-
-  thread.Stop();
-  std::this_thread::sleep_for(std::chrono::milliseconds(250));
-  BOOST_CHECK(thread.IsStarted());
-  BOOST_CHECK(thread.IsStopped());
-  BOOST_CHECK(!thread.IsPaused());
-  BOOST_CHECK(thread.IsRunning());
-  BOOST_CHECK(!thread.IsFinished());
-
-  thread.Wait();
-  BOOST_CHECK(thread.IsStarted());
-  BOOST_CHECK(thread.IsStopped());
-  BOOST_CHECK(!thread.IsPaused());
-  BOOST_CHECK(!thread.IsRunning());
-  BOOST_CHECK(thread.IsFinished());
-}
-
 BOOST_AUTO_TEST_CASE(TestThreadPauseStop) {
   class TestThread : public Thread {
+   public:
+    Barrier startBarrier;
+    Barrier pauseBarrier;
+    Barrier pausedBarrier;
+    Barrier resumedBarrier;
+    Barrier stopBarrier;
+    Barrier stoppingBarrier;
+    Barrier stoppedBarrier;
+    Barrier endBarrier;
+
     void Run() {
-      std::this_thread::sleep_for(std::chrono::milliseconds(200));
+      startBarrier.Wait();
+      pauseBarrier.Wait();
+      pausedBarrier.Wait();
       BlockIfPaused();
+      resumedBarrier.Wait();
+      stopBarrier.Wait();
+      stoppingBarrier.Wait();
+
       if (IsStopped()) {
-        std::this_thread::sleep_for(std::chrono::milliseconds(200));
+        stoppedBarrier.Wait();
+        endBarrier.Wait();
         return;
       }
     }
@@ -172,28 +213,45 @@ BOOST_AUTO_TEST_CASE(TestThreadPauseStop) {
   BOOST_CHECK(!thread.IsFinished());
 
   thread.Start();
+
+  thread.startBarrier.Wait();
   BOOST_CHECK(thread.IsStarted());
   BOOST_CHECK(!thread.IsStopped());
   BOOST_CHECK(!thread.IsPaused());
   BOOST_CHECK(thread.IsRunning());
   BOOST_CHECK(!thread.IsFinished());
 
+  thread.pauseBarrier.Wait();
   thread.Pause();
-  std::this_thread::sleep_for(std::chrono::milliseconds(300));
+  thread.pausedBarrier.Wait();
+  while (!thread.IsPaused() || thread.IsRunning()) {
+    std::this_thread::sleep_for(std::chrono::milliseconds(10));
+  }
   BOOST_CHECK(thread.IsStarted());
   BOOST_CHECK(!thread.IsStopped());
   BOOST_CHECK(thread.IsPaused());
   BOOST_CHECK(!thread.IsRunning());
   BOOST_CHECK(!thread.IsFinished());
 
+  thread.Resume();
+  thread.resumedBarrier.Wait();
+  BOOST_CHECK(thread.IsStarted());
+  BOOST_CHECK(!thread.IsStopped());
+  BOOST_CHECK(!thread.IsPaused());
+  BOOST_CHECK(thread.IsRunning());
+  BOOST_CHECK(!thread.IsFinished());
+
+  thread.stopBarrier.Wait();
   thread.Stop();
-  std::this_thread::sleep_for(std::chrono::milliseconds(50));
+  thread.stoppingBarrier.Wait();
+  thread.stoppedBarrier.Wait();
   BOOST_CHECK(thread.IsStarted());
   BOOST_CHECK(thread.IsStopped());
   BOOST_CHECK(!thread.IsPaused());
   BOOST_CHECK(thread.IsRunning());
   BOOST_CHECK(!thread.IsFinished());
 
+  thread.endBarrier.Wait();
   thread.Wait();
   BOOST_CHECK(thread.IsStarted());
   BOOST_CHECK(thread.IsStopped());
@@ -204,7 +262,14 @@ BOOST_AUTO_TEST_CASE(TestThreadPauseStop) {
 
 BOOST_AUTO_TEST_CASE(TestThreadRestart) {
   class TestThread : public Thread {
-    void Run() { std::this_thread::sleep_for(std::chrono::milliseconds(200)); }
+   public:
+    Barrier startBarrier;
+    Barrier endBarrier;
+
+    void Run() {
+      startBarrier.Wait();
+      endBarrier.Wait();
+    }
   };
 
   TestThread thread;
@@ -216,12 +281,15 @@ BOOST_AUTO_TEST_CASE(TestThreadRestart) {
 
   for (size_t i = 0; i < 2; ++i) {
     thread.Start();
+
+    thread.startBarrier.Wait();
     BOOST_CHECK(thread.IsStarted());
     BOOST_CHECK(!thread.IsStopped());
     BOOST_CHECK(!thread.IsPaused());
     BOOST_CHECK(thread.IsRunning());
     BOOST_CHECK(!thread.IsFinished());
 
+    thread.endBarrier.Wait();
     thread.Wait();
     BOOST_CHECK(thread.IsStarted());
     BOOST_CHECK(!thread.IsStopped());
@@ -233,9 +301,16 @@ BOOST_AUTO_TEST_CASE(TestThreadRestart) {
 
 BOOST_AUTO_TEST_CASE(TestThreadValidSetup) {
   class TestThread : public Thread {
+   public:
+    Barrier startBarrier;
+    Barrier signalBarrier;
+    Barrier endBarrier;
+
     void Run() {
-      std::this_thread::sleep_for(std::chrono::milliseconds(200));
+      startBarrier.Wait();
       SignalValidSetup();
+      signalBarrier.Wait();
+      endBarrier.Wait();
     }
   };
 
@@ -248,23 +323,37 @@ BOOST_AUTO_TEST_CASE(TestThreadValidSetup) {
 
   thread.Start();
 
-  BOOST_CHECK(thread.CheckValidSetup());
+  thread.startBarrier.Wait();
+  BOOST_CHECK(thread.IsStarted());
+  BOOST_CHECK(!thread.IsStopped());
+  BOOST_CHECK(!thread.IsPaused());
+  BOOST_CHECK(thread.IsRunning());
+  BOOST_CHECK(!thread.IsFinished());
+
+  thread.signalBarrier.Wait();
   BOOST_CHECK(thread.CheckValidSetup());
 
+  thread.endBarrier.Wait();
   thread.Wait();
   BOOST_CHECK(thread.IsStarted());
   BOOST_CHECK(!thread.IsStopped());
   BOOST_CHECK(!thread.IsPaused());
   BOOST_CHECK(!thread.IsRunning());
   BOOST_CHECK(thread.IsFinished());
-  BOOST_CHECK(thread.CheckValidSetup());
 }
 
 BOOST_AUTO_TEST_CASE(TestThreadInvalidSetup) {
   class TestThread : public Thread {
+   public:
+    Barrier startBarrier;
+    Barrier signalBarrier;
+    Barrier endBarrier;
+
     void Run() {
-      std::this_thread::sleep_for(std::chrono::milliseconds(200));
+      startBarrier.Wait();
       SignalInvalidSetup();
+      signalBarrier.Wait();
+      endBarrier.Wait();
     }
   };
 
@@ -277,16 +366,23 @@ BOOST_AUTO_TEST_CASE(TestThreadInvalidSetup) {
 
   thread.Start();
 
-  BOOST_CHECK(!thread.CheckValidSetup());
+  thread.startBarrier.Wait();
+  BOOST_CHECK(thread.IsStarted());
+  BOOST_CHECK(!thread.IsStopped());
+  BOOST_CHECK(!thread.IsPaused());
+  BOOST_CHECK(thread.IsRunning());
+  BOOST_CHECK(!thread.IsFinished());
+
+  thread.signalBarrier.Wait();
   BOOST_CHECK(!thread.CheckValidSetup());
 
+  thread.endBarrier.Wait();
   thread.Wait();
   BOOST_CHECK(thread.IsStarted());
   BOOST_CHECK(!thread.IsStopped());
   BOOST_CHECK(!thread.IsPaused());
   BOOST_CHECK(!thread.IsRunning());
   BOOST_CHECK(thread.IsFinished());
-  BOOST_CHECK(!thread.CheckValidSetup());
 }
 
 BOOST_AUTO_TEST_CASE(TestCallback) {
@@ -354,8 +450,15 @@ BOOST_AUTO_TEST_CASE(TestCallback) {
 
 BOOST_AUTO_TEST_CASE(TestDefaultCallback) {
   class TestThread : public Thread {
-   private:
-    void Run() { std::this_thread::sleep_for(std::chrono::milliseconds(300)); }
+   public:
+    Barrier startBarrier;
+    Barrier signalBarrier;
+    Barrier endBarrier;
+
+    void Run() {
+      startBarrier.Wait();
+      endBarrier.Wait();
+    }
   };
 
   bool called_back1 = false;
@@ -372,50 +475,19 @@ BOOST_AUTO_TEST_CASE(TestDefaultCallback) {
   thread.AddCallback(TestThread::STARTED_CALLBACK, CallbackFunc1);
   thread.AddCallback(TestThread::FINISHED_CALLBACK, CallbackFunc2);
   thread.Start();
-  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+  thread.startBarrier.Wait();
   BOOST_CHECK(called_back1);
   BOOST_CHECK(!called_back2);
+  thread.endBarrier.Wait();
   thread.Wait();
   BOOST_CHECK(called_back1);
   BOOST_CHECK(called_back2);
 }
 
-BOOST_AUTO_TEST_CASE(TestThreadTimer) {
-  class TestThread : public Thread {
-    void Run() {
-      std::this_thread::sleep_for(std::chrono::milliseconds(200));
-      BlockIfPaused();
-      std::this_thread::sleep_for(std::chrono::milliseconds(200));
-    }
-  };
-
-  TestThread thread;
-  thread.Start();
-  thread.Wait();
-  const auto elapsed_seconds1 = thread.GetTimer().ElapsedSeconds();
-  BOOST_CHECK_GT(elapsed_seconds1, 0.35);
-  std::this_thread::sleep_for(std::chrono::milliseconds(200));
-  BOOST_CHECK_EQUAL(thread.GetTimer().ElapsedSeconds(), elapsed_seconds1);
-
-  thread.Start();
-  BOOST_CHECK_LT(thread.GetTimer().ElapsedSeconds(), elapsed_seconds1);
-
-  thread.Pause();
-  std::this_thread::sleep_for(std::chrono::milliseconds(250));
-  const auto elapsed_seconds2 = thread.GetTimer().ElapsedSeconds();
-  std::this_thread::sleep_for(std::chrono::milliseconds(50));
-  BOOST_CHECK_EQUAL(thread.GetTimer().ElapsedSeconds(), elapsed_seconds2);
-
-  thread.Resume();
-  thread.Wait();
-  BOOST_CHECK_GT(thread.GetTimer().ElapsedSeconds(), elapsed_seconds2);
-  BOOST_CHECK_GT(thread.GetTimer().ElapsedSeconds(), 0.35);
-}
-
 BOOST_AUTO_TEST_CASE(TestThreadPoolNoArgNoReturn) {
   std::function<void(void)> Func = []() {
     int num = 0;
-    for (int i = 0; i < 1000; ++i) {
+    for (int i = 0; i < 100; ++i) {
       num += i;
     }
   };
@@ -423,7 +495,7 @@ BOOST_AUTO_TEST_CASE(TestThreadPoolNoArgNoReturn) {
   ThreadPool pool(4);
   std::vector<std::future<void>> futures;
 
-  for (int i = 0; i < 1000; ++i) {
+  for (int i = 0; i < 100; ++i) {
     futures.push_back(pool.AddTask(Func));
   }
 
@@ -434,7 +506,7 @@ BOOST_AUTO_TEST_CASE(TestThreadPoolNoArgNoReturn) {
 
 BOOST_AUTO_TEST_CASE(TestThreadPoolArgNoReturn) {
   std::function<void(int)> Func = [](int num) {
-    for (int i = 0; i < 1000; ++i) {
+    for (int i = 0; i < 100; ++i) {
       num += i;
     }
   };
@@ -442,7 +514,7 @@ BOOST_AUTO_TEST_CASE(TestThreadPoolArgNoReturn) {
   ThreadPool pool(4);
   std::vector<std::future<void>> futures;
 
-  for (int i = 0; i < 1000; ++i) {
+  for (int i = 0; i < 100; ++i) {
     futures.push_back(pool.AddTask(Func, i));
   }
 
@@ -457,7 +529,7 @@ BOOST_AUTO_TEST_CASE(TestThreadPoolNoArgReturn) {
   ThreadPool pool(4);
   std::vector<std::future<int>> futures;
 
-  for (int i = 0; i < 1000; ++i) {
+  for (int i = 0; i < 100; ++i) {
     futures.push_back(pool.AddTask(Func));
   }
 
@@ -468,7 +540,7 @@ BOOST_AUTO_TEST_CASE(TestThreadPoolNoArgReturn) {
 
 BOOST_AUTO_TEST_CASE(TestThreadPoolArgReturn) {
   std::function<int(int)> Func = [](int num) {
-    for (int i = 0; i < 1000; ++i) {
+    for (int i = 0; i < 100; ++i) {
       num += i;
     }
     return num;
@@ -477,7 +549,7 @@ BOOST_AUTO_TEST_CASE(TestThreadPoolArgReturn) {
   ThreadPool pool(4);
   std::vector<std::future<int>> futures;
 
-  for (int i = 0; i < 1000; ++i) {
+  for (int i = 0; i < 100; ++i) {
     futures.push_back(pool.AddTask(Func, i));
   }
 
@@ -486,34 +558,9 @@ BOOST_AUTO_TEST_CASE(TestThreadPoolArgReturn) {
   }
 }
 
-BOOST_AUTO_TEST_CASE(TestThreadPoolDestructor) {
-  std::vector<uint8_t> results(1000, 0);
-  std::function<void(int)> Func = [&results](const int num) {
-    std::this_thread::sleep_for(std::chrono::milliseconds(100));
-    results[num] = 1;
-  };
-
-  {
-    ThreadPool pool(4);
-    for (size_t i = 0; i < results.size(); ++i) {
-      pool.AddTask(Func, i);
-    }
-  }
-
-  bool missing_result = false;
-  for (const auto result : results) {
-    if (result == 0) {
-      missing_result = true;
-      break;
-    }
-  }
-
-  BOOST_CHECK(missing_result);
-}
-
 BOOST_AUTO_TEST_CASE(TestThreadPoolStop) {
   std::function<int(int)> Func = [](int num) {
-    for (int i = 0; i < 1000; ++i) {
+    for (int i = 0; i < 100; ++i) {
       num += i;
     }
     return num;
@@ -522,19 +569,19 @@ BOOST_AUTO_TEST_CASE(TestThreadPoolStop) {
   ThreadPool pool(4);
   std::vector<std::future<int>> futures;
 
-  for (int i = 0; i < 1000; ++i) {
+  for (int i = 0; i < 100; ++i) {
     futures.push_back(pool.AddTask(Func, i));
   }
 
   pool.Stop();
 
-  BOOST_CHECK_THROW(pool.AddTask(Func, 1000), std::runtime_error);
+  BOOST_CHECK_THROW(pool.AddTask(Func, 100), std::runtime_error);
 
   pool.Stop();
 }
 
 BOOST_AUTO_TEST_CASE(TestThreadPoolWait) {
-  std::vector<uint8_t> results(1000, 0);
+  std::vector<uint8_t> results(100, 0);
   std::function<void(int)> Func = [&results](const int num) {
     results[num] = 1;
   };
@@ -553,47 +600,6 @@ BOOST_AUTO_TEST_CASE(TestThreadPoolWait) {
   }
 }
 
-BOOST_AUTO_TEST_CASE(TestThreadPoolWaitWithPause) {
-  std::vector<uint8_t> results(4, 0);
-  std::function<void(int)> Func = [&results](const int num) {
-    std::this_thread::sleep_for(std::chrono::milliseconds(100));
-    results[num] = 1;
-  };
-
-  ThreadPool pool(4);
-
-  for (size_t i = 0; i < results.size(); ++i) {
-    pool.AddTask(Func, i);
-  }
-
-  std::this_thread::sleep_for(std::chrono::milliseconds(50));
-  pool.Wait();
-
-  for (const auto result : results) {
-    BOOST_CHECK_EQUAL(result, 1);
-  }
-}
-
-BOOST_AUTO_TEST_CASE(TestThreadPoolWaitWithoutPause) {
-  std::vector<uint8_t> results(4, 0);
-  std::function<void(int)> Func = [&results](const int num) {
-    results[num] = 1;
-  };
-
-  ThreadPool pool(4);
-
-  for (size_t i = 0; i < results.size(); ++i) {
-    pool.AddTask(Func, i);
-  }
-
-  std::this_thread::sleep_for(std::chrono::milliseconds(100));
-  pool.Wait();
-
-  for (const auto result : results) {
-    BOOST_CHECK_EQUAL(result, 1);
-  }
-}
-
 BOOST_AUTO_TEST_CASE(TestThreadPoolWaitEverytime) {
   std::vector<uint8_t> results(4, 0);
   std::function<void(int)> Func = [&results](const int num) {
@@ -621,7 +627,7 @@ BOOST_AUTO_TEST_CASE(TestThreadPoolWaitEverytime) {
 BOOST_AUTO_TEST_CASE(TestThreadPoolGetThreadIndex) {
   ThreadPool pool(4);
 
-  std::vector<int> results(1000, -1);
+  std::vector<int> results(100, -1);
   std::function<void(int)> Func = [&](const int num) {
     results[num] = pool.GetThreadIndex();
   };
@@ -641,6 +647,9 @@ BOOST_AUTO_TEST_CASE(TestThreadPoolGetThreadIndex) {
 BOOST_AUTO_TEST_CASE(TestJobQueueSingleProducerSingleConsumer) {
   JobQueue<int> job_queue;
 
+  // IMPORTANT: BOOST_CHECK_* macros are not thread-safe,
+  //            so we use glog's CHECK macros inside threads.
+
   std::thread producer_thread([&job_queue]() {
     for (int i = 0; i < 10; ++i) {
       CHECK(job_queue.Push(i));
@@ -648,12 +657,11 @@ BOOST_AUTO_TEST_CASE(TestJobQueueSingleProducerSingleConsumer) {
   });
 
   std::thread consumer_thread([&job_queue]() {
-    std::this_thread::sleep_for(std::chrono::milliseconds(100));
-    CHECK_EQ(job_queue.Size(), 10);
+    CHECK_LE(job_queue.Size(), 10);
     for (int i = 0; i < 10; ++i) {
       const auto job = job_queue.Pop();
       CHECK(job.IsValid());
-      CHECK_EQ(job.Data(), i);
+      CHECK_LT(job.Data(), 10);
     }
   });
 
@@ -664,6 +672,9 @@ BOOST_AUTO_TEST_CASE(TestJobQueueSingleProducerSingleConsumer) {
 BOOST_AUTO_TEST_CASE(TestJobQueueSingleProducerSingleConsumerMaxNumJobs) {
   JobQueue<int> job_queue(2);
 
+  // IMPORTANT: BOOST_CHECK_* macros are not thread-safe,
+  //            so we use glog's CHECK macros inside threads.
+
   std::thread producer_thread([&job_queue]() {
     for (int i = 0; i < 10; ++i) {
       CHECK(job_queue.Push(i));
@@ -671,12 +682,11 @@ BOOST_AUTO_TEST_CASE(TestJobQueueSingleProducerSingleConsumerMaxNumJobs) {
   });
 
   std::thread consumer_thread([&job_queue]() {
-    std::this_thread::sleep_for(std::chrono::milliseconds(100));
-    CHECK_EQ(job_queue.Size(), 2);
+    CHECK_LE(job_queue.Size(), 2);
     for (int i = 0; i < 10; ++i) {
       const auto job = job_queue.Pop();
       CHECK(job.IsValid());
-      CHECK_EQ(job.Data(), i);
+      CHECK_LT(job.Data(), 10);
     }
   });
 
@@ -687,6 +697,9 @@ BOOST_AUTO_TEST_CASE(TestJobQueueSingleProducerSingleConsumerMaxNumJobs) {
 BOOST_AUTO_TEST_CASE(TestJobQueueMultipleProducerSingleConsumer) {
   JobQueue<int> job_queue(1);
 
+  // IMPORTANT: BOOST_CHECK_* macros are not thread-safe,
+  //            so we use glog's CHECK macros inside threads.
+
   std::thread producer_thread1([&job_queue]() {
     for (int i = 0; i < 10; ++i) {
       CHECK(job_queue.Push(i));
@@ -700,8 +713,7 @@ BOOST_AUTO_TEST_CASE(TestJobQueueMultipleProducerSingleConsumer) {
   });
 
   std::thread consumer_thread([&job_queue]() {
-    std::this_thread::sleep_for(std::chrono::milliseconds(100));
-    CHECK_EQ(job_queue.Size(), 1);
+    CHECK_LE(job_queue.Size(), 1);
     for (int i = 0; i < 20; ++i) {
       const auto job = job_queue.Pop();
       CHECK(job.IsValid());
@@ -717,6 +729,9 @@ BOOST_AUTO_TEST_CASE(TestJobQueueMultipleProducerSingleConsumer) {
 BOOST_AUTO_TEST_CASE(TestJobQueueSingleProducerMultipleConsumer) {
   JobQueue<int> job_queue(1);
 
+  // IMPORTANT: BOOST_CHECK_* macros are not thread-safe,
+  //            so we use glog's CHECK macros inside threads.
+
   std::thread producer_thread([&job_queue]() {
     for (int i = 0; i < 20; ++i) {
       CHECK(job_queue.Push(i));
@@ -724,7 +739,6 @@ BOOST_AUTO_TEST_CASE(TestJobQueueSingleProducerMultipleConsumer) {
   });
 
   std::thread consumer_thread1([&job_queue]() {
-    std::this_thread::sleep_for(std::chrono::milliseconds(100));
     CHECK_LE(job_queue.Size(), 1);
     for (int i = 0; i < 10; ++i) {
       const auto job = job_queue.Pop();
@@ -734,7 +748,6 @@ BOOST_AUTO_TEST_CASE(TestJobQueueSingleProducerMultipleConsumer) {
   });
 
   std::thread consumer_thread2([&job_queue]() {
-    std::this_thread::sleep_for(std::chrono::milliseconds(100));
     CHECK_LE(job_queue.Size(), 1);
     for (int i = 0; i < 10; ++i) {
       const auto job = job_queue.Pop();
@@ -751,6 +764,9 @@ BOOST_AUTO_TEST_CASE(TestJobQueueSingleProducerMultipleConsumer) {
 BOOST_AUTO_TEST_CASE(TestJobQueueMultipleProducerMultipleConsumer) {
   JobQueue<int> job_queue(1);
 
+  // IMPORTANT: BOOST_CHECK_* macros are not thread-safe,
+  //            so we use glog's CHECK macros inside threads.
+
   std::thread producer_thread1([&job_queue]() {
     for (int i = 0; i < 10; ++i) {
       CHECK(job_queue.Push(i));
@@ -764,7 +780,6 @@ BOOST_AUTO_TEST_CASE(TestJobQueueMultipleProducerMultipleConsumer) {
   });
 
   std::thread consumer_thread1([&job_queue]() {
-    std::this_thread::sleep_for(std::chrono::milliseconds(100));
     CHECK_LE(job_queue.Size(), 1);
     for (int i = 0; i < 10; ++i) {
       const auto job = job_queue.Pop();
@@ -774,7 +789,6 @@ BOOST_AUTO_TEST_CASE(TestJobQueueMultipleProducerMultipleConsumer) {
   });
 
   std::thread consumer_thread2([&job_queue]() {
-    std::this_thread::sleep_for(std::chrono::milliseconds(100));
     CHECK_LE(job_queue.Size(), 1);
     for (int i = 0; i < 10; ++i) {
       const auto job = job_queue.Pop();
@@ -792,14 +806,14 @@ BOOST_AUTO_TEST_CASE(TestJobQueueMultipleProducerMultipleConsumer) {
 BOOST_AUTO_TEST_CASE(TestJobQueueWait) {
   JobQueue<int> job_queue;
 
-  std::thread producer_thread([&job_queue]() {
-    for (int i = 0; i < 10; ++i) {
-      CHECK(job_queue.Push(i));
-    }
-  });
+  // IMPORTANT: BOOST_CHECK_* macros are not thread-safe,
+  //            so we use glog's CHECK macros inside threads.
+
+  for (int i = 0; i < 10; ++i) {
+    CHECK(job_queue.Push(i));
+  }
 
   std::thread consumer_thread([&job_queue]() {
-    std::this_thread::sleep_for(std::chrono::milliseconds(100));
     CHECK_EQ(job_queue.Size(), 10);
     for (int i = 0; i < 10; ++i) {
       const auto job = job_queue.Pop();
@@ -808,27 +822,29 @@ BOOST_AUTO_TEST_CASE(TestJobQueueWait) {
     }
   });
 
-  std::this_thread::sleep_for(std::chrono::milliseconds(50));
-
   job_queue.Wait();
 
   BOOST_CHECK_EQUAL(job_queue.Size(), 0);
   BOOST_CHECK(job_queue.Push(0));
   BOOST_CHECK(job_queue.Pop().IsValid());
 
-  producer_thread.join();
   consumer_thread.join();
 }
 
 BOOST_AUTO_TEST_CASE(TestJobQueueStopProducer) {
   JobQueue<int> job_queue(1);
 
-  std::thread producer_thread([&job_queue]() {
+  // IMPORTANT: BOOST_CHECK_* macros are not thread-safe,
+  //            so we use glog's CHECK macros inside threads.
+
+  Barrier stopBarrier;
+  std::thread producer_thread([&job_queue, &stopBarrier]() {
     CHECK(job_queue.Push(0));
+    stopBarrier.Wait();
     CHECK(!job_queue.Push(0));
   });
 
-  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+  stopBarrier.Wait();
   BOOST_CHECK_EQUAL(job_queue.Size(), 1);
 
   job_queue.Stop();
@@ -841,16 +857,21 @@ BOOST_AUTO_TEST_CASE(TestJobQueueStopProducer) {
 BOOST_AUTO_TEST_CASE(TestJobQueueStopConsumer) {
   JobQueue<int> job_queue(1);
 
+  // IMPORTANT: BOOST_CHECK_* macros are not thread-safe,
+  //            so we use glog's CHECK macros inside threads.
+
   BOOST_CHECK(job_queue.Push(0));
 
-  std::thread consumer_thread([&job_queue]() {
+  Barrier popBarrier;
+  std::thread consumer_thread([&job_queue, &popBarrier]() {
     const auto job = job_queue.Pop();
     CHECK(job.IsValid());
     CHECK_EQ(job.Data(), 0);
+    popBarrier.Wait();
     CHECK(!job_queue.Pop().IsValid());
   });
 
-  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+  popBarrier.Wait();
   BOOST_CHECK_EQUAL(job_queue.Size(), 0);
 
   job_queue.Stop();
